@@ -41,6 +41,7 @@
 
 #include "../common/list.h"
 #include "../common/elements.h"
+#include "../common/text_priv.h"
 #include "../_error.h"
 
 /***************************************************************************/
@@ -57,7 +58,6 @@ const char *PART_TO_CHAR[] = { NULL, "h", "o", "a" };
  * */
 struct cpe_item {		// the node <cpe-item>
 
-	struct xml_metadata xml;
 	struct cpe_name *name;	// CPE name as CPE URI
 	struct oscap_list *titles;	// titles of cpe-item (can be in various languages)
 
@@ -75,14 +75,13 @@ OSCAP_ACCESSOR_STRING(cpe_item, deprecation_date)
 OSCAP_GETTER(struct cpe_item_metadata *, cpe_item, metadata)
 OSCAP_IGETINS_GEN(cpe_reference, cpe_item, references, reference)
 OSCAP_IGETINS_GEN(cpe_check, cpe_item, checks, check)
-OSCAP_IGETINS(oscap_title, cpe_item, titles, title)
-OSCAP_IGETINS(oscap_title, cpe_item, notes, note)
+OSCAP_IGETINS(oscap_text, cpe_item, titles, title)
+OSCAP_IGETINS(oscap_text, cpe_item, notes, note)
 OSCAP_ITERATOR_REMOVE_F(cpe_reference) OSCAP_ITERATOR_REMOVE_F(cpe_check)
 
 /* <cpe-item><item-metadata>
  * */
 struct cpe_item_metadata {
-	struct xml_metadata xml;
 	char *modification_date;
 	char *status;
 	char *nvd_id;
@@ -96,7 +95,6 @@ OSCAP_ACCESSOR_STRING(cpe_item_metadata, modification_date)
 /* <cpe-item><check>
  * */
 struct cpe_check {
-	struct xml_metadata xml;
 	char *system;		// system check URI
 	char *href;		// external file reference (NULL if not present)
 	char *identifier;	// test identifier
@@ -108,7 +106,6 @@ OSCAP_ACCESSOR_STRING(cpe_check, system)
 /* <cpe-item><references><reference>
  * */
 struct cpe_reference {
-	struct xml_metadata xml;
 	char *href;		// reference URL
 	char *content;		// reference description
 };
@@ -118,8 +115,6 @@ OSCAP_ACCESSOR_STRING(cpe_reference, href)
 /* <generator>
  * */
 struct cpe_generator {
-
-	struct xml_metadata xml;
 	char *product_name;	// generator software name
 	char *product_version;	// generator software version
 	char *schema_version;	// generator schema version
@@ -133,14 +128,10 @@ OSCAP_ACCESSOR_STRING(cpe_generator, product_name)
 /* <cpe-list>
  * */
 struct cpe_dict_model {		// the main node
-
-	struct oscap_list *xmlns;
-	struct xml_metadata xml;
 	struct oscap_list *items;	// dictionary items
 	struct oscap_list *vendors;
 	struct cpe_generator *generator;
 };
-OSCAP_IGETINS(xml_metadata, cpe_dict_model, xmlns, xml)
 OSCAP_GETTER(struct cpe_generator *, cpe_dict_model, generator)
 OSCAP_IGETTER_GEN(cpe_item, cpe_dict_model, items)
 OSCAP_ITERATOR_REMOVE_F(cpe_item)
@@ -152,20 +143,18 @@ OSCAP_IGETINS_GEN(cpe_vendor, cpe_dict_model, vendors, vendor) OSCAP_ITERATOR_RE
 /* vendor
  * */
 struct cpe_vendor {
-	struct xml_metadata xml;
 	char *value;
 	struct oscap_list *titles;
 	struct oscap_list *products;
 };
 OSCAP_ACCESSOR_STRING(cpe_vendor, value)
-    OSCAP_IGETINS(oscap_title, cpe_vendor, titles, title)
+    OSCAP_IGETINS(oscap_text, cpe_vendor, titles, title)
     OSCAP_IGETINS_GEN(cpe_product, cpe_vendor, products, product)
     OSCAP_ITERATOR_REMOVE_F(cpe_product)
 
 /* vendor -> product 
  * */
 struct cpe_product {
-	struct xml_metadata xml;
 	char *value;
 	cpe_part_t part;
 	struct oscap_list *versions;
@@ -178,7 +167,6 @@ OSCAP_ACCESSOR_STRING(cpe_product, value)
 /* vendor -> product -> version 
  * */
 struct cpe_version {
-	struct xml_metadata xml;
 	char *value;
 	struct oscap_list *updates;
 };
@@ -189,7 +177,6 @@ OSCAP_ACCESSOR_STRING(cpe_version, value)
 /* vendor -> product -> version -> update 
  * */
 struct cpe_update {
-	struct xml_metadata xml;
 	char *value;
 	struct oscap_list *editions;
 };
@@ -200,7 +187,6 @@ OSCAP_ACCESSOR_STRING(cpe_update, value)
 /* vendor -> product -> version -> update -> edition 
  * */
 struct cpe_edition {
-	struct xml_metadata xml;
 	char *value;
 	struct oscap_list *languages;
 };
@@ -211,7 +197,6 @@ OSCAP_ACCESSOR_STRING(cpe_edition, value)
 /* vendor -> product -> version -> update -> edition -> language
  * */
 struct cpe_language {
-	struct xml_metadata xml;
 	char *value;
 };
 OSCAP_ACCESSOR_STRING(cpe_language, value)
@@ -259,6 +244,9 @@ OSCAP_ACCESSOR_STRING(cpe_language, value)
 #define NS_META_STR         BAD_CAST "meta"
 #define ATTR_XML_LANG_STR   BAD_CAST "xml:lang"
 #define VAL_TRUE_STR        BAD_CAST "true"
+/* Namespaces */
+#define CPEDICT_NS BAD_CAST "http://cpe.mitre.org/dictionary/2.0"
+#define CPEMETA_NS BAD_CAST "http://scap.nist.gov/schema/cpe-dictionary-metadata/0.2"
 /* End of XML string variables definitions
  * */
 /***************************************************************************/
@@ -278,7 +266,6 @@ static void cpe_version_export(const struct cpe_version *version, xmlTextWriterP
 static void cpe_update_export(const struct cpe_update *update, xmlTextWriterPtr writer);
 static void cpe_edition_export(const struct cpe_edition *edition, xmlTextWriterPtr writer);
 static void cpe_language_export(const struct cpe_language *language, xmlTextWriterPtr writer);
-static void cpe_note_export(const struct oscap_title *title, xmlTextWriterPtr writer);
 static void cpe_check_export(const struct cpe_check *check, xmlTextWriterPtr writer);
 static void cpe_reference_export(const struct cpe_reference *ref, xmlTextWriterPtr writer);
 
@@ -391,11 +378,6 @@ struct cpe_dict_model *cpe_dict_model_new()
 
 	dict->vendors = oscap_list_new();
 	dict->items = oscap_list_new();
-	dict->xmlns = oscap_list_new();
-
-	dict->xml.lang = NULL;
-	dict->xml.nspace = NULL;
-	dict->xml.URI = NULL;
 
 	return dict;
 }
@@ -415,10 +397,6 @@ struct cpe_item_metadata *cpe_item_metadata_new()
 	item->nvd_id = NULL;
 	item->deprecated_by_nvd_id = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -437,10 +415,6 @@ struct cpe_item *cpe_item_new()
 	item->checks = oscap_list_new();
 	item->titles = oscap_list_new();
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -458,10 +432,6 @@ struct cpe_check *cpe_check_new()
 	item->href = NULL;
 	item->identifier = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -478,10 +448,6 @@ struct cpe_reference *cpe_reference_new()
 	item->href = NULL;
 	item->content = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -500,10 +466,6 @@ struct cpe_generator *cpe_generator_new()
 	item->schema_version = NULL;
 	item->timestamp = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -521,10 +483,6 @@ struct cpe_vendor *cpe_vendor_new()
 	item->titles = oscap_list_new();
 	item->products = oscap_list_new();
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -541,10 +499,6 @@ struct cpe_product *cpe_product_new()
 	item->versions = oscap_list_new();
 	item->value = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -561,10 +515,6 @@ struct cpe_version *cpe_version_new()
 	item->updates = oscap_list_new();
 	item->value = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -581,10 +531,6 @@ struct cpe_update *cpe_update_new()
 	item->editions = oscap_list_new();
 	item->value = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -601,10 +547,6 @@ struct cpe_edition *cpe_edition_new()
 	item->languages = oscap_list_new();
 	item->value = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -620,10 +562,6 @@ struct cpe_language *cpe_language_new()
 
 	item->value = NULL;
 
-	item->xml.lang = NULL;
-	item->xml.nspace = NULL;
-	item->xml.URI = NULL;
-
 	return item;
 }
 
@@ -665,7 +603,6 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 
 	struct cpe_dict_model *ret = NULL;
 	struct cpe_item *item = NULL;
-	struct xml_metadata *xml = NULL;
 	struct cpe_vendor *vendor = NULL;
 	int next_ret = 1;
 
@@ -684,16 +621,6 @@ struct cpe_dict_model *cpe_dict_model_parse(xmlTextReaderPtr reader)
 	if (ret == NULL)
 		return NULL;
 
-	/* Reading XML namespaces */
-	if (xmlTextReaderHasAttributes(reader) && xmlTextReaderMoveToFirstAttribute(reader) == 1) {
-		do {
-			xml = oscap_alloc(sizeof(struct xml_metadata));
-			xml->lang = NULL;
-			xml->nspace = oscap_strdup((char *)xmlTextReaderConstName(reader));
-			xml->URI = oscap_strdup((char *)xmlTextReaderConstValue(reader));
-			oscap_list_add(ret->xmlns, xml);
-		} while (xmlTextReaderMoveToNextAttribute(reader) == 1);
-	}
 	// go through elements and switch through actions till end of file..
 	next_ret = xmlTextReaderNextElement(reader);
 	while (next_ret != 0) {
@@ -748,9 +675,6 @@ struct cpe_generator *cpe_generator_parse(xmlTextReaderPtr reader)
 		if (ret == NULL)
 			return NULL;
 
-		ret->xml.lang = (char *)xmlTextReaderConstXmlLang(reader);
-		ret->xml.nspace = (char *)xmlTextReaderPrefix(reader);
-
 		// skip nodes until new element
 		xmlTextReaderNextElement(reader);
 
@@ -794,7 +718,6 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 {
 
 	struct cpe_item *ret = NULL;
-	struct oscap_title *title = NULL;
 	struct cpe_check *check = NULL;
 	struct cpe_reference *ref = NULL;
 	char *data;
@@ -809,9 +732,6 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 		if (ret == NULL)
 			return NULL;
 
-		ret->xml.lang = (char *)xmlTextReaderConstXmlLang(reader);
-		ret->xml.nspace = (char *)xmlTextReaderPrefix(reader);
-
 		// Get a name attribute of cpe-item
 		data = (char *)xmlTextReaderGetAttribute(reader, ATTR_NAME_STR);
 		if (data != NULL)
@@ -853,15 +773,11 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 			}
 
 			if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_TITLE_STR) == 0) {
-				title = oscap_title_parse(reader, (char *)TAG_TITLE_STR);
-				if (title)
-					oscap_list_add(ret->titles, title);
+				oscap_list_add(ret->titles, oscap_text_new_parse(OSCAP_TEXT_TRAITS_PLAIN, reader));
 			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_NOTE_STR) == 0) {
 				// it's OK to use title varible here, because note is the same structure,
 				// not the reason to make a new same one
-				title = oscap_title_parse(reader, (char *)TAG_NOTE_STR);
-				if (title)
-					oscap_list_add(ret->notes, title);
+				oscap_list_add(ret->notes, oscap_text_new_parse(OSCAP_TEXT_TRAITS_PLAIN, reader));
 			} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CHECK_STR) == 0) {
 				check = cpe_check_parse(reader);
 				if (check)
@@ -880,8 +796,6 @@ struct cpe_item *cpe_item_parse(xmlTextReaderPtr reader)
 					return NULL;
 				}
 				ret->metadata->modification_date = data;
-				ret->metadata->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-				ret->metadata->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 
 				data = (char *)xmlTextReaderGetAttribute(reader, ATTR_STATUS_STR);
 				if (data)
@@ -926,8 +840,6 @@ static struct cpe_check *cpe_check_parse(xmlTextReaderPtr reader)
 		return NULL;
 	memset(ret, 0, sizeof(struct cpe_check));
 
-	ret->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-	ret->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 	ret->system = (char *)xmlTextReaderGetAttribute(reader, ATTR_SYSTEM_STR);
 	ret->href = (char *)xmlTextReaderGetAttribute(reader, ATTR_HREF_STR);
 	ret->identifier = oscap_trim((char *)xmlTextReaderReadString(reader));
@@ -949,8 +861,6 @@ static struct cpe_reference *cpe_reference_parse(xmlTextReaderPtr reader)
 		return NULL;
 	memset(ret, 0, sizeof(struct cpe_reference));
 
-	ret->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-	ret->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 	ret->href = (char *)xmlTextReaderGetAttribute(reader, ATTR_HREF_STR);
 	ret->content = oscap_trim((char *)xmlTextReaderReadString(reader));
 
@@ -961,7 +871,6 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 {
 
 	struct cpe_vendor *ret = NULL;
-	struct oscap_title *title = NULL;
 	struct cpe_product *product = NULL;
 	struct cpe_version *version = NULL;
 	struct cpe_update *update = NULL;
@@ -978,7 +887,6 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 	if (ret == NULL)
 		return NULL;
 
-	ret->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 	ret->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 	// jump to next element (which should be product)
 	xmlTextReaderNextElement(reader);
@@ -991,14 +899,10 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 		}
 
 		if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_TITLE_STR) == 0) {
-			title = oscap_title_parse(reader, (char *)TAG_TITLE_STR);
-			if (title)
-				oscap_list_add(ret->titles, title);
+			oscap_list_add(ret->titles, oscap_text_new_parse(OSCAP_TEXT_TRAITS_PLAIN, reader));
 		} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_PRODUCT_STR) == 0) {
 			// initialization
 			product = cpe_product_new();
-			product->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-			product->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 			product->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 
 			data = (char *)xmlTextReaderGetAttribute(reader, ATTR_PART_STR);
@@ -1024,29 +928,21 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
 		} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_VERSION_STR) == 0) {
 			// initialization
 			version = cpe_version_new();
-			version->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-			version->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 			version->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 			oscap_list_add(product->versions, version);
 		} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_UPDATE_STR) == 0) {
 			// initialization
 			update = cpe_update_new();
-			update->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-			update->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 			update->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 			oscap_list_add(version->updates, update);
 		} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_EDITION_STR) == 0) {
 			// initialization
 			edition = cpe_edition_new();
-			edition->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-			edition->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 			edition->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 			oscap_list_add(update->editions, edition);
 		} else if (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_LANGUAGE_STR) == 0) {
 			// initialization
 			language = cpe_language_new();
-			language->xml.lang = oscap_strdup((char *)xmlTextReaderConstXmlLang(reader));
-			language->xml.nspace = (char *)xmlTextReaderPrefix(reader);
 			language->value = (char *)xmlTextReaderGetAttribute(reader, ATTR_VALUE_STR);
 			oscap_list_add(edition->languages, language);
 		}
@@ -1098,20 +994,19 @@ void cpe_dict_export(const struct cpe_dict_model *dict, xmlTextWriterPtr writer)
 	__attribute__nonnull__(dict);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST dict->xml.nspace, TAG_CPE_LIST_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_LIST_STR, CPEDICT_NS);
+	xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:meta", CPEMETA_NS);
+	xmlTextWriterWriteAttribute(writer, BAD_CAST "xmlns:xsi", BAD_CAST "http://www.w3.org/2001/XMLSchema-instance");
+	xmlTextWriterWriteAttribute(writer, BAD_CAST "xsi:schemaLocation", BAD_CAST
+			"http://scap.nist.gov/schema/cpe-dictionary-metadata/0.2 http://nvd.nist.gov/schema/cpe-dictionary-metadata_0.2.xsd "
+			"http://cpe.mitre.org/dictionary/2.0 http://cpe.mitre.org/files/cpe-dictionary_2.1.xsd");
 
-	OSCAP_FOREACH(xml_metadata, xml, cpe_dict_model_get_xmlns(dict),
-		      if (xml->URI != NULL) xmlTextWriterWriteAttribute(writer, BAD_CAST xml->nspace,
-									BAD_CAST xml->URI);)
-
-		if (dict->generator != NULL) {
-			cpe_generator_export(dict->generator, writer);
-		}
+	if (dict->generator) cpe_generator_export(dict->generator, writer);
 	OSCAP_FOREACH(cpe_item, item, cpe_dict_model_get_items(dict),
 		      // dump its contents to XML tree
 		      cpe_item_export(item, writer);)
 	    // TODO: NEED TO HAVE COMPONENT-TREE STRUCTURE TO GET XML-NAMESPACE 
-	    xmlTextWriterStartElementNS(writer, NS_META_STR, TAG_COMPONENT_TREE_STR, NULL);
+	    xmlTextWriterStartElementNS(writer, NULL, TAG_COMPONENT_TREE_STR, CPEMETA_NS);
 	OSCAP_FOREACH(cpe_vendor, vendor, cpe_dict_model_get_vendors(dict), cpe_vendor_export(vendor, writer);)
 	    xmlTextWriterEndElement(writer);	//</component-tree>
 
@@ -1126,24 +1021,24 @@ void cpe_generator_export(const struct cpe_generator *generator, xmlTextWriterPt
 	__attribute__nonnull__(generator);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST generator->xml.nspace, TAG_GENERATOR_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_GENERATOR_STR, NULL);
 	if (generator->product_name != NULL) {
-		xmlTextWriterStartElementNS(writer, BAD_CAST generator->xml.nspace, TAG_PRODUCT_NAME_STR, NULL);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_PRODUCT_NAME_STR, NULL);
 		xmlTextWriterWriteString(writer, BAD_CAST generator->product_name);
 		xmlTextWriterEndElement(writer);
 	}
 	if (generator->product_version != NULL) {
-		xmlTextWriterStartElementNS(writer, BAD_CAST generator->xml.nspace, TAG_PRODUCT_VERSION_STR, NULL);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_PRODUCT_VERSION_STR, NULL);
 		xmlTextWriterWriteString(writer, BAD_CAST generator->product_version);
 		xmlTextWriterEndElement(writer);
 	}
 	if (generator->schema_version != NULL) {
-		xmlTextWriterStartElementNS(writer, BAD_CAST generator->xml.nspace, TAG_SCHEMA_VERSION_STR, NULL);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_SCHEMA_VERSION_STR, NULL);
 		xmlTextWriterWriteString(writer, BAD_CAST generator->schema_version);
 		xmlTextWriterEndElement(writer);
 	}
 	if (generator->timestamp != NULL) {
-		xmlTextWriterStartElementNS(writer, BAD_CAST generator->xml.nspace, TAG_TIMESTAMP_STR, NULL);
+		xmlTextWriterStartElementNS(writer, NULL, TAG_TIMESTAMP_STR, NULL);
 		xmlTextWriterWriteString(writer, BAD_CAST generator->timestamp);
 		xmlTextWriterEndElement(writer);
 	}
@@ -1162,7 +1057,7 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer)
 	__attribute__nonnull__(item);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST item->xml.nspace, TAG_CPE_ITEM_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_CPE_ITEM_STR, CPEDICT_NS);
 	if (item->name != NULL) {
 		temp = cpe_name_get_uri(item->name);
 		xmlTextWriterWriteAttribute(writer, ATTR_NAME_STR, BAD_CAST temp);
@@ -1176,11 +1071,10 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer)
 		oscap_free(temp);
 	}
 
-	OSCAP_FOREACH(oscap_title, title, cpe_item_get_titles(item), oscap_title_export(title, writer);)
+	oscap_textlist_export(cpe_item_get_titles(item), writer, "title");
 
 	    if (item->metadata != NULL) {
-		xmlTextWriterStartElementNS(writer, BAD_CAST item->metadata->xml.nspace, TAG_ITEM_METADATA_STR,
-					    NULL);
+		xmlTextWriterStartElementNS(writer, NS_META_STR, TAG_ITEM_METADATA_STR, NULL);
 		if (item->metadata->modification_date != NULL)
 			xmlTextWriterWriteAttribute(writer, ATTR_MODIFICATION_DATE_STR,
 						    BAD_CAST item->metadata->modification_date);
@@ -1202,13 +1096,7 @@ void cpe_item_export(const struct cpe_item *item, xmlTextWriterPtr writer)
 	}
 	oscap_iterator_free(it);
 
-	it = oscap_iterator_new(item->notes);
-	if (oscap_iterator_has_more(it)) {
-		xmlTextWriterStartElementNS(writer, NULL, TAG_NOTES_STR, NULL);
-		OSCAP_FOREACH(oscap_title, note, cpe_item_get_notes(item), cpe_note_export(note, writer);)
-		    xmlTextWriterEndElement(writer);
-	}
-	oscap_iterator_free(it);
+	oscap_textlist_export(cpe_item_get_notes(item), writer, "title");
 
 	OSCAP_FOREACH(cpe_check, check, cpe_item_get_checks(item), cpe_check_export(check, writer);)
 
@@ -1224,11 +1112,11 @@ void cpe_vendor_export(const struct cpe_vendor *vendor, xmlTextWriterPtr writer)
 	__attribute__nonnull__(vendor);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST vendor->xml.nspace, TAG_VENDOR_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_VENDOR_STR, NULL);
 	if (vendor->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST vendor->value);
 
-	OSCAP_FOREACH(oscap_title, title, cpe_vendor_get_titles(vendor), oscap_title_export(title, writer);)
+	oscap_textlist_export(cpe_vendor_get_titles(vendor), writer, "title");
 
 	    OSCAP_FOREACH(cpe_product, product, cpe_vendor_get_products(vendor), cpe_product_export(product, writer);)
 
@@ -1243,7 +1131,7 @@ static void cpe_product_export(const struct cpe_product *product, xmlTextWriterP
 	__attribute__nonnull__(product);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST product->xml.nspace, TAG_PRODUCT_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_PRODUCT_STR, NULL);
 	if (product->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST product->value);
 	if (product->part != CPE_PART_NONE)
@@ -1260,7 +1148,7 @@ static void cpe_version_export(const struct cpe_version *version, xmlTextWriterP
 	__attribute__nonnull__(version);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST version->xml.nspace, TAG_VERSION_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_VERSION_STR, NULL);
 	if (version->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST version->value);
 
@@ -1278,7 +1166,7 @@ static void cpe_update_export(const struct cpe_update *update, xmlTextWriterPtr
 	__attribute__nonnull__(update);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST update->xml.nspace, TAG_UPDATE_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_UPDATE_STR, NULL);
 	if (update->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST update->value);
 
@@ -1293,7 +1181,7 @@ static void cpe_edition_export(const struct cpe_edition *edition, xmlTextWriterP
 	__attribute__nonnull__(edition);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST edition->xml.nspace, TAG_EDITION_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_EDITION_STR, NULL);
 	if (edition->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST edition->value);
 
@@ -1309,36 +1197,20 @@ static void cpe_language_export(const struct cpe_language *language, xmlTextWrit
 	__attribute__nonnull__(language);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST language->xml.nspace, TAG_LANGUAGE_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_LANGUAGE_STR, NULL);
 	if (language->value != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_VALUE_STR, BAD_CAST language->value);
-	if (language->xml.lang != NULL)
-		xmlTextWriterWriteAttribute(writer, ATTR_XML_LANG_STR, BAD_CAST language->xml.lang);
 
 	xmlTextWriterEndElement(writer);
 }
 
-static void cpe_note_export(const struct oscap_title *title, xmlTextWriterPtr writer)
-{
-
-	__attribute__nonnull__(title);
-	__attribute__nonnull__(writer);
-
-	xmlTextWriterStartElementNS(writer, BAD_CAST title->xml.nspace, TAG_NOTE_STR, NULL);
-	if (title->xml.lang != NULL)
-		xmlTextWriterWriteAttribute(writer, ATTR_XML_LANG_STR, BAD_CAST title->xml.lang);
-	if (title->content != NULL)
-		xmlTextWriterWriteString(writer, BAD_CAST title->content);
-	xmlTextWriterEndElement(writer);
-}
-
 static void cpe_check_export(const struct cpe_check *check, xmlTextWriterPtr writer)
 {
 
 	__attribute__nonnull__(check);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST check->xml.nspace, TAG_CHECK_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_CHECK_STR, NULL);
 	if (check->system != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_SYSTEM_STR, BAD_CAST check->system);
 	if (check->href != NULL)
@@ -1354,13 +1226,11 @@ static void cpe_reference_export(const struct cpe_reference *ref, xmlTextWriterP
 	__attribute__nonnull__(ref);
 	__attribute__nonnull__(writer);
 
-	xmlTextWriterStartElementNS(writer, BAD_CAST ref->xml.nspace, TAG_REFERENCE_STR, NULL);
+	xmlTextWriterStartElementNS(writer, NULL, TAG_REFERENCE_STR, NULL);
 	if (ref->href != NULL)
 		xmlTextWriterWriteAttribute(writer, ATTR_HREF_STR, BAD_CAST ref->href);
 	if (ref->content != NULL)
 		xmlTextWriterWriteString(writer, BAD_CAST ref->content);
-	if (ref->xml.lang != NULL)
-		xmlTextWriterWriteAttribute(writer, ATTR_XML_LANG_STR, BAD_CAST ref->xml.lang);
 	xmlTextWriterEndElement(writer);
 }
 
@@ -1379,9 +1249,7 @@ void cpe_dict_model_free(struct cpe_dict_model *dict)
 
 	oscap_list_free(dict->items, (oscap_destruct_func) cpe_item_free);
 	oscap_list_free(dict->vendors, (oscap_destruct_func) cpe_vendor_free);
-	oscap_list_free(dict->xmlns, (oscap_destruct_func) xml_metadata_free);
 	cpe_generator_free(dict->generator);
-	xml_metadata_free(&dict->xml);
 	oscap_free(dict);
 }
 
@@ -1395,10 +1263,9 @@ void cpe_item_free(struct cpe_item *item)
 	oscap_free(item->deprecation_date);
 	oscap_list_free(item->references, (oscap_destruct_func) cpe_reference_free);
 	oscap_list_free(item->checks, (oscap_destruct_func) cpe_check_free);
-	oscap_list_free(item->notes, (oscap_destruct_func) oscap_title_free);
-	oscap_list_free(item->titles, (oscap_destruct_func) oscap_title_free);
+	oscap_list_free(item->notes, (oscap_destruct_func) oscap_text_free);
+	oscap_list_free(item->titles, (oscap_destruct_func) oscap_text_free);
 	cpe_itemmetadata_free(item->metadata);
-	xml_metadata_free(&item->xml);
 	oscap_free(item);
 }
 
@@ -1412,7 +1279,6 @@ void cpe_generator_free(struct cpe_generator *generator)
 	oscap_free(generator->product_version);
 	oscap_free(generator->schema_version);
 	oscap_free(generator->timestamp);
-	xml_metadata_free(&generator->xml);
 	oscap_free(generator);
 }
 
@@ -1425,7 +1291,6 @@ void cpe_check_free(struct cpe_check *check)
 	oscap_free(check->identifier);
 	oscap_free(check->system);
 	oscap_free(check->href);
-	xml_metadata_free(&check->xml);
 	oscap_free(check);
 }
 
@@ -1437,7 +1302,6 @@ void cpe_reference_free(struct cpe_reference *ref)
 
 	oscap_free(ref->href);
 	oscap_free(ref->content);
-	xml_metadata_free(&ref->xml);
 	oscap_free(ref);
 }
 
@@ -1448,9 +1312,8 @@ void cpe_vendor_free(struct cpe_vendor *vendor)
 		return;
 
 	oscap_free(vendor->value);
-	oscap_list_free(vendor->titles, (oscap_destruct_func) oscap_title_free);
+	oscap_list_free(vendor->titles, (oscap_destruct_func) oscap_text_free);
 	oscap_list_free(vendor->products, (oscap_destruct_func) cpe_product_free);
-	xml_metadata_free(&vendor->xml);
 	oscap_free(vendor);
 }
 
@@ -1462,7 +1325,6 @@ void cpe_product_free(struct cpe_product *product)
 
 	oscap_free(product->value);
 	oscap_list_free(product->versions, (oscap_destruct_func) cpe_version_free);
-	xml_metadata_free(&product->xml);
 	oscap_free(product);
 }
 
@@ -1474,7 +1336,6 @@ void cpe_version_free(struct cpe_version *version)
 
 	oscap_free(version->value);
 	oscap_list_free(version->updates, (oscap_destruct_func) cpe_update_free);
-	xml_metadata_free(&version->xml);
 	oscap_free(version);
 }
 
@@ -1486,7 +1347,6 @@ void cpe_update_free(struct cpe_update *update)
 
 	oscap_free(update->value);
 	oscap_list_free(update->editions, (oscap_destruct_func) cpe_edition_free);
-	xml_metadata_free(&update->xml);
 	oscap_free(update);
 }
 
@@ -1498,7 +1358,6 @@ void cpe_edition_free(struct cpe_edition *edition)
 
 	oscap_free(edition->value);
 	oscap_list_free(edition->languages, (oscap_destruct_func) cpe_language_free);
-	xml_metadata_free(&edition->xml);
 	oscap_free(edition);
 }
 
@@ -1509,7 +1368,6 @@ void cpe_language_free(struct cpe_language *language)
 		return;
 
 	oscap_free(language->value);
-	xml_metadata_free(&language->xml);
 	oscap_free(language);
 }
 
@@ -1523,7 +1381,6 @@ void cpe_itemmetadata_free(struct cpe_item_metadata *meta)
 	oscap_free(meta->status);
 	oscap_free(meta->nvd_id);
 	oscap_free(meta->deprecated_by_nvd_id);
-	xml_metadata_free(&meta->xml);
 	oscap_free(meta);
 }
 
