@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include "config.h"
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
@@ -260,7 +261,7 @@ void oval_component_set_literal_value(struct oval_component *component, struct o
 			((struct oval_component_LITERAL *)component)->value = value;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_component_set_type(struct oval_component *component, oval_component_type_t type)
@@ -269,7 +270,7 @@ void oval_component_set_type(struct oval_component *component, oval_component_ty
 	if (component && !oval_component_is_locked(component)) {
 		component->type = type;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 struct oval_object *oval_component_get_object(struct oval_component *component)
@@ -289,7 +290,7 @@ void oval_component_set_object(struct oval_component *component, struct oval_obj
 			((struct oval_component_OBJECTREF *)component)->object = object;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 char *oval_component_get_object_field(struct oval_component *component) {
@@ -308,7 +309,7 @@ void oval_component_set_object_field(struct oval_component *component, char *fie
 			((struct oval_component_OBJECTREF *)component)->object_field = oscap_strdup(field);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 struct oval_variable *oval_component_get_variable(struct oval_component *component) {
@@ -359,7 +360,7 @@ void oval_component_set_arithmetic_operation(struct oval_component *component, o
 			arithmetic->operation = operation;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 char *oval_component_get_prefix(struct oval_component *component)
@@ -384,7 +385,7 @@ void oval_component_set_prefix(struct oval_component *component, char *character
 			begin->character = oscap_strdup(character);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 char *oval_component_get_suffix(struct oval_component *component)
@@ -458,7 +459,7 @@ void oval_component_set_substring_start(struct oval_component *component, int st
 			substring->start = start;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 int oval_component_get_substring_length(struct oval_component *component)
@@ -483,7 +484,7 @@ void oval_component_set_substring_length(struct oval_component *component, int l
 			substring->length = length;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 oval_datetime_format_t oval_component_get_timedif_format_1(struct oval_component
@@ -509,7 +510,7 @@ void oval_component_set_timedif_format_1(struct oval_component *component, oval_
 			timedif->format_1 = format;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 oval_datetime_format_t oval_component_get_timedif_format_2(struct oval_component
@@ -535,7 +536,7 @@ void oval_component_set_timedif_format_2(struct oval_component *component, oval_
 			timedif->format_2 = format;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 char *oval_component_get_regex_pattern(struct oval_component *component) {
@@ -558,7 +559,7 @@ void oval_component_set_regex_pattern(struct oval_component *component, char *pa
 			regex->pattern = oscap_strdup(pattern);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 struct oval_component *oval_component_new(struct oval_definition_model *model, oval_component_type_t type)
@@ -685,7 +686,7 @@ struct oval_component *oval_component_new(struct oval_definition_model *model, o
 
 bool oval_component_is_valid(struct oval_component * component)
 {
-        oscap_dprintf("WARNING: NOOP.\n");
+        oscap_dlprintf(DBG_W, "NOOP.\n");
 	return true;		//TODO
 }
 
@@ -861,7 +862,7 @@ void oval_component_add_function_component(struct oval_component *component, str
 				oval_collection_add(function->function_components, func_component);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_component_set_variable(struct oval_component *component, struct oval_variable *variable)
@@ -873,7 +874,7 @@ void oval_component_set_variable(struct oval_component *component, struct oval_v
 			varref->variable = variable;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 static void oval_value_consume(struct oval_value *value, void *component)
@@ -1083,14 +1084,14 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 		component = oval_component_new(model, OVAL_FUNCTION_REGEX_CAPTURE);
 		return_code = _oval_component_parse_REGEX_CAPTURE_tag(reader, context, component);
 	} else {
-		oscap_dprintf("NOTICE::oval_component_parse_tag::<%s> not handled (line = %d)", tagname,
+		oscap_dlprintf(DBG_I, "Tag <%s> not handled, line: %d.\n", tagname,
                               xmlTextReaderGetParserLineNumber(reader));
 		return_code = oval_parser_skip_tag(reader, context);
 	}
 	if (component != NULL)
 		(*consumer) (component, user);
 	if (return_code != 1) {
-		oscap_dprintf("NOTICE: oval_component_parse_tag::parse of <%s> terminated on error at line %d", tagname,
+		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n", tagname,
                             xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
@@ -1218,7 +1219,7 @@ xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, x
 	}
 
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
-	xmlNode *component_node = xmlNewChild(parent, ns_definitions,
+	xmlNode *component_node = xmlNewTextChild(parent, ns_definitions,
 					      BAD_CAST local_name,
 					      BAD_CAST content);
 
@@ -1337,44 +1338,41 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(oval_ar
 
 	struct oval_component_OBJECTREF *objref = (struct oval_component_OBJECTREF *)component;
 	struct oval_object *object = objref->object;
-	char *obj_id;
-	struct oval_syschar_model *sysmod;
 	struct oval_syschar *syschar;
 
 	if (!object)
 		return flag;
 
 	if (argu->mode == OVAL_MODE_QUERY) {
-		if (oval_probe_session_query_object(argu->u.sess, object) != 0)
+		if (oval_probe_query_object(argu->u.sess, object, 0, &syschar) != 0)
 			return flag;
-		sysmod = oval_probe_session_getmodel(argu->u.sess);
 	} else {
-		sysmod = argu->u.sysmod;
-	}
+		char *obj_id;
 
-	obj_id = oval_object_get_id(object);
-	syschar = oval_syschar_model_get_syschar(sysmod, obj_id);
+		obj_id = oval_object_get_id(object);
+		syschar = oval_syschar_model_get_syschar(argu->u.sysmod, obj_id);
+	}
 
 	if (syschar) {
 		flag = oval_syschar_get_flag(syschar);
 		char *field_name = objref->object_field;
-		struct oval_sysdata_iterator *sysdatas = oval_syschar_get_sysdata(syschar);
-		while (oval_sysdata_iterator_has_more(sysdatas)) {
-			struct oval_sysdata *sysdata = oval_sysdata_iterator_next(sysdatas);
-			struct oval_sysitem_iterator *items = oval_sysdata_get_items(sysdata);
-			while (oval_sysitem_iterator_has_more(items)) {
-				struct oval_sysitem *item = oval_sysitem_iterator_next(items);
-				char *item_name = oval_sysitem_get_name(item);
+		struct oval_sysitem_iterator *sysitems = oval_syschar_get_sysitem(syschar);
+		while (oval_sysitem_iterator_has_more(sysitems)) {
+			struct oval_sysitem *sysitem = oval_sysitem_iterator_next(sysitems);
+			struct oval_sysent_iterator *items = oval_sysitem_get_items(sysitem);
+			while (oval_sysent_iterator_has_more(items)) {
+				struct oval_sysent *item = oval_sysent_iterator_next(items);
+				char *item_name = oval_sysent_get_name(item);
 				if (strcmp(field_name, item_name) == 0) {
-					char *text = oval_sysitem_get_value(item);
-					oval_datatype_t datatype = oval_sysitem_get_datatype(item);
+					char *text = oval_sysent_get_value(item);
+					oval_datatype_t datatype = oval_sysent_get_datatype(item);
 					struct oval_value *value = oval_value_new(datatype, text);
 					oval_collection_add(value_collection, value);
 				}
 			}
-			oval_sysitem_iterator_free(items);
+			oval_sysent_iterator_free(items);
 		}
-		oval_sysdata_iterator_free(sysdatas);
+		oval_sysitem_iterator_free(sysitems);
 	}
 
 	return flag;
@@ -1397,13 +1395,13 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_VARREF(oval_argu_
 	struct oval_variable *variable = varref->variable;
 
 	if (!variable) {
-		oscap_dprintf("ERROR: No variable bound to VARREF Component.\n");
+		oscap_dlprintf(DBG_E, "No variable bound to VARREF Component.\n");
 		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No variable bound to VARREF componenet");
 		return flag;
 	}
 
 	if (argu->mode == OVAL_MODE_QUERY) {
-		if (oval_probe_session_query_variable(argu->u.sess, variable) != 0)
+		if (oval_probe_query_variable(argu->u.sess, variable) != 0)
 			return flag;
 	} else {
 		if (oval_syschar_model_compute_variable(argu->u.sysmod, variable) != 0)
@@ -1454,7 +1452,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(oval_argu_t
 		}
 		oval_component_iterator_free(subcomps);
 	} else {
-		oscap_dprintf("ERROR: No prefix specified for begin function (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_E, "No prefix specified for begin function.\n");
 		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No prefix specified for component evaluation");
 	}
 	return flag;
@@ -1491,7 +1489,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_END(oval_argu_t *
 		}
 		oval_component_iterator_free(subcomps);
 	} else {
-		oscap_dprintf("ERROR: No suffix specified for end function  (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_E, "No suffix specified for end function.\n");
 		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No suffix specified for component evaluation");
 	}
 	return flag;
@@ -1660,8 +1658,11 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(oval_ar
                         txtlen    = strlen(text);
 
                         if (beg < txtlen) {
-                                sublen = (beg + len) >= txtlen ? 0 : txtlen - (beg + len);
-
+                                /* Check if we are still inside the string
+                                 * beg: position of start of next substring (from 0! last position is txtlen-1)
+                                 * len: length of susbstring
+                                 * txtlen: length of whole string */
+                                sublen = (beg + len) > txtlen ? 0 : txtlen - (beg + len) + 1;
                                 if (sublen < len)
                                         len = sublen;
 
@@ -1679,11 +1680,306 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(oval_ar
 	return flag;
 }
 
+static int _substr2i(char **s, int len)
+{
+	char buf[len + 1];
+
+	memcpy(buf, *s, len);
+	buf[len] = '\0';
+	*s += len;
+
+	return atoi(buf);
+}
+
+static int _substr2mon(char **sp)
+{
+	int m = 0;
+	size_t len;
+	char *s = *sp;
+
+	len = strcspn(s, "0123456789,");
+	if (len == 0)
+		return -1;
+
+	*sp += len;
+
+	if (!strncasecmp(s, "jan", len) || !strncasecmp(s, "january", len)) {
+		m = 1;
+	} else if (!strncasecmp(s, "feb", len) || !strncasecmp(s, "february", len)) {
+		m = 2;
+	} else if (!strncasecmp(s, "mar", len) || !strncasecmp(s, "march", len)) {
+		m = 3;
+	} else if (!strncasecmp(s, "apr", len) || !strncasecmp(s, "april", len)) {
+		m = 4;
+	} else if (!strncasecmp(s, "may", len)) {
+		m = 5;
+	} else if (!strncasecmp(s, "jun", len) || !strncasecmp(s, "june", len)) {
+		m = 6;
+	} else if (!strncasecmp(s, "jul", len) || !strncasecmp(s, "july", len)) {
+		m = 7;
+	} else if (!strncasecmp(s, "aug", len) || !strncasecmp(s, "august", len)) {
+		m = 8;
+	} else if (!strncasecmp(s, "sep", len) || !strncasecmp(s, "september", len)) {
+		m = 9;
+	} else if (!strncasecmp(s, "oct", len) || !strncasecmp(s, "october", len)) {
+		m = 10;
+	} else if (!strncasecmp(s, "nov", len) || !strncasecmp(s, "november", len)) {
+		m = 11;
+	} else if (!strncasecmp(s, "dec", len) || !strncasecmp(s, "december", len)) {
+		m = 12;
+	}
+
+	return m;
+}
+
+static long unsigned int _comp_sec(int year, int month, int day, int hour, int minute, int second)
+{
+	time_t t;
+	struct tm *ts;
+
+	t = time(NULL);
+	ts = localtime(&t);
+
+	ts->tm_year = year - 1900;
+	ts->tm_mon = month - 1;
+	ts->tm_mday = day;
+	ts->tm_hour = hour;
+	ts->tm_min = minute;
+	ts->tm_sec = second;
+	ts->tm_isdst = -1;
+	t = mktime(ts);
+	ts = localtime(&t);
+
+	if (ts->tm_isdst == 1)
+		t -= 3600;
+
+	return (long unsigned int) t;
+}
+
+/*
+year_month_day
+The year_month_day value specifies date-time strings that follow the formats:
+  'yyyymmdd', 'yyyymmddThhmmss', 'yyyy/mm/dd hh:mm:ss', 'yyyy/mm/dd', 'yyyy-mm-dd hh:mm:ss', or 'yyyy-mm-dd'
+*/
+static long unsigned int _parse_fmt_ymd(char *dt)
+{
+	int year, month, day, hour, minute, second;
+
+	year = _substr2i(&dt, 4);
+	if (*dt == '/' || *dt == '-')
+		dt++;
+	month = _substr2i(&dt, 2);
+	if (*dt == '/' || *dt == '-')
+		dt++;
+	day = _substr2i(&dt, 2);
+
+	if (*dt == '\0') {
+		hour = minute = second = 0;
+	} else if (*dt == 'T') {
+		dt++;
+		hour = _substr2i(&dt, 2);
+		minute = _substr2i(&dt, 2);
+		second = _substr2i(&dt, 2);
+	} else {
+		dt++;
+		hour = _substr2i(&dt, 2);
+		dt++;
+		minute = _substr2i(&dt, 2);
+		dt++;
+		second = _substr2i(&dt, 2);
+	}
+
+	return _comp_sec(year, month, day, hour, minute, second);
+}
+
+/*
+month_day_year
+The month_day_year value specifies date-time strings that follow the formats:
+  'mm/dd/yyyy hh:mm:ss', 'mm/dd/yyyy', 'mm-dd-yyyy hh:mm:ss', 'mm-dd-yyyy', 'NameOfMonth, dd yyyy hh:mm:ss' or
+  'NameOfMonth, dd yyyy','AbreviatedNameOfMonth, dd yyyy hh:mm:ss', or 'AbreviatedNameOfMonth, dd yyyy'
+*/
+static long unsigned int _parse_fmt_mdy(char *dt)
+{
+	int year, month, day, hour, minute, second;
+
+	if ((month = _substr2mon(&dt)) == -1) {
+		month = _substr2i(&dt, 2);
+		dt++;
+	} else {
+		dt += 2;
+	}
+
+	day = _substr2i(&dt, 2);
+	dt++;
+	year = _substr2i(&dt, 4);
+
+	if (*dt == '\0') {
+		hour = minute = second = 0;
+	} else {
+		dt++;
+		hour = _substr2i(&dt, 2);
+		dt++;
+		minute = _substr2i(&dt, 2);
+		dt++;
+		second = _substr2i(&dt, 2);
+	}
+
+	return _comp_sec(year, month, day, hour, minute, second);
+}
+
+/*
+day_month_year
+The day_month_year value specifies date-time strings that follow the formats:
+  'dd/mm/yyyy hh:mm:ss', 'dd/mm/yyyy', 'dd-mm-yyyy hh:mm:ss', or 'dd-mm-yyyy'
+*/
+static long unsigned int _parse_fmt_dmy(char *dt)
+{
+	int year, month, day, hour, minute, second;
+
+	day = _substr2i(&dt, 2);
+	dt++;
+	month = _substr2i(&dt, 2);
+	dt++;
+	year = _substr2i(&dt, 4);
+
+	if (*dt == '\0') {
+		hour = minute = second = 0;
+	} else {
+		dt++;
+		hour = _substr2i(&dt, 2);
+		dt++;
+		minute = _substr2i(&dt, 2);
+		dt++;
+		second = _substr2i(&dt, 2);
+	}
+
+	return _comp_sec(year, month, day, hour, minute, second);
+}
+
+static long unsigned int _parse_fmt_win(char *dt)
+{
+	// todo
+	return 0;
+}
+
+static long unsigned int _parse_fmt_sse(char *dt)
+{
+	time_t t;
+	struct tm *ts;
+
+	t = (time_t) atol(dt);
+	ts = localtime(&t);
+	if (ts->tm_isdst == 1)
+		t -= 3600;
+
+	return (long unsigned int) t;
+}
+
+static long unsigned int _parse_fmt(struct oval_value *val, oval_datetime_format_t fmt)
+{
+	long unsigned int v = 0;
+	char *sv;
+
+	sv = oval_value_get_text(val);
+
+	switch (fmt) {
+	case OVAL_DATETIME_YEAR_MONTH_DAY:
+		v = _parse_fmt_ymd(sv);
+		break;
+	case OVAL_DATETIME_MONTH_DAY_YEAR:
+		v = _parse_fmt_mdy(sv);
+		break;
+	case OVAL_DATETIME_DAY_MONTH_YEAR:
+		v = _parse_fmt_dmy(sv);
+		break;
+	case OVAL_DATETIME_WIN_FILETIME:
+		v = _parse_fmt_win(sv);
+		break;
+	case OVAL_DATETIME_SECONDS_SINCE_EPOCH:
+		v = _parse_fmt_sse(sv);
+		break;
+	default:
+		break;
+	}
+
+	return v;
+}
+
 static oval_syschar_collection_flag_t _oval_component_evaluate_TIMEDIF(oval_argu_t *argu,
 								       struct oval_component *component,
 								       struct oval_collection *value_collection)
 {
-	return SYSCHAR_FLAG_NOT_COLLECTED;	/*TODO: implement this function */
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	struct oval_component_iterator *subcomps;
+	int subcomp_idx;
+	struct oval_collection *val_cols[2];
+	struct oval_value_iterator *v1_itr;
+	oval_datetime_format_t fmt1, fmt2;
+
+	subcomps = oval_component_get_function_components(component);
+	subcomp_idx = 0;
+	while (oval_component_iterator_has_more(subcomps) && subcomp_idx < 2) {
+		struct oval_component *subcomp;
+
+		subcomp = oval_component_iterator_next(subcomps);
+		val_cols[subcomp_idx] = oval_collection_new();
+		flag = oval_component_eval_common(argu, subcomp, val_cols[subcomp_idx]);
+		subcomp_idx++;
+	}
+
+	if (oval_component_iterator_has_more(subcomps)) {
+		oval_component_iterator_free(subcomps);
+		oval_collection_free_items(val_cols[0], (oscap_destruct_func) oval_value_free);
+		oval_collection_free_items(val_cols[1], (oscap_destruct_func) oval_value_free);
+		return SYSCHAR_FLAG_ERROR;
+	}
+	oval_component_iterator_free(subcomps);
+	if (subcomp_idx == 0) {
+		return SYSCHAR_FLAG_ERROR;
+	}
+
+	fmt1 = oval_component_get_timedif_format_1(component);
+	fmt2 = oval_component_get_timedif_format_2(component);
+
+	if (subcomp_idx == 1) {
+		struct oval_value *ov;
+		char ts[16];
+
+		val_cols[1] = val_cols[0];
+		val_cols[0] = oval_collection_new();
+		snprintf(ts, sizeof (ts), "%lu", (long unsigned int) time(NULL));
+		ov = oval_value_new(OVAL_DATATYPE_INTEGER, ts);
+		oval_collection_add(val_cols[0], ov);
+		fmt1 = OVAL_DATETIME_SECONDS_SINCE_EPOCH;
+	}
+
+	v1_itr = (struct oval_value_iterator *) oval_collection_iterator(val_cols[0]);
+	while (oval_value_iterator_has_more(v1_itr)) {
+		struct oval_value *ov1;
+		struct oval_value_iterator *v2_itr;
+
+		ov1 = oval_value_iterator_next(v1_itr);
+
+		v2_itr = (struct oval_value_iterator *) oval_collection_iterator(val_cols[1]);
+		while (oval_value_iterator_has_more(v2_itr)) {
+			long unsigned int v;
+			char ts[16];
+			struct oval_value *ov2, *ov;
+
+			ov2 = oval_value_iterator_next(v2_itr);
+			v = _parse_fmt(ov1, fmt1) - _parse_fmt(ov2, fmt2);
+			snprintf(ts, sizeof (ts), "%lu", v);
+			ov = oval_value_new(OVAL_DATATYPE_INTEGER, ts);
+			oval_collection_add(value_collection, ov);
+		}
+		oval_value_iterator_free(v2_itr);
+	}
+	oval_value_iterator_free(v1_itr);
+
+	oval_collection_free_items(val_cols[0], (oscap_destruct_func) oval_value_free);
+	oval_collection_free_items(val_cols[1], (oscap_destruct_func) oval_value_free);
+
+	return flag;
 }
 
 static bool _isEscape(char chr)
@@ -1742,7 +2038,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(ova
 	pattern = oval_component_get_regex_pattern(component);
 	re = pcre_compile(pattern, PCRE_UTF8, &error, &erroffset, NULL);
 	if (re == NULL) {
-		oscap_dprintf("ERROR: pcre_compile() failed: \"%s\".\n", error);
+		oscap_dlprintf(DBG_E, "pcre_compile() failed: \"%s\".\n", error);
 		return SYSCHAR_FLAG_ERROR;
 	}
 #elif defined USE_REGEX_POSIX
@@ -1750,7 +2046,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(ova
 
 	pattern = oval_component_get_regex_pattern(component);
 	if ((rc = regcomp(&re, pattern, REG_EXTENDED | REG_NEWLINE)) != 0) {
-		oscap_dprintf("ERROR: regcomp() failed: %d.\n", rc);
+		oscap_dlprintf(DBG_E, "regcomp() failed: %d.\n", rc);
 		return SYSCHAR_FLAG_ERROR;
 	}
 #endif
@@ -1772,7 +2068,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(ova
 
 			rc = pcre_exec(re, NULL, text, strlen(text), 0, 0, ovector, ovector_len);
 			if (rc < -1) {
-				oscap_dprintf("ERROR: pcre_exec() failed: %d.\n", rc);
+				oscap_dlprintf(DBG_E, "pcre_exec() failed: %d.\n", rc);
 				flag = SYSCHAR_FLAG_ERROR;
 				break;
 			}
@@ -1818,12 +2114,137 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(ova
 	return flag;
 }
 
+struct val_col_lst_s {
+	void *val_col;
+	struct val_col_lst_s *next;
+};
+
+static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC_rec(struct val_col_lst_s *val_col_lst, double val,
+						    oval_datatype_t datatype, oval_arithmetic_operation_t op,
+						    struct oval_collection *res_val_col)
+{
+	struct oval_value_iterator *val_itr;
+
+	if (val_col_lst == NULL) {
+		struct oval_value *ov;
+		char sv[32];
+
+		if (datatype == OVAL_DATATYPE_INTEGER) {
+			snprintf(sv, sizeof (sv), "%ld", (long int) val);
+		} else if (datatype == OVAL_DATATYPE_FLOAT) {
+			snprintf(sv, sizeof (sv), "%f", val);
+		}
+		ov = oval_value_new(datatype, sv);
+		oval_collection_add(res_val_col, ov);
+
+		return SYSCHAR_FLAG_COMPLETE;
+	}
+
+	val_itr = (struct oval_value_iterator *) oval_collection_iterator(val_col_lst->val_col);
+	while (oval_value_iterator_has_more(val_itr)) {
+		struct oval_value *ov;
+		oval_datatype_t dt;
+		double new_val;
+
+		ov = oval_value_iterator_next(val_itr);
+		dt = oval_value_get_datatype(ov);
+		if (dt == OVAL_DATATYPE_INTEGER) {
+			new_val = (double) oval_value_get_integer(ov);
+		} else if (dt == OVAL_DATATYPE_FLOAT) {
+			new_val = (double) oval_value_get_float(ov);
+		} else {
+			oscap_dlprintf(DBG_E, "Unexpected value type: %s.\n", oval_datatype_get_text(dt));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected value type");
+			oval_value_iterator_free(val_itr);
+			return SYSCHAR_FLAG_ERROR;
+		}
+
+		if (op == OVAL_ARITHMETIC_ADD) {
+			new_val += val;
+		} else if (op == OVAL_ARITHMETIC_MULTIPLY) {
+			new_val *= val;
+		} else {
+			oscap_dlprintf(DBG_E, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected arithmetic operation");
+			oval_value_iterator_free(val_itr);
+			return SYSCHAR_FLAG_ERROR;
+		}
+
+		if (datatype == OVAL_DATATYPE_FLOAT)
+			dt = OVAL_DATATYPE_FLOAT;
+		_oval_component_evaluate_ARITHMETIC_rec(val_col_lst->next, new_val, dt, op, res_val_col);
+	}
+	oval_value_iterator_free(val_itr);
+
+	return SYSCHAR_FLAG_COMPLETE;
+}
+
 static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_argu_t *argu,
 									  struct oval_component *component,
 									  struct oval_collection *value_collection)
 {
-	return SYSCHAR_FLAG_UNKNOWN;
-	//TODO: Missing implementation
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	struct oval_component_iterator *subcomps;
+	struct val_col_lst_s *vcl_root, *vcl_elm;
+	oval_arithmetic_operation_t op;
+	struct oval_value_iterator *val_itr;
+
+	op = oval_component_get_arithmetic_operation(component);
+	if (op != OVAL_ARITHMETIC_ADD && op != OVAL_ARITHMETIC_MULTIPLY) {
+		oscap_dlprintf(DBG_E, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected arithmetic operation");
+		return SYSCHAR_FLAG_ERROR;
+	}
+
+	vcl_root = NULL;
+	subcomps = oval_component_get_function_components(component);
+	while (oval_component_iterator_has_more(subcomps)) {
+		struct oval_component *subcomp;
+		struct oval_collection *val_col;
+
+		subcomp = oval_component_iterator_next(subcomps);
+		val_col = oval_collection_new();
+		// todo: combine flags
+		flag = oval_component_eval_common(argu, subcomp, val_col);
+		vcl_elm = oscap_alloc(sizeof (struct val_col_lst_s));
+		vcl_elm->val_col = val_col;
+		vcl_elm->next = vcl_root;
+		vcl_root = vcl_elm;
+	}
+	oval_component_iterator_free(subcomps);
+
+	val_itr = (struct oval_value_iterator *) oval_collection_iterator(vcl_root->val_col);
+	while (oval_value_iterator_has_more(val_itr)) {
+		struct oval_value *ov;
+		oval_datatype_t datatype;
+		double val;
+
+		ov = oval_value_iterator_next(val_itr);
+		datatype = oval_value_get_datatype(ov);
+		if (datatype == OVAL_DATATYPE_INTEGER) {
+			val = (double) oval_value_get_integer(ov);
+		} else if (datatype == OVAL_DATATYPE_FLOAT) {
+			val = (double) oval_value_get_float(ov);
+		} else {
+			oscap_dlprintf(DBG_E, "Unexpected value type: %s.\n", oval_datatype_get_text(datatype));
+			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected value type");
+			flag = SYSCHAR_FLAG_ERROR;
+			goto cleanup;
+		}
+
+		flag = _oval_component_evaluate_ARITHMETIC_rec(vcl_root->next, val, datatype, op, value_collection);
+	}
+
+ cleanup:
+	oval_value_iterator_free(val_itr);
+	while (vcl_root != NULL) {
+		oval_collection_free_items(vcl_root->val_col, (oscap_destruct_func) oval_value_free);
+		vcl_elm = vcl_root;
+		vcl_root = vcl_root->next;
+		oscap_free(vcl_elm);
+	}
+
+	return flag;
 }
 
 typedef oval_syschar_collection_flag_t(_oval_component_evaluator)
@@ -1861,7 +2282,7 @@ static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *ar
 	if (evaluator) {
 		flag = (*evaluator) (argu, component, value_collection);
 	} else {
-		oscap_dprintf("ERROR component type %d not supported.\n", type);
+		oscap_dlprintf(DBG_E, "Component type %d not supported.\n", type);
 		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Component type not supported");
 	}
 	return flag;
