@@ -149,7 +149,14 @@ struct oval_criteria_node *oval_definition_get_criteria(struct oval_definition
 
 struct oval_definition *oval_definition_new(struct oval_definition_model *model, const char *id)
 {
-	struct oval_definition *definition = (struct oval_definition *)oscap_alloc(sizeof(oval_definition_t));
+	struct oval_definition *definition;
+
+	if (model && oval_definition_model_is_locked(model)) {
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+		return NULL;
+	}
+
+	definition = (struct oval_definition *)oscap_alloc(sizeof(oval_definition_t));;
 	if (definition == NULL)
 		return NULL;
 
@@ -164,6 +171,9 @@ struct oval_definition *oval_definition_new(struct oval_definition_model *model,
 	definition->notes = oval_collection_new();
 	definition->criteria = NULL;
 	definition->model = model;
+
+	oval_definition_model_add_definition(model, definition);
+
 	return definition;
 }
 
@@ -172,14 +182,14 @@ bool oval_definition_is_valid(struct oval_definition * definition)
 	struct oval_criteria_node *criteria_node;
 
 	if (definition == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+		oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
-        }
+	}
 
-        if (oval_definition_get_class(definition) == OVAL_CLASS_UNKNOWN) {
-                oscap_dprintf("WARNING: argument is not valid: class == OVAL_CLASS_UNKNOWN.\n");
-                return false;
-        }
+	if (oval_definition_get_class(definition) == OVAL_CLASS_UNKNOWN) {
+		oscap_dlprintf(DBG_W, "Argument is not valid: class == OVAL_CLASS_UNKNOWN.\n");
+		return false;
+	}
 
 	criteria_node = oval_definition_get_criteria(definition);
 	if (oval_criteria_node_is_valid(criteria_node) != true)
@@ -229,8 +239,6 @@ struct oval_definition *oval_definition_clone
 
 		oval_definition_set_criteria(new_definition,
 					     oval_criteria_node_clone(new_model, old_definition->criteria));
-
-		oval_definition_model_add_definition(new_model, new_definition);
 	}
 	return new_definition;
 }
@@ -290,7 +298,7 @@ void oval_definition_set_version(struct oval_definition *definition, int version
 	if (definition && !oval_definition_is_locked(definition)) {
 		definition->version = version;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_set_class(struct oval_definition *definition, oval_definition_class_t class)
@@ -298,7 +306,7 @@ void oval_definition_set_class(struct oval_definition *definition, oval_definiti
 	if (definition && !oval_definition_is_locked(definition)) {
 		definition->class = class;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_set_deprecated(struct oval_definition *definition, bool deprecated)
@@ -306,7 +314,7 @@ void oval_definition_set_deprecated(struct oval_definition *definition, bool dep
 	if (definition && !oval_definition_is_locked(definition)) {
 		definition->deprecated = deprecated;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_set_title(struct oval_definition *definition, char *title)
@@ -316,7 +324,7 @@ void oval_definition_set_title(struct oval_definition *definition, char *title)
 			oscap_free(definition->title);
 		definition->title = (title == NULL) ? NULL : oscap_strdup(title);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_set_description(struct oval_definition *definition, char *description)
@@ -326,7 +334,7 @@ void oval_definition_set_description(struct oval_definition *definition, char *d
 			oscap_free(definition->description);
 		definition->description = (description == NULL) ? NULL : oscap_strdup(description);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_set_criteria(struct oval_definition *definition, struct oval_criteria_node *criteria)
@@ -334,7 +342,7 @@ void oval_definition_set_criteria(struct oval_definition *definition, struct ova
 	if (definition && !oval_definition_is_locked(definition)) {
 		definition->criteria = criteria;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_add_affected(struct oval_definition *definition, struct oval_affected *affected)
@@ -342,7 +350,7 @@ void oval_definition_add_affected(struct oval_definition *definition, struct ova
 	if (definition && !oval_definition_is_locked(definition)) {
 		oval_collection_add(definition->affected, affected);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_add_reference(struct oval_definition *definition, struct oval_reference *ref)
@@ -350,14 +358,14 @@ void oval_definition_add_reference(struct oval_definition *definition, struct ov
 	if (definition && !oval_definition_is_locked(definition)) {
 		oval_collection_add(definition->reference, ref);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_definition_add_note(struct oval_definition *definition, char *note) {
 	if (definition && !oval_definition_is_locked(definition)) {
 		oval_collection_add(definition->notes, note);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 static oval_definition_class_t _odaclass(char *class)
@@ -452,7 +460,7 @@ static int _oval_definition_parse_metadata(xmlTextReaderPtr reader, struct oval_
 		int depth = xmlTextReaderDepth(reader);
 		if (depth == -1)
 			oscap_setxmlerr(xmlGetLastError());
-		oscap_dprintf("NOTICE::(oval_definition_parse_metadata)skipping <%s> depth = %d line = %d",
+		oscap_dlprintf(DBG_I, "Skipping tag: %s, depth: %d, line: %d.\n",
 			      tagname, depth, xmlTextReaderGetParserLineNumber(reader));
 		return_code = oval_parser_skip_tag(reader, context);
 	}
@@ -481,7 +489,7 @@ static int _oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parse
 		int depth = xmlTextReaderDepth(reader);
 		if (depth == -1)
 			oscap_setxmlerr(xmlGetLastError());
-		oscap_dprintf("NOTICE::(oval_definition)skipping <%s> depth = %d line = %d", tagname, depth,
+		oscap_dlprintf(DBG_I, "Skipping tag: %s, depth: %d, line: %d.\n", tagname, depth,
                               xmlTextReaderGetParserLineNumber(reader));
 		return_code = oval_parser_skip_tag(reader, context);
 	}
@@ -553,7 +561,7 @@ void oval_definition_to_print(struct oval_definition *definition, char *indent,
 xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc, xmlNode * parent)
 {
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
-	xmlNode *definition_node = xmlNewChild(parent, ns_definitions, BAD_CAST "definition", NULL);
+	xmlNode *definition_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "definition", NULL);
 
 	char *id = oval_definition_get_id(definition);
 	xmlNewProp(definition_node, BAD_CAST "id", BAD_CAST id);
@@ -570,27 +578,27 @@ xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc
 	if (deprecated)
 		xmlNewProp(definition_node, BAD_CAST "deprecated", BAD_CAST "true");
 
-	xmlNode *metadata_node = xmlNewChild(definition_node, ns_definitions, BAD_CAST "metadata", NULL);
+	xmlNode *metadata_node = xmlNewTextChild(definition_node, ns_definitions, BAD_CAST "metadata", NULL);
 
 	char *title = oval_definition_get_title(definition);
-	xmlNewChild(metadata_node, ns_definitions, BAD_CAST "title", BAD_CAST title);
+	xmlNewTextChild(metadata_node, ns_definitions, BAD_CAST "title", BAD_CAST title);
 
 	struct oval_affected_iterator *affecteds = oval_definition_get_affected(definition);
 	while (oval_affected_iterator_has_more(affecteds)) {
-		xmlNode *affected_node = xmlNewChild(metadata_node, ns_definitions, BAD_CAST "affected", NULL);
+		xmlNode *affected_node = xmlNewTextChild(metadata_node, ns_definitions, BAD_CAST "affected", NULL);
 		struct oval_affected *affected = oval_affected_iterator_next(affecteds);
 		oval_affected_family_t family = oval_affected_get_family(affected);
 		xmlNewProp(affected_node, BAD_CAST "family", BAD_CAST oval_affected_family_get_text(family));
 		struct oval_string_iterator *platforms = oval_affected_get_platforms(affected);
 		while (oval_string_iterator_has_more(platforms)) {
 			char *platform = oval_string_iterator_next(platforms);
-			xmlNewChild(affected_node, ns_definitions, BAD_CAST "platform", BAD_CAST platform);
+			xmlNewTextChild(affected_node, ns_definitions, BAD_CAST "platform", BAD_CAST platform);
 		}
 		oval_string_iterator_free(platforms);
 		struct oval_string_iterator *products = oval_affected_get_products(affected);
 		while (oval_string_iterator_has_more(products)) {
 			char *product = oval_string_iterator_next(products);
-			xmlNewChild(affected_node, ns_definitions, BAD_CAST "product", BAD_CAST product);
+			xmlNewTextChild(affected_node, ns_definitions, BAD_CAST "product", BAD_CAST product);
 		}
 		oval_string_iterator_free(products);
 	}
@@ -599,7 +607,7 @@ xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc
 	struct oval_reference_iterator *references = oval_definition_get_references(definition);
 	while (oval_reference_iterator_has_more(references)) {
 		struct oval_reference *ref = oval_reference_iterator_next(references);
-		xmlNode *referenceNode = xmlNewChild(metadata_node, ns_definitions, BAD_CAST "reference", NULL);
+		xmlNode *referenceNode = xmlNewTextChild(metadata_node, ns_definitions, BAD_CAST "reference", NULL);
 		char *source = oval_reference_get_source(ref);
 		char *ref_id = oval_reference_get_id(ref);
 		char *ref_url = oval_reference_get_url(ref);
@@ -611,14 +619,14 @@ xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc
 	oval_reference_iterator_free(references);
 
 	char *description = oval_definition_get_description(definition);
-	xmlNewChild(metadata_node, ns_definitions, BAD_CAST "description", BAD_CAST description);
+	xmlNewTextChild(metadata_node, ns_definitions, BAD_CAST "description", BAD_CAST description);
 
 	struct oval_string_iterator *notes = oval_definition_get_notes(definition);
 	if (oval_string_iterator_has_more(notes)) {
-		xmlNode *notes_node = xmlNewChild(definition_node, ns_definitions, BAD_CAST "notes", NULL);
+		xmlNode *notes_node = xmlNewTextChild(definition_node, ns_definitions, BAD_CAST "notes", NULL);
 		while (oval_string_iterator_has_more(notes)) {
 			char *note = oval_string_iterator_next(notes);
-			xmlNewChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
+			xmlNewTextChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
 		}
 	}
 	oval_string_iterator_free(notes);
