@@ -52,6 +52,7 @@ oval_datatype_t oval_datatype_parse(xmlTextReaderPtr, char *, oval_datatype_t);
 oval_entity_varref_type_t oval_entity_varref_type_parse(xmlTextReaderPtr);
 oval_setobject_type_t oval_set_type_parse(xmlTextReaderPtr);
 oval_setobject_operation_t oval_set_operation_parse(xmlTextReaderPtr, char *, oval_setobject_operation_t);
+oval_filter_action_t oval_filter_action_parse(xmlTextReaderPtr, char *, oval_filter_action_t);
 oval_variable_type_t oval_variable_type_parse(xmlTextReaderPtr);
 oval_component_type_t oval_component_type_parse(xmlTextReaderPtr);
 oval_arithmetic_operation_t oval_arithmetic_operation_parse(xmlTextReaderPtr, char *, oval_arithmetic_operation_t);
@@ -84,6 +85,9 @@ xmlNode *oval_definition_to_dom(struct oval_definition *, xmlDoc *, xmlNode *);
 int oval_object_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context);
 void oval_object_to_print(struct oval_object *object, char *indent, int idx);
 xmlNode *oval_object_to_dom(struct oval_object *, xmlDoc *, xmlNode *);
+struct oval_object *oval_object_clone2(struct oval_definition_model *, struct oval_object *, char *);
+struct oval_object *oval_object_create_internal(struct oval_object *, char *);
+struct oval_object *oval_object_get_base_obj(struct oval_object *);
 
 int oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context);
 void oval_state_to_print(struct oval_state *, char *indent, int index);
@@ -99,11 +103,18 @@ int oval_variable_binding_parse_tag
     (xmlTextReaderPtr, struct oval_parser_context *, oval_variable_binding_consumer, void *);
 
 const char *oval_variable_type_get_text(oval_variable_type_t);
+const char *oval_filter_action_get_text(oval_filter_action_t);
+
+typedef void (*oval_filter_consumer) (struct oval_filter *, void *);
+int oval_filter_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oval_filter_consumer, void *);
+xmlNode *oval_filter_to_dom(struct oval_filter *, xmlDoc *, xmlNode *);
 
 typedef void (*oval_object_content_consumer) (struct oval_object_content *, void *);
 xmlNode *oval_object_content_to_dom(struct oval_object_content *, xmlDoc *, xmlNode *);
 int oval_object_content_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oval_object_content_consumer, void *);
 void oval_object_content_to_print(struct oval_object_content *, char *indent, int index);
+struct oval_filter *oval_object_content_get_filter(struct oval_object_content *);
+void oval_object_content_set_filter(struct oval_object_content *, struct oval_filter *);
 
 int oval_state_content_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oscap_consumer_func, void *);
 xmlNode *oval_state_content_to_dom(struct oval_state_content *, xmlDoc *, xmlNode *);
@@ -121,11 +132,13 @@ typedef void (*oval_set_consumer) (struct oval_setobject *, void *);
 int oval_set_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oval_set_consumer, void *);
 void oval_set_to_print(struct oval_setobject *, char *indent, int index);
 xmlNode *oval_set_to_dom(struct oval_setobject *, xmlDoc *, xmlNode *);
+void oval_set_propagate_filters(struct oval_definition_model *, struct oval_setobject *, char *);
 
 typedef void (*oval_value_consumer) (struct oval_value *, void *);
 int oval_value_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oval_value_consumer, void *);
 void oval_value_to_print(struct oval_value *value, char *indent, int idx);
 xmlNode *oval_value_to_dom(struct oval_value *, xmlDoc *, xmlNode *);
+int oval_value_cast(struct oval_value *value, oval_datatype_t new_dt);
 
 oval_syschar_collection_flag_t oval_component_compute(struct oval_syschar_model *sysmod, struct oval_component *component,
 						      struct oval_collection *value_collection);
@@ -142,6 +155,17 @@ int oval_message_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, oscap
 void oval_message_to_print(struct oval_message *message, char *indent, int index);
 void oval_message_to_dom(struct oval_message *, xmlDoc *, xmlNode *);
 
+int oval_generator_parse_tag(xmlTextReader *, struct oval_parser_context *, struct oval_generator *);
+xmlNode *oval_generator_to_dom(struct oval_generator *, xmlDocPtr, xmlNode *);
+
+void oval_definition_model_add_definition(struct oval_definition_model *, struct oval_definition *);
+void oval_definition_model_add_test(struct oval_definition_model *, struct oval_test *);
+void oval_definition_model_add_object(struct oval_definition_model *, struct oval_object *);
+void oval_definition_model_add_state(struct oval_definition_model *, struct oval_state *);
+void oval_definition_model_add_variable(struct oval_definition_model *, struct oval_variable *);
+
+void oval_definition_model_optimize_by_filter_propagation(struct oval_definition_model *);
+
 //Synthetic object subtype for probing system info.
 #define  OVAL_INDEPENDENT_SYSCHAR_SUBTYPE 7999
 
