@@ -38,6 +38,7 @@
 typedef struct oval_object {
 	struct oval_definition_model *model;
 	oval_subtype_t subtype;
+	struct oval_object *base_obj_ref;
 	struct oval_collection *notes;
 	char *comment;
 	char *id;
@@ -143,19 +144,30 @@ struct oval_behavior_iterator *oval_object_get_behaviors(struct oval_object *obj
 
 struct oval_object *oval_object_new(struct oval_definition_model *model, const char *id)
 {
-	oval_object_t *object = (oval_object_t *) oscap_alloc(sizeof(oval_object_t));
+	oval_object_t *object;
+
+	if (model && oval_definition_model_is_locked(model)) {
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+		return NULL;
+	}
+
+	object = (oval_object_t *) oscap_alloc(sizeof(oval_object_t));
 	if (object == NULL)
 		return NULL;
 
 	object->comment = NULL;
 	object->id = oscap_strdup(id);
 	object->subtype = OVAL_SUBTYPE_UNKNOWN;
+	object->base_obj_ref = NULL;
 	object->deprecated = 0;
 	object->version = 0;
 	object->behaviors = oval_collection_new();
 	object->notes = oval_collection_new();
 	object->object_content = oval_collection_new();
 	object->model = model;
+
+	oval_definition_model_add_object(model, object);
+
 	return object;
 }
 
@@ -166,12 +178,12 @@ bool oval_object_is_valid(struct oval_object * object)
 	struct oval_behavior_iterator *behaviors_itr;
 
 	if (object == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
         }
 
         if (oval_object_get_subtype(object) == OVAL_SUBTYPE_UNKNOWN) {
-                oscap_dprintf("WARNING: argument is not valid: subtype == OVAL_SUBTYPE_UNKNOWN.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: subtype == OVAL_SUBTYPE_UNKNOWN.\n");
                 return false;
         }
 
@@ -215,12 +227,18 @@ bool oval_object_is_locked(struct oval_object * object)
 	return oval_definition_model_is_locked(object->model);
 }
 
-struct oval_object *oval_object_clone(struct oval_definition_model *new_model, struct oval_object *old_object) {
+struct oval_object *oval_object_clone2(struct oval_definition_model *new_model, struct oval_object *old_object, char *new_id)
+{
 	__attribute__nonnull__(old_object);
 
-	struct oval_object *new_object = oval_definition_model_get_object(new_model, old_object->id);
+	struct oval_object *new_object;
+
+	if (new_id == NULL)
+		new_id = old_object->id;
+
+	new_object = oval_definition_model_get_object(new_model, new_id);
 	if (new_object == NULL) {
-		new_object = oval_object_new(new_model, old_object->id);
+		new_object = oval_object_new(new_model, new_id);
 		oval_object_set_comment(new_object, old_object->comment);
 		oval_object_set_subtype(new_object, old_object->subtype);
 		oval_object_set_deprecated(new_object, old_object->deprecated);
@@ -245,12 +263,15 @@ struct oval_object *oval_object_clone(struct oval_definition_model *new_model, s
 						       oval_object_content_clone(new_model, object_content));
 		}
 		oval_object_content_iterator_free(object_contents);
-
-		oval_definition_model_add_object(new_model, new_object);
 	}
 	return new_object;
 }
 
+struct oval_object *oval_object_clone(struct oval_definition_model *new_model, struct oval_object *old_object)
+{
+	return oval_object_clone2(new_model, old_object, NULL);
+}
+
 void oval_object_free(struct oval_object *object)
 {
 	if (object == NULL)
@@ -277,7 +298,7 @@ void oval_object_set_subtype(struct oval_object *object, oval_subtype_t subtype)
 	if (object && !oval_object_is_locked(object)) {
 		object->subtype = subtype;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_add_note(struct oval_object *object, char *note)
@@ -285,7 +306,7 @@ void oval_object_add_note(struct oval_object *object, char *note)
 	if (object && !oval_object_is_locked(object)) {
 		oval_collection_add(object->notes, (void *)oscap_strdup(note));
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_set_comment(struct oval_object *object, char *comm)
@@ -295,7 +316,7 @@ void oval_object_set_comment(struct oval_object *object, char *comm)
 			oscap_free(object->comment);
 		object->comment = (comm == NULL) ? NULL : oscap_strdup(comm);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_set_deprecated(struct oval_object *object, bool deprecated)
@@ -303,7 +324,7 @@ void oval_object_set_deprecated(struct oval_object *object, bool deprecated)
 	if (object && !oval_object_is_locked(object)) {
 		object->deprecated = deprecated;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_set_version(struct oval_object *object, int version)
@@ -311,7 +332,7 @@ void oval_object_set_version(struct oval_object *object, int version)
 	if (object && !oval_object_is_locked(object)) {
 		object->version = version;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_add_object_content(struct oval_object *object, struct oval_object_content *content)
@@ -319,7 +340,7 @@ void oval_object_add_object_content(struct oval_object *object, struct oval_obje
 	if (object && !oval_object_is_locked(object)) {
 		oval_collection_add(object->object_content, (void *)content);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_add_behavior(struct oval_object *object, struct oval_behavior *behavior)
@@ -327,7 +348,7 @@ void oval_object_add_behavior(struct oval_object *object, struct oval_behavior *
 	if (object && !oval_object_is_locked(object)) {
 		oval_collection_add(object->behaviors, (void *)behavior);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 static void oval_note_consume(char *text, void *object)
@@ -367,8 +388,8 @@ static int _oval_object_parse_tag(xmlTextReaderPtr reader, struct oval_parser_co
 		return_code = oval_object_content_parse_tag(reader, context, &oval_content_consume, object);
 	}
 	if (return_code != 1) {
-		oscap_dprintf("NOTICE: oval_object_parse_tag::parse of %s terminated on error at <%s> line %d\n",
-			      object->id, tagname, xmlTextReaderGetParserLineNumber(reader));
+		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n",
+			       tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	oscap_free(namespace);
@@ -381,7 +402,7 @@ int oval_object_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *c
 {
 	struct oval_definition_model *model = oval_parser_context_model(context);
 	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
-	oscap_dprintf("DEBUG::oval_object_parse_tag::id = %s", id);
+	oscap_dlprintf(DBG_I, "Object id: %s.\n", id);
 	struct oval_object *object = oval_object_get_new(model, id);
 	oscap_free(id);
 	id = NULL;
@@ -449,7 +470,7 @@ xmlNode *oval_object_to_dom(struct oval_object *object, xmlDoc * doc, xmlNode *
 	char object_name[strlen(subtype_text) + 8];
 	*object_name = '\0';
 	strcat(strcat(object_name, subtype_text), "_object");
-	xmlNode *object_node = xmlNewChild(parent, NULL, BAD_CAST object_name, NULL);
+	xmlNode *object_node = xmlNewTextChild(parent, NULL, BAD_CAST object_name, NULL);
 
 	oval_family_t family = oval_object_get_family(object);
 	const char *family_text = oval_family_get_text(family);
@@ -479,17 +500,17 @@ xmlNode *oval_object_to_dom(struct oval_object *object, xmlDoc * doc, xmlNode *
 	struct oval_string_iterator *notes = oval_object_get_notes(object);
 	if (oval_string_iterator_has_more(notes)) {
 		xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
-		xmlNode *notes_node = xmlNewChild(object_node, ns_definitions, BAD_CAST "notes", NULL);
+		xmlNode *notes_node = xmlNewTextChild(object_node, ns_definitions, BAD_CAST "notes", NULL);
 		while (oval_string_iterator_has_more(notes)) {
 			char *note = oval_string_iterator_next(notes);
-			xmlNewChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
+			xmlNewTextChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
 		}
 	}
 	oval_string_iterator_free(notes);
 
 	struct oval_behavior_iterator *behaviors = oval_object_get_behaviors(object);
 	if (oval_behavior_iterator_has_more(behaviors)) {
-		xmlNode *behaviors_node = xmlNewChild(object_node, ns_family, BAD_CAST "behaviors", NULL);
+		xmlNode *behaviors_node = xmlNewTextChild(object_node, ns_family, BAD_CAST "behaviors", NULL);
 		while (oval_behavior_iterator_has_more(behaviors)) {
 			struct oval_behavior *behavior = oval_behavior_iterator_next(behaviors);
 			char *key = oval_behavior_get_key(behavior);
@@ -509,3 +530,29 @@ xmlNode *oval_object_to_dom(struct oval_object *object, xmlDoc * doc, xmlNode *
 
 	return object_node;
 }
+
+struct oval_object *oval_object_create_internal(struct oval_object *obj, char *set_id)
+{
+	struct oval_object *new_obj;
+	size_t oid_len, sid_len;
+	char *new_obj_id;
+
+	oid_len = strlen(obj->id);
+	set_id = strrchr(set_id, ':') + 1;
+	sid_len = strlen(set_id);
+	new_obj_id = malloc(oid_len + sid_len + 2);
+	memcpy(new_obj_id, obj->id, oid_len);
+	new_obj_id[oid_len] = 'i';
+	memcpy(new_obj_id + oid_len + 1, set_id, sid_len);
+	new_obj_id[oid_len + sid_len + 1] = '\0';
+	new_obj = oval_object_clone2(obj->model, obj, new_obj_id);
+	new_obj->base_obj_ref = obj;
+	oscap_free(new_obj_id);
+
+	return new_obj;
+}
+
+struct oval_object *oval_object_get_base_obj(struct oval_object *obj)
+{
+	return obj->base_obj_ref;
+}
