@@ -57,6 +57,13 @@ typedef struct oval_object_content_SET {
 	struct oval_setobject *set;	/*type == OVAL_OBJECTCONTENT_SET */
 } oval_object_content_SET_t;
 
+typedef struct oval_object_content_FILTER {
+	struct oval_definition_model *model;
+	char *fieldName;
+	oval_object_content_type_t type;
+	struct oval_filter *filter;	/*type == OVAL_OBJECTCONTENT_FILTER */
+} oval_object_content_FILTER_t;
+
 bool oval_object_content_iterator_has_more(struct oval_object_content_iterator
 					   *oc_object_content)
 {
@@ -132,6 +139,18 @@ struct oval_setobject *oval_object_content_get_setobject(struct oval_object_cont
 	return set;
 }
 
+struct oval_filter *oval_object_content_get_filter(struct oval_object_content *content)
+{
+	__attribute__nonnull__(content);
+
+	/* type == OVAL_OBJECTCONTENT_FILTER */
+	struct oval_filter *filter = NULL;
+	if (oval_object_content_get_type(content) == OVAL_OBJECTCONTENT_FILTER) {
+		filter = ((struct oval_object_content_FILTER *)content)->filter;
+	}
+	return filter;
+}
+
 struct oval_object_content
 *oval_object_content_new(struct oval_definition_model *model, oval_object_content_type_t type)
 {
@@ -158,6 +177,16 @@ struct oval_object_content
 			content = (oval_object_content_t *) set;
 		}
 		break;
+	case OVAL_OBJECTCONTENT_FILTER:{
+			struct oval_object_content_FILTER *filter =
+			    (oval_object_content_FILTER_t *) oscap_alloc(sizeof(oval_object_content_FILTER_t));
+			if (filter == NULL)
+				return NULL;
+
+			filter->filter = NULL;
+			content = (oval_object_content_t *) filter;
+		}
+		break;
 	case OVAL_OBJECTCONTENT_UNKNOWN:
 		break;
 	}
@@ -172,7 +201,7 @@ bool oval_object_content_is_valid(struct oval_object_content * object_content)
 	oval_object_content_type_t type;
 
 	if (object_content == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
         }
 
@@ -196,8 +225,17 @@ bool oval_object_content_is_valid(struct oval_object_content * object_content)
 				return false;
 		}
 		break;
+	case OVAL_OBJECTCONTENT_FILTER:
+		{
+			struct oval_filter *filter;
+
+			filter = oval_object_content_get_filter(object_content);
+			if (oval_filter_is_valid(filter) != true)
+				return false;
+		}
+		break;
 	default:
-                oscap_dprintf("WARNING: argument is not valid: wrong object content type: %d.\n", type);
+                oscap_dlprintf(DBG_W, "Argument is not valid: wrong object content type: %d.\n", type);
 		return false;
 	}
 
@@ -226,7 +264,11 @@ struct oval_object_content *oval_object_content_clone
 	case OVAL_OBJECTCONTENT_SET:{
 			struct oval_setobject *set = oval_object_content_get_setobject(old_content);
 			oval_object_content_set_setobject(new_content, oval_setobject_clone(new_model, set));
-		}
+		} break;
+	case OVAL_OBJECTCONTENT_FILTER:{
+			struct oval_filter *filter = oval_object_content_get_filter(old_content);
+			oval_object_content_set_filter(new_content, oval_filter_clone(new_model, filter));
+		} break;
 	default:
 		/*NOOP*/;
 	}
@@ -255,6 +297,13 @@ void oval_object_content_free(struct oval_object_content *content)
 			set->set = NULL;
 		}
 		break;
+	case OVAL_OBJECTCONTENT_FILTER:{
+			struct oval_object_content_FILTER *filter = (oval_object_content_FILTER_t *) content;
+			if (filter->filter != NULL)
+				oval_filter_free(filter->filter);
+			filter->filter = NULL;
+		}
+		break;
 	case OVAL_OBJECTCONTENT_UNKNOWN:
 		break;
 	}
@@ -266,7 +315,7 @@ void oval_object_content_set_type(struct oval_object_content *content, oval_obje
 	if (content && !oval_object_content_is_locked(content)) {
 		content->type = type;
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_content_set_field_name(struct oval_object_content *content, char *name)
@@ -276,7 +325,7 @@ void oval_object_content_set_field_name(struct oval_object_content *content, cha
 			oscap_free(content->fieldName);
 		content->fieldName = (name == NULL) ? NULL : oscap_strdup(name);
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_content_set_entity(struct oval_object_content *content, struct oval_entity *entity)
@@ -287,7 +336,7 @@ void oval_object_content_set_entity(struct oval_object_content *content, struct
 			content_ENTITY->entity = entity;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_content_set_varCheck(struct oval_object_content *content, oval_check_t check)
@@ -298,7 +347,7 @@ void oval_object_content_set_varCheck(struct oval_object_content *content, oval_
 			content_ENTITY->varCheck = check;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_object_content_set_setobject(struct oval_object_content *content, struct oval_setobject *set)
@@ -309,13 +358,23 @@ void oval_object_content_set_setobject(struct oval_object_content *content, stru
 			content_SET->set = set;
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+}
+
+void oval_object_content_set_filter(struct oval_object_content *content, struct oval_filter *filter)
+{				/*type == OVAL_OBJECTCONTENT_FILTER */
+	if (content && !oval_object_content_is_locked(content)) {
+		if (content->type == OVAL_OBJECTCONTENT_FILTER) {
+			oval_object_content_FILTER_t *content_FILTER = (oval_object_content_FILTER_t *) content;
+			content_FILTER->filter = filter;
+		}
+	} else
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 /*typedef void (*oval_object_content_consumer)(struct oval_object_content*,void*);*/
 static void oval_consume_entity(struct oval_entity *entity, void *content_entity)
 {
-
 	__attribute__nonnull__(entity);
 
 	((struct oval_object_content_ENTITY *)content_entity)->entity = entity;
@@ -323,12 +382,18 @@ static void oval_consume_entity(struct oval_entity *entity, void *content_entity
 
 static void oval_consume_set(struct oval_setobject *set, void *content_set)
 {
-
 	__attribute__nonnull__(content_set);
 
 	((struct oval_object_content_SET *)content_set)->set = set;
 }
 
+static void oval_consume_filter(struct oval_filter *filter, void *content_filter)
+{
+	__attribute__nonnull__(content_filter);
+
+	((struct oval_object_content_FILTER *)content_filter)->filter = filter;
+}
+
 int oval_object_content_parse_tag(xmlTextReaderPtr reader,
 				  struct oval_parser_context *context,
 				  oval_object_content_consumer consumer, void *user)
@@ -337,15 +402,23 @@ int oval_object_content_parse_tag(xmlTextReaderPtr reader,
 
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
+	oval_object_content_type_t type = OVAL_OBJECTCONTENT_UNKNOWN;
+	struct oval_object_content *content;
+	int return_code = 0;
 
-	oval_object_content_type_t type =
-	    (strcmp(tagname, "set") == 0) ? OVAL_OBJECTCONTENT_SET : OVAL_OBJECTCONTENT_ENTITY;
-	struct oval_object_content *content = oval_object_content_new(context->definition_model, type);
+	if (!strcmp(tagname, "set")) {
+		type = OVAL_OBJECTCONTENT_SET;
+	} else if (!strcmp(tagname, "filter")) {
+		type = OVAL_OBJECTCONTENT_FILTER;
+	} else {
+		type = OVAL_OBJECTCONTENT_ENTITY;
+	}
+
+	content = oval_object_content_new(context->definition_model, type);
 	if (content == NULL)
 		return -1;
 
 	content->fieldName = tagname;
-	int return_code = 0;
 	switch (type) {
 	case OVAL_OBJECTCONTENT_ENTITY:{
 			struct oval_object_content_ENTITY *content_entity =
@@ -361,13 +434,19 @@ int oval_object_content_parse_tag(xmlTextReaderPtr reader,
 			return_code = oval_set_parse_tag(reader, context, &oval_consume_set, content_set);
 		};
 		break;
+	case OVAL_OBJECTCONTENT_FILTER:{
+			struct oval_object_content_FILTER *content_filter =
+			    (struct oval_object_content_FILTER *) content;
+			return_code = oval_filter_parse_tag(reader, context, &oval_consume_filter, content_filter);
+		};
+		break;
 	case OVAL_OBJECTCONTENT_UNKNOWN:
 		break;
 	}
 	(*consumer) (content, user);
 	if (return_code != 1) {
-		oscap_dprintf("NOTICE: oval_object_content_parse_tag::parse of <%s> terminated on error line %d",
-			      tagname, xmlTextReaderGetParserLineNumber(reader));
+		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n",
+			       tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(namespace);
 	return return_code;
@@ -401,6 +480,7 @@ void oval_object_content_to_print(struct oval_object_content *content, char *ind
 			struct oval_setobject *set = oval_object_content_get_setobject(content);
 			oval_set_to_print(set, nxtindent, 0);
 		} break;
+	case OVAL_OBJECTCONTENT_FILTER:
 	case OVAL_OBJECTCONTENT_UNKNOWN:
 		break;
 	}
@@ -421,6 +501,13 @@ xmlNode *oval_object_content_to_dom(struct oval_object_content *content, xmlDoc
 			struct oval_setobject *set = oval_object_content_get_setobject(content);
 			content_node = oval_set_to_dom(set, doc, parent);
 		} break;
+	case OVAL_OBJECTCONTENT_FILTER:{
+			struct oval_filter *filter;
+
+			filter = oval_object_content_get_filter(content);
+			content_node = oval_filter_to_dom(filter, doc, parent);
+		}
+		break;
 	default:
 		content_node = NULL;
 	}
