@@ -136,74 +136,268 @@ oval_subtype_t oval_str2subtype(const char *str)
         return (d == NULL ? OVAL_SUBTYPE_UNKNOWN : d->type);
 }
 
-struct oval_syschar *oval_probe_object_query(oval_probe_session_t *psess, struct oval_object *object, int flags)
+static void _obj_collect_var_refs(struct oval_object *obj, struct oval_string_map *vm);
+static void _var_collect_var_refs(struct oval_variable *var, struct oval_string_map *vm);
+
+static void _comp_collect_var_refs(struct oval_component *comp, struct oval_string_map *vm)
 {
-        struct oval_syschar *o_sys;
+	struct oval_object *obj;
+	struct oval_variable *var;
+	struct oval_component_iterator *cmp_itr;
+
+	switch (oval_component_get_type(comp)) {
+	case OVAL_COMPONENT_OBJECTREF:
+		obj = oval_component_get_object(comp);
+		_obj_collect_var_refs(obj, vm);
+		break;
+	case OVAL_COMPONENT_VARREF:
+		var = oval_component_get_variable(comp);
+		_var_collect_var_refs(var, vm);
+		break;
+	case OVAL_FUNCTION_ARITHMETIC:
+	case OVAL_FUNCTION_BEGIN:
+	case OVAL_FUNCTION_CONCAT:
+	case OVAL_FUNCTION_END:
+	case OVAL_FUNCTION_ESCAPE_REGEX:
+	case OVAL_FUNCTION_REGEX_CAPTURE:
+	case OVAL_FUNCTION_SPLIT:
+	case OVAL_FUNCTION_SUBSTRING:
+	case OVAL_FUNCTION_TIMEDIF:
+		cmp_itr = oval_component_get_function_components(comp);
+		while (oval_component_iterator_has_more(cmp_itr)) {
+			struct oval_component *cmp;
+
+			cmp = oval_component_iterator_next(cmp_itr);
+			_comp_collect_var_refs(cmp, vm);
+		}
+		oval_component_iterator_free(cmp_itr);
+		break;
+	default:
+		break;
+	}
+}
+
+static void _var_collect_var_refs(struct oval_variable *var, struct oval_string_map *vm)
+{
+	char *var_id;
+
+	var_id = oval_variable_get_id(var);
+	oval_string_map_put(vm, var_id, var);
+
+	if (oval_variable_get_type(var) == OVAL_VARIABLE_LOCAL) {
+		struct oval_component *comp;
+
+		comp = oval_variable_get_component(var);
+		_comp_collect_var_refs(comp, vm);
+	}
+}
+
+static void _ent_collect_var_refs(struct oval_entity *ent, struct oval_string_map *vm)
+{
+	if (oval_entity_get_varref_type(ent) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+		struct oval_variable *var;
+
+		var = oval_entity_get_variable(ent);
+		_var_collect_var_refs(var, vm);
+	}
+}
+
+static void _ste_collect_var_refs(struct oval_state *ste, struct oval_string_map *vm)
+{
+	struct oval_state_content_iterator *cont_itr;
+
+	cont_itr = oval_state_get_contents(ste);
+	while (oval_state_content_iterator_has_more(cont_itr)) {
+		struct oval_state_content *cont;
+		struct oval_entity *ent;
+
+		cont = oval_state_content_iterator_next(cont_itr);
+		ent = oval_state_content_get_entity(cont);
+		_ent_collect_var_refs(ent, vm);
+	}
+	oval_state_content_iterator_free(cont_itr);
+}
+
+static void _set_collect_var_refs(struct oval_setobject *set, struct oval_string_map *vm)
+{
+	struct oval_setobject_iterator *subset_itr;
+	struct oval_object_iterator *obj_itr;
+	struct oval_filter_iterator *fil_itr;
+
+	switch (oval_setobject_get_type(set)) {
+	case OVAL_SET_AGGREGATE:
+		subset_itr = oval_setobject_get_subsets(set);
+		while (oval_setobject_iterator_has_more(subset_itr)) {
+			struct oval_setobject *subset;
+
+			subset = oval_setobject_iterator_next(subset_itr);
+			_set_collect_var_refs(subset, vm);
+		}
+		oval_setobject_iterator_free(subset_itr);
+		break;
+	case OVAL_SET_COLLECTIVE:
+		obj_itr = oval_setobject_get_objects(set);
+		while (oval_object_iterator_has_more(obj_itr)) {
+			struct oval_object *obj;
+
+			obj = oval_object_iterator_next(obj_itr);
+			_obj_collect_var_refs(obj, vm);
+		}
+		oval_object_iterator_free(obj_itr);
+		fil_itr = oval_setobject_get_filters(set);
+		while (oval_filter_iterator_has_more(fil_itr)) {
+			struct oval_filter *fil;
+			struct oval_state *ste;
+
+			fil = oval_filter_iterator_next(fil_itr);
+			ste = oval_filter_get_state(fil);
+			_ste_collect_var_refs(ste, vm);
+		}
+		oval_filter_iterator_free(fil_itr);
+		break;
+	default:
+		break;
+	}
+}
+
+static void _obj_collect_var_refs(struct oval_object *obj, struct oval_string_map *vm)
+{
+	struct oval_object_content_iterator *cont_itr;
+
+	cont_itr = oval_object_get_object_contents(obj);
+	while (oval_object_content_iterator_has_more(cont_itr)) {
+		struct oval_object_content *cont;
+		struct oval_entity *ent;
+		struct oval_setobject *set;
+
+		cont = oval_object_content_iterator_next(cont_itr);
+
+		switch (oval_object_content_get_type(cont)) {
+		case OVAL_OBJECTCONTENT_ENTITY:
+			ent = oval_object_content_get_entity(cont);
+			_ent_collect_var_refs(ent, vm);
+			break;
+		case OVAL_OBJECTCONTENT_SET:
+			set = oval_object_content_get_setobject(cont);
+			_set_collect_var_refs(set, vm);
+			break;
+		default:
+			break;
+		}
+	}
+	oval_object_content_iterator_free(cont_itr);
+}
+
+static void _syschar_add_bindings(struct oval_syschar *sc, struct oval_string_map *vm)
+{
+	struct oval_iterator *var_itr;
+
+	var_itr = oval_string_map_values(vm);
+	while (oval_collection_iterator_has_more(var_itr)) {
+		struct oval_variable *var;
+		struct oval_value_iterator *val_itr;
+		struct oval_variable_binding *binding;
+
+		var = oval_collection_iterator_next(var_itr);
+		binding = oval_variable_binding_new(var, NULL);
+
+		val_itr = oval_variable_get_values(var);
+		while (oval_value_iterator_has_more(val_itr)) {
+			struct oval_value *val;
+			char *txt;
+
+			val = oval_value_iterator_next(val_itr);
+			txt = oval_value_get_text(val);
+			txt = oscap_strdup(txt);
+			oval_variable_binding_add_value(binding, txt);
+		}
+		oval_value_iterator_free(val_itr);
+
+		oval_syschar_add_variable_binding(sc, binding);
+	}
+	oval_collection_iterator_free(var_itr);
+}
+
+int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *object, int flags, struct oval_syschar **out_syschar)
+{
+	char *oid;
+	struct oval_syschar *sysc;
         oval_subtype_t type;
         oval_ph_t *ph;
+	struct oval_string_map *vm;
+	struct oval_syschar_model *model;
+	int ret;
 
+	oid = oval_object_get_id(object);
+	model = psess->sys_model;
+
+	sysc = oval_syschar_model_get_syschar(model, oid);
+	if (sysc != NULL) {
+		if (out_syschar)
+			*out_syschar = sysc;
+		return 0;
+	}
+
+	sysc = oval_syschar_new(model, object);
         type = oval_object_get_subtype(object);
         ph   = oval_probe_handler_get(psess->ph, type);
 
         if (ph == NULL) {
-                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, "OVAL object not supported");
-                return(NULL);
+                char *msg = "OVAL object not supported.\n";
+
+		dW(msg);
+		oval_syschar_add_new_message(sysc, msg, OVAL_MESSAGE_LEVEL_WARNING);
+		oval_syschar_set_flag(sysc, SYSCHAR_FLAG_NOT_COLLECTED);
+
+		return 1;
         }
 
-        o_sys = NULL;
+	if ((ret = ph->func(type, ph->uptr, PROBE_HANDLER_ACT_EVAL, sysc, flags)) != 0) {
+		return ret;
+	}
 
-        if (ph->func(type, ph->uptr, PROBE_HANDLER_ACT_EVAL, object, &o_sys, flags) != 0) {
-                return(NULL);
-        }
+	if (!(flags & OVAL_PDFLAG_NOREPLY)) {
+		vm = oval_string_map_new();
+		_obj_collect_var_refs(object, vm);
+		_syschar_add_bindings(sysc, vm);
+		oval_string_map_free(vm, NULL);
+	}
 
-        return(o_sys);
+	if (out_syschar)
+		*out_syschar = sysc;
+
+	return 0;
 }
 
-struct oval_sysinfo *oval_probe_sysinfo_query(oval_probe_session_t *sess)
+int oval_probe_query_sysinfo(oval_probe_session_t *sess, struct oval_sysinfo **out_sysinfo)
 {
 	struct oval_sysinfo *sysinf;
         oval_ph_t *ph;
+	int ret;
 
         ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_SYSINFO);
 
         if (ph == NULL) {
                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, "OVAL object not supported");
-                return(NULL);
+		return(-1);
         }
 
         sysinf = NULL;
 
-        if (ph->func(OVAL_SUBTYPE_SYSINFO, ph->uptr, PROBE_HANDLER_ACT_EVAL, NULL, &sysinf, 0) != 0) {
-                return(NULL);
-        }
+	ret = ph->func(OVAL_SUBTYPE_SYSINFO, ph->uptr, PROBE_HANDLER_ACT_EVAL, NULL, &sysinf, 0);
+	if (ret != 0)
+		return(ret);
 
-        return(sysinf);
+	*out_sysinfo = sysinf;
+	return(0);
 }
 
-static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode);
-int oval_probe_session_query_object(oval_probe_session_t *sess, struct oval_object *object);
+static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode);
 
-int oval_probe_session_query_sysinfo(oval_probe_session_t *sess) {
-        struct oval_syschar_model *syschar_model;
-	struct oval_sysinfo *sysinfo;
-
-        sysinfo = oval_probe_sysinfo_query(sess);
-        if (sysinfo == NULL) {
-		/* Report error in this case */
-		return -1;
-	}
-
-        syschar_model = sess->sys_model;
-        oval_syschar_model_set_sysinfo(syschar_model, sysinfo);
-        oval_sysinfo_free(sysinfo);
-	return 0;
-}
-
-int oval_probe_session_query_objects(oval_probe_session_t *sess)
+int oval_probe_query_objects(oval_probe_session_t *sess)
 {
 	struct oval_syschar_model * syschar_model;
 	struct oval_definition_model *definition_model;
-	int ret;
 
 	syschar_model = sess->sys_model;
 	definition_model = oval_syschar_model_get_definition_model(syschar_model);
@@ -211,11 +405,12 @@ int oval_probe_session_query_objects(oval_probe_session_t *sess)
 	if (definition_model) {
 		struct oval_object_iterator *objects = oval_definition_model_get_objects(definition_model);
 		while (oval_object_iterator_has_more(objects)) {
+			int ret;
 			struct oval_object *object = oval_object_iterator_next(objects);
-			ret = oval_probe_session_query_object(sess, object);
-			if( ret != 0 ) {
+			ret = oval_probe_query_object(sess, object, 0, NULL);
+			if (ret != 0) {
 				oval_object_iterator_free(objects);
-				return -1;
+				return ret;
 			}
 		}
 		oval_object_iterator_free(objects);
@@ -223,8 +418,7 @@ int oval_probe_session_query_objects(oval_probe_session_t *sess)
 	return 0;
 }
 
-
-int oval_probe_session_query_definition(oval_probe_session_t *sess, const char *id) {
+int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
 
 	struct oval_syschar_model * syschar_model;
         struct oval_definition_model *definition_model;
@@ -234,12 +428,14 @@ int oval_probe_session_query_definition(oval_probe_session_t *sess, const char *
 	syschar_model = sess->sys_model;
         definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	definition = oval_definition_model_get_definition(definition_model, id);
+	if (definition == NULL)
+		return -1;
 
 	struct oval_criteria_node * cnode = oval_definition_get_criteria(definition);
 	if (cnode == NULL)
 		return -1;
 
-	ret = oval_probe_session_query_criteria(sess, cnode);
+	ret = oval_probe_query_criteria(sess, cnode);
 
 	return ret;
 }
@@ -247,7 +443,7 @@ int oval_probe_session_query_definition(oval_probe_session_t *sess, const char *
 /**
  * @returns 0 on success
  */
-static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode) {
+static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode) {
 	int ret;
 
 	switch (oval_criteria_node_get_type(cnode)) {
@@ -265,7 +461,7 @@ static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct
 		if (object == NULL)
 			return 0;
 		/* probe object */
-		ret = oval_probe_session_query_object(sess, object);
+		ret = oval_probe_query_object(sess, object, 0, NULL);
 		if (ret != 0)
 			return ret;
 		/* probe objects referenced like this: test->state->variable->object */
@@ -276,17 +472,29 @@ static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct
 				struct oval_state_content *content = oval_state_content_iterator_next(contents);
 				struct oval_entity * entity = oval_state_content_get_entity(content);
 				if (oval_entity_get_varref_type(entity) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+					oval_syschar_collection_flag_t flag;
 					struct oval_variable *var = oval_entity_get_variable(entity);
-					ret = oval_probe_session_query_variable(sess, var); 
+
+					ret = oval_probe_query_variable(sess, var);
 					if (ret != 0) {
 						oval_state_content_iterator_free(contents);
 						return ret;
 					}
+
+					flag = oval_variable_get_collection_flag(var);
+					switch (flag) {
+					case SYSCHAR_FLAG_COMPLETE:
+					case SYSCHAR_FLAG_INCOMPLETE:
+						break;
+					default:
+						oval_state_content_iterator_free(contents);
+						return 0;
+					}
 				}
 			}
 			oval_state_content_iterator_free(contents);
 		}
-		return ret;
+		return 0;
 
 		}
 		break;
@@ -301,10 +509,10 @@ static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct
                         struct oval_criteria_node *node;
                         while (oval_criteria_node_iterator_has_more(cnode_it)) {
                                 node = oval_criteria_node_iterator_next(cnode_it);
-                                ret = oval_probe_session_query_criteria(sess, node);
+                                ret = oval_probe_query_criteria(sess, node);
                                 if (ret != 0) {
                                         oval_criteria_node_iterator_free(cnode_it);
-                                        return -1;
+                                        return ret;
                                 }
                         }
                         oval_criteria_node_iterator_free(cnode_it);
@@ -316,7 +524,7 @@ static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct
         case OVAL_NODETYPE_EXTENDDEF:{
                         struct oval_definition *oval_def = oval_criteria_node_get_definition(cnode);
 			struct oval_criteria_node *node =  oval_definition_get_criteria(oval_def);
-                        return oval_probe_session_query_criteria(sess, node);
+                        return oval_probe_query_criteria(sess, node);
                 }
                 break;
         case OVAL_NODETYPE_UNKNOWN:
@@ -326,27 +534,3 @@ static int oval_probe_session_query_criteria(oval_probe_session_t *sess, struct
 	/* we shouldn't get here */
         return -1;
 }
-
-/**
- * @returns 0 on success
- */
-int oval_probe_session_query_object(oval_probe_session_t *sess, struct oval_object *object) {
-
-	char *objid = oval_object_get_id(object);
-	struct oval_syschar_model *sys_model = sess->sys_model;
-	struct oval_syschar *syschar = oval_syschar_model_get_syschar(sys_model, objid);
-
-	if (syschar == NULL) {
-		syschar = oval_probe_object_query(sess, object, 0);
-		if (syschar == NULL) {
-			if(  oscap_err() ) {
-		        	/* does it make sense to continue? */
-				 return -1;
-			}
-                       	syschar = oval_syschar_new(sys_model, object);
-	                oval_syschar_set_flag(syschar, SYSCHAR_FLAG_NOT_COLLECTED);
-		}
-		oval_syschar_model_add_syschar(sys_model, syschar);
-	}
-	return 0;
-}
