@@ -77,11 +77,11 @@ oval_pext_t *oval_pext_new(void)
         pext->do_init = true;
         pthread_mutex_init(&pext->lock, NULL);
 
-#if defined(OVAL_PROBEDIR_ENV)
+//#if defined(OVAL_PROBEDIR_ENV)
         pext->probe_dir = getenv("OVAL_PROBE_DIR");
-#else
-        pext->probe_dir = NULL;
-#endif
+//#else
+//        pext->probe_dir = NULL;
+//#endif
         if (pext->probe_dir == NULL)
                 pext->probe_dir = OVAL_PROBE_DIR;
 
@@ -123,8 +123,9 @@ static void oval_pdtbl_free(oval_pdtbl_t *tbl)
         register size_t i;
 
         for (i = 0; i < tbl->count; ++i) {
-                SEAP_close(tbl->ctx, tbl->memb[i].sd);
-                oscap_free(tbl->memb[i].uri);
+                SEAP_close(tbl->ctx, tbl->memb[i]->sd);
+                oscap_free(tbl->memb[i]->uri);
+		oscap_free(tbl->memb[i]);
         }
 
         oscap_free(tbl->memb);
@@ -134,28 +135,39 @@ static void oval_pdtbl_free(oval_pdtbl_t *tbl)
 	return;
 }
 
-static int oval_pdtbl_pdcmp(const oval_pd_t *a, const oval_pd_t *b)
+static int oval_pdtbl_pdcmp(const oval_pd_t **a, const oval_pd_t **b)
 {
-	return (a->subtype - b->subtype);
+	assume_d(*a != NULL, -1);
+	assume_d(*b != NULL, -1);
+	return ((*a)->subtype - (*b)->subtype);
 }
 
-static int oval_pdtbl_typecmp(oval_subtype_t *a, oval_pd_t *b)
+static int oval_pdtbl_typecmp(oval_subtype_t *a, oval_pd_t **b)
 {
-        return (*a - b->subtype);
+	assume_d(a != NULL, -1);
+	assume_d(*b != NULL, -1);
+        return (*a - (*b)->subtype);
 }
 
-static int oval_pdtbl_add(oval_pdtbl_t * tbl, oval_subtype_t type, int sd, const char *uri)
+static int oval_pdtbl_add(oval_pdtbl_t *tbl, oval_subtype_t type, int sd, const char *uri)
 {
+	oval_pd_t *pd;
+
 	assume_d (tbl != NULL, -1);
 	assume_d (uri != NULL, -1);
 
-	tbl->memb = oscap_realloc(tbl->memb, sizeof(oval_pd_t) * (++tbl->count));
+	pd = oscap_talloc(oval_pd_t);
+	pd->subtype = type;
+	pd->sd      = sd;
+	pd->uri     = strdup(uri);
 
-	tbl->memb[tbl->count - 1].subtype = type;
-	tbl->memb[tbl->count - 1].sd      = sd;
-	tbl->memb[tbl->count - 1].uri     = strdup(uri);
+	tbl->memb = oscap_realloc(tbl->memb, sizeof(oval_pd_t *) * (++tbl->count));
 
-	qsort(tbl->memb, tbl->count, sizeof(oval_pd_t),
+	assume_d(tbl->memb != NULL, -1);
+
+	tbl->memb[tbl->count - 1] = pd;
+
+	qsort(tbl->memb, tbl->count, sizeof(oval_pd_t *),
               (int (*)(const void *, const void *))oval_pdtbl_pdcmp);
 
 	return (0);
@@ -163,8 +175,12 @@ static int oval_pdtbl_add(oval_pdtbl_t * tbl, oval_subtype_t type, int sd, const
 
 static oval_pd_t *oval_pdtbl_get(oval_pdtbl_t * tbl, oval_subtype_t type)
 {
-	return oscap_bfind(tbl->memb, tbl->count, sizeof(oval_pd_t),
-			   &type, (int (*)(void *, void *))oval_pdtbl_typecmp);
+	oval_pd_t **pdp;
+
+	pdp = oscap_bfind(tbl->memb, tbl->count, sizeof(oval_pd_t *),
+			  &type, (int (*)(void *, void *))oval_pdtbl_typecmp);
+
+	return (pdp == NULL ? NULL : *pdp);
 }
 
 /*
@@ -181,13 +197,13 @@ static int oval_probe_cmd_init(oval_pext_t *pext)
 	if (SEAP_cmd_register(pext->pdtbl->ctx, PROBECMD_OBJ_EVAL, SEAP_CMDREG_USEARG,
                               &oval_probe_cmd_obj_eval, (void *)pext) != 0)
         {
-		oscap_dprintf("FAIL: can't register command: %s: errno=%u, %s.\n", "obj_eval", errno, strerror(errno));
+		oscap_dlprintf(DBG_E, "Can't register command: %s: errno=%u, %s.\n", "obj_eval", errno, strerror(errno));
 		return (-1);
 	}
 
 	if (SEAP_cmd_register(pext->pdtbl->ctx, PROBECMD_STE_FETCH, SEAP_CMDREG_USEARG,
 			      &oval_probe_cmd_ste_fetch, (void *)pext) != 0) {
-		oscap_dprintf("FAIL: can't register command: %s: errno=%u, %s.\n", "ste_fetch", errno, strerror(errno));
+		oscap_dlprintf(DBG_E, "Can't register command: %s: errno=%u, %s.\n", "ste_fetch", errno, strerror(errno));
 
 		/* FIXME: unregister the first command */
 
@@ -201,36 +217,47 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 {
 	char *id_str;
 	struct oval_definition_model *defs;
-	struct oval_object *obj;
+	struct oval_object  *obj;
+	struct oval_syschar *res;
 	oval_pext_t *pext = (oval_pext_t *) arg;
+	SEXP_t *ret, *ret_code;
+	int r;
 
         assume_d (sexp != NULL, NULL);
         assume_d (arg  != NULL, NULL);
 
 	if (!SEXP_stringp(sexp)) {
-		oscap_dprintf("FAIL: invalid argument: type=%s\n", SEXP_strtype(sexp));
+		oscap_dlprintf(DBG_E, "Invalid argument: type=%s.\n", SEXP_strtype(sexp));
 		return (NULL);
 	}
 
 	id_str = SEXP_string_cstr(sexp);
 	defs   = oval_syschar_model_get_definition_model(*(pext->model));
 	obj    = oval_definition_model_get_object(defs, id_str);
+	ret    = SEXP_list_new (sexp, NULL);
 
-	oscap_dprintf("get_object: %s\n", id_str);
+	oscap_dlprintf(DBG_I, "Get_object: %s.\n", id_str);
 
 	if (obj == NULL) {
-		oscap_dprintf("FAIL: can't find obj: id=%s\n", id_str);
+		oscap_dlprintf(DBG_E, "Can't find obj: id=%s.\n", id_str);
 		oscap_free(id_str);
 
 		return (NULL);
 	}
 
 	oscap_clearerr();
-	oval_probe_object_query(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY); /* DO NOT PUT THIS INSIDE ASSERT! */
+	r = oval_probe_query_object(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY, &res);
+	if (r < 0)
+		ret_code = SEXP_number_newu((unsigned int) SYSCHAR_FLAG_COMPLETE);
+	else
+		ret_code = SEXP_number_newu((unsigned int) oval_syschar_get_flag(res));
+
+	SEXP_list_add(ret, ret_code);
+	SEXP_free(ret_code);
 
 	if (oscap_err()) {
-		oscap_dprintf("FAIL: obj eval failed: id=%s, err: %d, %d, %s\n",
-		   id_str, oscap_err_family(), oscap_err_code(), oscap_err_desc());
+		oscap_dlprintf(DBG_E, "Failed: id: %s, err: %d, %d, %s.\n",
+			       id_str, oscap_err_family(), oscap_err_code(), oscap_err_desc());
 		oscap_clearerr();
 		oscap_free(id_str);
 
@@ -239,7 +266,7 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 
 	oscap_free(id_str);
 
-	return SEXP_ref(sexp);
+	return (ret);
 }
 
 static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
@@ -249,6 +276,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 	struct oval_state *ste;
 	struct oval_definition_model *definition_model;
 	oval_pext_t *pext = (oval_pext_t *)arg;
+	int ret;
 
         assume_d (sexp != NULL, NULL);
         assume_d (arg  != NULL, NULL);
@@ -262,12 +290,23 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 			ste = oval_definition_model_get_state(definition_model, id_str);
 
 			if (ste == NULL) {
-				oscap_dprintf("FAIL: can't find ste: id=%s\n", id_str);
+				oscap_dlprintf(DBG_E, "Can't find ste: id: %s.\n", id_str);
 				SEXP_list_free(ste_list);
 				oscap_free(id_str);
+
+				return (NULL);
+			}
+
+			ret = oval_state2sexp(pext->sess_ptr, ste, &ste_sexp);
+			if (ret !=0) {
+				oscap_dlprintf(DBG_E, "Failed to convert OVAL state to SEXP, id: %s.\n",
+					       id_str);
+				SEXP_list_free(ste_list);
+				oscap_free(id_str);
+
+				return (NULL);
 			}
 
-			ste_sexp = oval_state2sexp(ste);
 			SEXP_list_add(ste_list, ste_sexp);
                         SEXP_free(ste_sexp);
 
@@ -278,9 +317,10 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 	return (ste_list);
 }
 
-static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj, int noreply)
+static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj, int noreply, SEXP_t **out_sexp)
 {
-	int retry;
+	int retry, ret;
+	bool aborted = false;
 
 	SEAP_msg_t *s_imsg, *s_omsg;
 	SEXP_t *s_oobj;
@@ -299,17 +339,17 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
 
 			if (pd->sd < 0) {
                                 protect_errno {
-                                        oscap_dprintf("Can't connect: %u, %s.\n", errno, strerror(errno));
+                                        oscap_dlprintf(DBG_W, "Can't connect: %u, %s.\n", errno, strerror(errno));
                                 }
 
 				if (++retry <= OVAL_PROBE_MAXRETRY) {
-					oscap_dprintf("connect: retry %u/%u\n", retry, OVAL_PROBE_MAXRETRY);
+					oscap_dlprintf(DBG_I, "Connect: retry %u/%u.\n", retry, OVAL_PROBE_MAXRETRY);
 					continue;
 				} else {
                                         char errbuf[__ERRBUF_SIZE];
 
                                         protect_errno {
-                                                oscap_dprintf("connect: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
+                                                oscap_dlprintf(DBG_E, "Connect: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
                                         }
 
                                         if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
@@ -317,7 +357,7 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
                                         else
                                                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECONN, errbuf);
 
-					return (NULL);
+					return (-1);
 				}
 			}
 		}
@@ -328,28 +368,29 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
 		if (noreply) {
 			if (SEAP_msgattr_set(s_omsg, "no-reply", NULL) != 0) {
                                 protect_errno {
-                                        oscap_dprintf("Can't set no-reply attribute\n");
+                                        oscap_dlprintf(DBG_E, "Can't set no-reply attribute.\n");
                                 }
 
                                 SEAP_msg_free(s_omsg);
                                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBEUNKNOWN, NULL);
 
-				return (NULL);
+				return (-1);
 			}
 		}
 
-		oscap_dprintf("Sending message...\n");
+		oscap_dlprintf(DBG_I, "Sending message.\n");
 
-		if (SEAP_sendmsg(ctx, pd->sd, s_omsg) != 0) {
+		ret = SEAP_sendmsg(ctx, pd->sd, s_omsg);
+		if (ret != 0) {
                         protect_errno {
-                                oscap_dprintf("Can't send message: %u, %s\n", errno, strerror(errno));
+                                oscap_dlprintf(DBG_W, "Can't send message: %u, %s.\n", errno, strerror(errno));
                         }
 
 			if (SEAP_close(ctx, pd->sd) != 0) {
                                 char errbuf[__ERRBUF_SIZE];
 
                                 protect_errno {
-                                        oscap_dprintf("Can't close sd: %u, %s\n", errno, strerror(errno));
+                                        oscap_dlprintf(DBG_E, "Can't close sd: %u, %s.\n", errno, strerror(errno));
                                         SEAP_msg_free(s_omsg);
                                 }
 
@@ -358,19 +399,19 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
                                 else
                                         oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
 
-				return (NULL);
+				return (-1);
 			}
 
 			pd->sd = -1;
 
 			if (++retry <= OVAL_PROBE_MAXRETRY) {
-				oscap_dprintf("send: retry %u/%u\n", retry, OVAL_PROBE_MAXRETRY);
+				oscap_dlprintf(DBG_I, "Send: retry %u/%u.\n", retry, OVAL_PROBE_MAXRETRY);
 				continue;
 			} else {
                                 char errbuf[__ERRBUF_SIZE];
 
                                 protect_errno {
-                                        oscap_dprintf("send: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
+                                        oscap_dlprintf(DBG_E, "Send: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
                                         SEAP_msg_free(s_omsg);
                                 }
 
@@ -379,21 +420,25 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
                                 else
                                         oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
 
-				return (NULL);
+				return (ret);
 			}
 		}
 
-		oscap_dprintf("Waiting for reply...\n");
+		oscap_dlprintf(DBG_I, "Waiting for reply.\n");
 
  recv_retry:
 		s_imsg = NULL;
 
-		if (SEAP_recvmsg(ctx, pd->sd, &s_imsg) != 0) {
+		ret = SEAP_recvmsg(ctx, pd->sd, &s_imsg);
+		if (ret != 0) {
                         protect_errno {
-                                oscap_dprintf("Can't receive message: %u, %s\n", errno, strerror(errno));
+                                oscap_dlprintf(DBG_W, "Can't receive message: %u, %s.\n", errno, strerror(errno));
                         }
 
 			switch (errno) {
+			case ECONNABORTED:
+				aborted = true;
+				break;
 			case ECANCELED:
 				{
 					SEAP_err_t *err = NULL;
@@ -412,7 +457,7 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
                                 char errbuf[__ERRBUF_SIZE];
 
                                 protect_errno {
-                                        oscap_dprintf("Can't close sd: %u, %s\n", errno, strerror(errno));
+                                        oscap_dlprintf(DBG_E, "Can't close sd: %u, %s.\n", errno, strerror(errno));
                                         SEAP_msg_free(s_imsg);
                                         SEAP_msg_free(s_omsg);
                                 }
@@ -422,33 +467,41 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
                                 else
                                         oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
 
-				return (NULL);
+				return (-1);
 			}
 
 			pd->sd = -1;
 
-			if (++retry <= OVAL_PROBE_MAXRETRY) {
-				oscap_dprintf("recv: retry %u/%u\n", retry, OVAL_PROBE_MAXRETRY);
-				continue;
+			if (!aborted) {
+				if (++retry <= OVAL_PROBE_MAXRETRY) {
+					oscap_dlprintf(DBG_I, "Recv: retry %u/%u.\n", retry, OVAL_PROBE_MAXRETRY);
+					continue;
+				} else {
+					char errbuf[__ERRBUF_SIZE];
+
+					protect_errno {
+						oscap_dlprintf(DBG_E, "Recv: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
+						SEAP_msg_free(s_imsg);
+						SEAP_msg_free(s_omsg);
+					}
+
+					if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
+						oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, "Unable to receive a message from probe");
+					else
+						oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, errbuf);
+
+					return (ret);
+				}
 			} else {
-                                char errbuf[__ERRBUF_SIZE];
+				oscap_dlprintf(DBG_I, "Connection was aborted.\n");
+				SEAP_msg_free(s_imsg);
+				SEAP_msg_free(s_omsg);
 
-                                protect_errno {
-                                        oscap_dprintf("recv: retry limit (%u) reached.\n", OVAL_PROBE_MAXRETRY);
-                                        SEAP_msg_free(s_imsg);
-                                        SEAP_msg_free(s_omsg);
-                                }
-
-                                if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, "Unable to receive a message from probe");
-                                else
-                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBERECV, errbuf);
-
-				return (NULL);
+				return (-2);
 			}
 		}
 
-		oscap_dprintf("Message received.\n");
+		oscap_dlprintf(DBG_I, "Message received.\n");
 		break;
 	}
 
@@ -457,7 +510,8 @@ static SEXP_t *oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_i
 	SEAP_msg_free(s_imsg);
 	SEAP_msg_free(s_omsg);
 
-	return (s_oobj);
+	*out_sexp = s_oobj;
+	return (0);
 }
 
 static int oval_pdsc_typecmp(oval_subtype_t *a, oval_pdsc_t *b)
@@ -471,11 +525,12 @@ static oval_pdsc_t *oval_pdsc_lookup(oval_pdsc_t pdsc[], int count, oval_subtype
                            (int (*)(void *, void *))oval_pdsc_typecmp);
 }
 
-static struct oval_sysinfo *oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_syschar_model *model)
+static int oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_syschar_model *model, struct oval_sysinfo **out_sysinf)
 {
 	struct oval_sysinfo *sysinf;
 	struct oval_sysint *ife;
 	SEXP_t *s_obj, *s_sinf, *ent, *r0, *r1;
+	int ret;
 
 	/*
 	 * Prepare a dummy object. We can't simply send an empty object
@@ -495,14 +550,19 @@ static struct oval_sysinfo *oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd,
                 SEXP_free (r0);
         }
 
-        r0 = oval_probe_comm(ctx, pd, s_obj, 0);
+        ret = oval_probe_comm(ctx, pd, s_obj, 0, &r0);
         SEXP_free(s_obj);
 
-	if (r0 == NULL)
-		return (NULL);
+	if (ret != 0)
+		return (ret);
+
+	r1 = probe_cobj_get_items(r0);
+	s_sinf = SEXP_list_first(r1);
+	SEXP_vfree(r0, r1, NULL);
+
+	if (s_sinf == NULL)
+		return (-1);
 
-	s_sinf = _probe_cobj_get_items(r0);
-	SEXP_free(r0);
 	sysinf = oval_sysinfo_new(model);
 
 	/*
@@ -532,7 +592,7 @@ static struct oval_sysinfo *oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd,
 
 	if (s_sinf == NULL) {
 		oval_sysinfo_free(sysinf);
-		return (NULL);
+		return (-1);
 	}
 
 	SYSINF_EXT(s_sinf, os_name, sysinf, fail_gen);
@@ -584,7 +644,8 @@ static struct oval_sysinfo *oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd,
 
 	SEXP_free(s_sinf);
 
-	return (sysinf);
+	*out_sysinf = sysinf;
+	return (0);
  fail_int:
 	SEXP_free(ent);
 	oval_sysint_free(ife);
@@ -592,7 +653,7 @@ static struct oval_sysinfo *oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd,
 	SEXP_free(s_sinf);
 	oval_sysinfo_free(sysinf);
 
-	return (NULL);
+	return (-1);
 }
 
 int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
@@ -623,8 +684,7 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
                 }
 
                 assume_r(pd != NULL, -1);
-                *inf = oval_probe_sys_eval(pext->pdtbl->ctx, pd, *(pext->model));
-                ret  = (*inf == NULL ? -1 : 0);
+		ret = oval_probe_sys_eval(pext->pdtbl->ctx, pd, *(pext->model), inf);
                 break;
         }
         case PROBE_HANDLER_ACT_OPEN:
@@ -639,7 +699,7 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
                 probe_urilen = snprintf(probe_uri, sizeof probe_uri,
                                         "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
-                oscap_dprintf("URI: %s\n", probe_uri);
+                oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
                 if (oval_pdtbl_add(pext->pdtbl, type, -1, probe_uri) != 0) {
                         char errmsg[__ERRBUF_SIZE];
@@ -676,10 +736,13 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
         switch(act) {
         case PROBE_HANDLER_ACT_EVAL:
         {
-                struct oval_object   *obj = va_arg(ap, struct oval_object *);
-                struct oval_syschar **sys = va_arg(ap, struct oval_syschar **);
-                int flags = va_arg(ap, int);
+		struct oval_object *obj;
+		struct oval_syschar *sys;
+		int flags;
 
+		sys = va_arg(ap, struct oval_syschar *);
+		flags = va_arg(ap, int);
+		obj = oval_syschar_get_object(sys);
                 pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
 
                 if (pd == NULL) {
@@ -693,7 +756,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                         probe_urilen = snprintf(probe_uri, sizeof probe_uri,
                                                 "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
-                        oscap_dprintf("URI: %s\n", probe_uri);
+                        oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
                         if (oval_pdtbl_add(pext->pdtbl, oval_object_get_subtype(obj), -1, probe_uri) != 0) {
                                 char errmsg[__ERRBUF_SIZE];
@@ -702,13 +765,31 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
 
                                 return (-1);
-                        } else
-                                pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
+			}
+			pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
                 }
 
-                *sys = oval_probe_ext_eval(pext->pdtbl->ctx, pd, pext, obj, flags);
-                ret  = (*sys == NULL ? -1 : 0);
-                break;
+		ret = oval_probe_ext_eval(pext->pdtbl->ctx, pd, pext, sys, flags);
+
+		if (ret >= 0)
+			ret = 0;
+
+		if (ret < 0 && errno == ECONNABORTED) {
+			if (!pext->do_init) {
+				oval_pdtbl_free(pext->pdtbl);
+			}
+
+			pext->do_init  = true;
+			pext->pdtbl    = NULL;
+			pext->pdsc     = NULL;
+			pext->pdsc_cnt = 0;
+
+			oval_probe_ext_init(pext);
+
+			errno = ECONNABORTED;
+		}
+
+		return ret;
         }
         case PROBE_HANDLER_ACT_OPEN:
                 break;
@@ -716,23 +797,31 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                 ret = oval_probe_ext_init(pext);
                 break;
         case PROBE_HANDLER_ACT_RESET:
+	case PROBE_HANDLER_ACT_ABORT:
         {
                 size_t i;
 
                 if (type == OVAL_SUBTYPE_ALL) {
                         /*
-                         * Iterate trhu probe descriptor table and execute the reset operation
+                         * Iterate thru probe descriptor table and execute the reset operation
                          * for each probe descriptor.
                          */
                         for (i = 0; i < pext->pdtbl->count; ++i) {
-                                pd  = pext->pdtbl->memb + i;
+                                pd  = pext->pdtbl->memb[i];
 
                                 //fprintf(stderr, "Sending reset to %p(%s)...\n", pd, oval_subtype2str(pd->subtype));
 
-                                ret = oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
 
-                                if (ret != 0)
-                                        return(ret);
+				if (pd == NULL)
+					return(0);
+
+				if (act == PROBE_HANDLER_ACT_RESET)
+					ret = oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
+				else
+					ret = oval_probe_ext_abort(pext->pdtbl->ctx, pd, pext);
+
+				if (ret != 0)
+					return (ret);
                         }
 
                         return(0);
@@ -745,7 +834,10 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                         if (pd == NULL)
                                 return(0);
 
-                        return oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
+			if (act == PROBE_HANDLER_ACT_RESET)
+				return oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
+			else
+				return oval_probe_ext_abort(pext->pdtbl->ctx, pd, pext);
                 }
                 break;
         }
@@ -781,67 +873,57 @@ int oval_probe_ext_init(oval_pext_t *pext)
         return(ret);
 }
 
-struct oval_syschar *oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struct oval_object *object, int flags)
+int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struct oval_syschar *syschar, int flags)
 {
         SEXP_t *s_obj, *s_sys;
-        struct oval_syschar *o_sys;
-        struct oval_syschar_model *model = *(pext->model);
+	struct oval_object *object;
+	int ret;
 
-        s_obj = oval_object2sexp(oval_subtype2str(oval_object_get_subtype(object)), object, model, pext->sess_ptr);
+	if (syschar == NULL) {
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBEINVAL, "Internal error: syschar == NULL");
+		return (-1);
+	}
 
-        if (s_obj == NULL) {
-                oscap_dprintf("Can't translate OVAL object to S-exp\n");
-                oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBEOBJINVAL, "Can't translate OVAL object to S-exp");
+	object = oval_syschar_get_object(syschar);
+	ret = oval_object2sexp(pext->sess_ptr, oval_subtype2str(oval_object_get_subtype(object)), syschar, &s_obj);
 
-                return(NULL);
-        }
+	if (ret != 0)
+		return (1);
 
-        s_sys = oval_probe_comm(ctx, pd, s_obj, flags & OVAL_PDFLAG_NOREPLY);
+	ret = oval_probe_comm(ctx, pd, s_obj, flags & OVAL_PDFLAG_NOREPLY, &s_sys);
+	SEXP_free(s_obj);
 
-        if (s_sys == NULL) {
-                SEXP_free(s_obj);
+	if (ret != 0) {
+		switch (errno) {
+		case ECONNABORTED:
+			dI("Closing sd=%d (pd=%p) after abort\n", pd->sd, pd);
 
-                if (flags & OVAL_PDFLAG_NOREPLY) {
-                        /*
-                         * NULL is ok here because the no-reply flag is set and we don't expect
-                         * any data to be returned.
-                         */
-                        return(NULL);
-                } else {
-                        if (!oscap_err()) {
-                                /*
-                                 * oval_probe_comm didn't set an error so we have to do it here.
-                                 */
-                                oscap_dprintf("oval_probe_comm failed but didn't set an error!\n");
-                                oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBENODATA, "No data received");
-                        }
+			SEAP_close(ctx, pd->sd);
+			pd->sd = -1;
+			errno  = ECONNABORTED;
+		}
+		return (ret);
+	}
 
-                        return(NULL);
-                }
-        } else {
-                if (flags & OVAL_PDFLAG_NOREPLY) {
+	if (flags & OVAL_PDFLAG_NOREPLY) {
+		if (s_sys != NULL) {
                         /*
                          * The no-reply flag is set and oval_probe_comm returned some
-                         * data. This considered as a non-fatal error.
+                         * data. This is considered a non-fatal error.
                          */
-                        oscap_dprintf ("obtrusive data from probe!\n");
-
+                        oscap_dlprintf(DBG_W, "Obtrusive data from probe!\n");
                         SEXP_free(s_sys);
-			SEXP_free(s_obj);
-
-			return (NULL);
-                }
-        }
+		}
+		return (0);
+	}
 
         /*
 	 * Convert the received S-exp to OVAL system characteristic.
 	 */
-	o_sys = oval_sexp2sysch(s_sys, model, object);
-
+	ret = oval_sexp2sysch(s_sys, syschar);
 	SEXP_free(s_sys);
-	SEXP_free(s_obj);
 
-	return (o_sys);
+	return (ret);
 }
 
 int oval_probe_ext_reset(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
@@ -852,3 +934,43 @@ int oval_probe_ext_reset(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
 
         return (0);
 }
+
+#include <signal.h>
+#include "SEAP/_seap-types.h"
+#include "SEAP/seap-descriptor.h"
+#include "SEAP/_seap-scheme.h"
+#include "SEAP/sch_pipe.h"
+
+int oval_probe_ext_abort(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
+{
+	SEAP_desc_t *dsc;
+	/*
+	 * Send SIGUSR1 to the probe
+	 */
+
+	assume_d(ctx  != NULL, -1);
+	assume_d(pd   != NULL, -1);
+	assume_d(pext != NULL, -1);
+
+	dI("Sending abort to sd=%d\n", pd->sd);
+
+	dsc = SEAP_desc_get(ctx->sd_table, pd->sd);
+
+	if (dsc == NULL)
+		return (-1);
+
+	switch (dsc->scheme) {
+	case SCH_PIPE:
+	{
+		sch_pipedata_t *pipeinfo = (sch_pipedata_t *)dsc->scheme_data;
+
+		dI("Sending SIGUSR1 to pid=%u\n", pipeinfo->pid);
+		kill(pipeinfo->pid, SIGUSR1);
+		break;
+	}
+	default:
+		return (-1);
+	}
+
+	return (0);
+}
