@@ -22,6 +22,7 @@
 #include <errno.h>
 #include <string.h>
 #include "common/assume.h"
+#include "common/debug_priv.h"
 #include "oval_sexp.h"
 #include "oval_probe_impl.h"
 #include "oval_definitions_impl.h"
@@ -61,58 +62,90 @@ static struct oval_value *oval_object_getentval(struct oval_object *obj, const c
         return(val);
 }
 
-static struct oval_syschar *oval_probe_envvar_eval(struct oval_object *obj, struct oval_syschar_model *model)
+static struct oval_variable *oval_probe_variable_objgetvar(struct oval_object *obj)
 {
-        struct oval_syschar *sys;
-        struct oval_value   *val;
-        char *var_name, *var_value;
+        struct oval_entity  *ent;
+        struct oval_object_content *con;
+        struct oval_object_content_iterator *cit;
+	struct oval_variable *var;
+        char *ent_name;
 
+	var = NULL;
+        cit = oval_object_get_object_contents(obj);
+
+	while (oval_object_content_iterator_has_more(cit)) {
+                con = oval_object_content_iterator_next(cit);
+
+                if (oval_object_content_get_type(con) != OVAL_OBJECTCONTENT_ENTITY)
+                        continue;
+
+                ent = oval_object_content_get_entity(con);
+                ent_name = oval_entity_get_name(ent);
+
+                if (strcmp(ent_name, "var_ref") != 0)
+                        continue;
+
+                var = oval_entity_get_variable(ent);
+                break;
+        }
+
+        oval_object_content_iterator_free(cit);
+
+        return(var);
+}
+
+static int oval_probe_envvar_eval(struct oval_syschar *syschar)
+{
+	struct oval_object *obj;
+	struct oval_value *val;
+        char *var_name, *var_value;
+	SEXP_t *r0, *r1, *r2, *cobj;
+	int ret;
+
+	obj = oval_syschar_get_object(syschar);
         val = oval_object_getentval(obj, "name");
 
         if (val == NULL)
-                return(NULL);
+		return(-1);
 
         if (oval_value_get_datatype(val) != OVAL_DATATYPE_STRING)
-                return(NULL);
+		return(-1);
 
         var_name  = oval_value_get_text(val);
         var_value = getenv(var_name);
+	cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
 
-        if (var_value == NULL)
-                sys = oval_syschar_new(model, obj);
-        else {
-                SEXP_t *items, *r0, *r1, *r2, *cobj;
+	if (var_value != NULL) {
+		r0 = probe_item_creat("environmentvariable_item", NULL,
+				      "name",  NULL, r1 = SEXP_string_new(var_name,  strlen(var_name)),
+				      "value", NULL, r2 = SEXP_string_new(var_value, strlen(var_value)),
+				      NULL);
+		probe_cobj_add_item(cobj, r0);
+		SEXP_vfree (r0, r1, r2, NULL);
+	}
 
-                items = SEXP_list_new(r0= probe_item_creat("environmentvariable_item", NULL,
-                                                           "name",  NULL, r1 = SEXP_string_new(var_name,  strlen(var_name)),
-                                                           "value", NULL, r2 = SEXP_string_new(var_value, strlen(var_value)),
-                                                           NULL),
-                                      NULL);
-                cobj = _probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, items);
-                sys  = oval_sexp2sysch(cobj, model, obj);
-                SEXP_vfree(r0, r1, r2, items, cobj, NULL);
-        }
+	probe_cobj_compute_flag(cobj);
+	ret = oval_sexp2sysch(cobj, syschar);
+	SEXP_vfree(cobj, NULL);
 
-        return(sys);
+	return(ret);
 }
 
 int oval_probe_envvar_handler(oval_subtype_t type, void *ptr, int act, ...)
 {
         int ret = 0;
         va_list ap;
-        struct oval_syschar_model *model = (struct oval_syschar_model *)ptr;
 
         va_start(ap, act);
 
         switch(act) {
         case PROBE_HANDLER_ACT_EVAL:
         {
-                struct oval_object   *obj = va_arg(ap, struct oval_object *);
-                struct oval_syschar **sys = va_arg(ap, struct oval_syschar **);
+		struct oval_syschar *sys;
 
-                va_arg(ap, int);
-                *sys = oval_probe_envvar_eval(obj, model);
-                ret  = (*sys == NULL ? -1 : 0);
+		sys = va_arg(ap, struct oval_syschar *);
+		va_arg(ap, int);
+		ret = oval_probe_envvar_eval(sys);
                 break;
         }
         case PROBE_HANDLER_ACT_INIT:
@@ -130,74 +163,92 @@ int oval_probe_envvar_handler(oval_subtype_t type, void *ptr, int act, ...)
         return(ret);
 }
 
-static struct oval_syschar *oval_probe_variable_eval(struct oval_object *obj, oval_probe_session_t *sess)
+static int oval_probe_variable_eval(oval_probe_session_t *sess, struct oval_syschar *syschar)
 {
         struct oval_value    *val;
-        struct oval_syschar  *sys;
         struct oval_value_iterator *vit;
         struct oval_variable *var;
-        struct oval_definition_model *def_model;
-        struct oval_syschar_model    *sys_model;
-        char *var_ref;
+	struct oval_object *obj;
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
+	int ret = 0;
 
-        val = oval_object_getentval(obj, "var_ref");
+	obj = oval_syschar_get_object(syschar);
+	var = oval_probe_variable_objgetvar(obj);
+	if (var == NULL) {
+		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_ERROR);
+		return(-1);
+	}
 
-        if (val == NULL)
-                return(NULL);
+	if (oval_probe_query_variable(sess, var) != 0) {
+		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_ERROR);
+		return(-1);
+	}
 
-        if (oval_value_get_datatype(val) != OVAL_DATATYPE_STRING)
-                return(NULL);
+	flag = oval_variable_get_collection_flag(var);
+	switch (flag) {
+	case SYSCHAR_FLAG_COMPLETE:
+	case SYSCHAR_FLAG_INCOMPLETE:
+		break;
+	default:
+	{
+		char msg[100];
 
-        var_ref = oval_value_get_text(val);
-        assume_d(var_ref != NULL, -1);
-
-        sys_model = oval_probe_session_getmodel(sess);
-        def_model = oval_syschar_model_get_definition_model(sys_model);
-        var = oval_definition_model_get_variable(def_model, var_ref);
-
-        if (oval_probe_session_query_variable(sess, var) != 0)
-                return oval_syschar_new(sys_model, obj);
+		snprintf(msg, sizeof(msg), "There was a problem processing referenced variable (%s).\n",
+			 oval_variable_get_id(var));
+		dW(msg);
+		oval_syschar_add_new_message(syschar, msg, OVAL_MESSAGE_LEVEL_WARNING);
+		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_DOES_NOT_EXIST);
+		return(1);
+	}
+	}
 
         vit = oval_variable_get_values(var);
 
-        if (vit != NULL)
-                sys = oval_syschar_new(sys_model, obj);
-        else {
-                SEXP_t *items, *r0, *item, *cobj, *vrent, *val_sexp;
+	if (vit == NULL) {
+		flag = SYSCHAR_FLAG_ERROR;
+		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_ERROR);
+		return(1);
+	} else {
+                SEXP_t *r0, *item, *cobj, *vrent, *val_sexp;
+		char *var_ref;
 
-                items = SEXP_list_new(NULL);
+                cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
 
                 /* Create shared entity */
+		var_ref = oval_variable_get_id(var);
                 vrent = probe_ent_creat1("var_ref", NULL,
                                          r0 = SEXP_string_new(var_ref, strlen(var_ref)));
                 SEXP_free(r0);
 
                 while (oval_value_iterator_has_more(vit)) {
+			oval_datatype_t dtype;
+
                         val = oval_value_iterator_next(vit);
 
-                        if (oval_value_get_datatype(val) == OVAL_DATATYPE_STRING) {
-                                val_sexp = oval_value_to_sexp(val, OVAL_DATATYPE_STRING);
-                                assume_d(val_sexp != NULL, -1);
+			oval_value_cast(val, OVAL_DATATYPE_STRING);
 
-                                item = probe_item_creat("variable_item", NULL,
-                                                        "value", NULL, val_sexp,
-                                                        NULL);
-                                /* Add shared entity */
-                                SEXP_list_add(item, vrent);
+			dtype = oval_value_get_datatype(val);
+			val_sexp = oval_value_to_sexp(val, dtype);
+			assume_d(val_sexp != NULL, -1);
 
-                                /* Add item to the item list */
-                                SEXP_list_add(items, item);
-                                SEXP_vfree(item, val_sexp, NULL);
-                        }
+			item = probe_item_creat("variable_item", NULL,
+						"value", NULL, val_sexp,
+						NULL);
+			/* Add shared entity */
+			SEXP_list_add(item, vrent);
+
+			/* Add item to the item list */
+			probe_cobj_add_item(cobj, item);
+			SEXP_vfree(item, val_sexp, NULL);
                 }
 
                 oval_value_iterator_free(vit);
-                cobj = _probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, items);
-                sys  = oval_sexp2sysch(cobj, sys_model, obj);
-                SEXP_vfree(cobj, items, vrent, NULL);
+		probe_cobj_compute_flag(cobj);
+		ret = oval_sexp2sysch(cobj, syschar);
+                SEXP_vfree(cobj, vrent, NULL);
         }
 
-        return(sys);
+	return(ret);
 }
 
 int oval_probe_var_handler(oval_subtype_t type, void *ptr, int act, ...)
@@ -211,12 +262,11 @@ int oval_probe_var_handler(oval_subtype_t type, void *ptr, int act, ...)
         switch(act) {
         case PROBE_HANDLER_ACT_EVAL:
         {
-                struct oval_object   *obj = va_arg(ap, struct oval_object *);
-                struct oval_syschar **sys = va_arg(ap, struct oval_syschar **);
+		struct oval_syschar *sys;
 
+		sys = va_arg(ap, struct oval_syschar *);
                 va_arg(ap, int);
-                *sys = oval_probe_variable_eval(obj, sess);
-                ret  = (*sys == NULL ? -1 : 0);
+		ret = oval_probe_variable_eval(sess, sys);
                 break;
         }
         case PROBE_HANDLER_ACT_INIT:
