@@ -95,7 +95,14 @@ static bool _oval_result_system_resolve_syschar(struct oval_syschar *syschar, st
 struct oval_result_system *oval_result_system_new(struct oval_results_model *model,
 						  struct oval_syschar_model *syschar_model)
 {
-	oval_result_system_t *sys = (oval_result_system_t *) oscap_alloc(sizeof(oval_result_system_t));
+	oval_result_system_t *sys;
+
+        if (model && oval_results_model_is_locked(model)) {
+                oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+                return NULL;
+        }
+
+	sys = (oval_result_system_t *) oscap_alloc(sizeof(oval_result_system_t));
 	if (sys == NULL)
 		return NULL;
 
@@ -104,6 +111,9 @@ struct oval_result_system *oval_result_system_new(struct oval_results_model *mod
 	sys->syschar_model = syschar_model;
 	sys->definitions_initialized = false;
 	sys->model = model;
+
+	oval_results_model_add_system(model, sys);
+
 	return sys;
 }
 
@@ -115,7 +125,7 @@ bool oval_result_system_is_valid(struct oval_result_system * result_system)
 	struct oval_syschar_model *syschar_model;
 
 	if (result_system == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
         }
 
@@ -337,7 +347,7 @@ void oval_result_system_add_definition(struct oval_result_system *sys, struct ov
 			oval_string_map_put(sys->definitions, id, definition);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 void oval_result_system_add_test(struct oval_result_system *sys, struct oval_result_test *test) {
@@ -348,7 +358,7 @@ void oval_result_system_add_test(struct oval_result_system *sys, struct oval_res
 			oval_string_map_put(sys->tests, id, test);
 		}
 	} else
-		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
 }
 
 static void _oval_result_system_test_consume(struct oval_result_test *test, struct oval_result_system *sys) {
@@ -378,7 +388,7 @@ static int _oval_result_system_parse
 	__attribute__nonnull__(sys);
 
 	xmlChar *localName = xmlTextReaderLocalName(reader);
-	oscap_dprintf("DEBUG: _oval_result_system_parse: parse <%s>", localName);
+	oscap_dlprintf(DBG_I, "Parsing <%s>.\n", localName);
 	int return_code = 0;
 	if (strcmp((const char *)localName, "definitions") == 0) {
 		return_code = oval_parser_parse_tag
@@ -391,7 +401,7 @@ static int _oval_result_system_parse
 		//return_code = oval_parser_skip_tag(reader, context);
 	} else {
 		return_code = 0;
-		oscap_dprintf("WARNING: _oval_result_system_parse: TODO: <%s> not handled", localName);
+		oscap_dlprintf(DBG_W, "Unhandled tag: <%s>.\n", localName);
 	}
 	oscap_free(localName);
 	return return_code;
@@ -446,7 +456,7 @@ oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys,
 	definition_model = oval_results_model_get_definition_model(res_model);
 	oval_definition = oval_definition_model_get_definition(definition_model, id);
 	if (oval_definition == NULL) {
-		oscap_dprintf("%s:%d No definition with ID (%s) in definition model.", __FILE__, __LINE__, id);
+		oscap_dlprintf(DBG_E, "No definition with ID: %s in definition model.\n", id);
 		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "No definition with such an ID in definition model.");
 		return OVAL_RESULT_ERROR;
 	}
@@ -466,11 +476,11 @@ xmlNode *oval_result_system_to_dom
      struct oval_results_model * results_model,
      struct oval_result_directives * directives, xmlDocPtr doc, xmlNode * parent) {
 	xmlNs *ns_results = xmlSearchNsByHref(doc, parent, OVAL_RESULTS_NAMESPACE);
-	xmlNode *system_node = xmlNewChild(parent, ns_results, BAD_CAST "system", NULL);
+	xmlNode *system_node = xmlNewTextChild(parent, ns_results, BAD_CAST "system", NULL);
 
 	struct oval_string_map *tstmap = oval_string_map_new();
 
-	xmlNode *definitions_node = xmlNewChild(system_node, ns_results, BAD_CAST "definitions", NULL);
+	xmlNode *definitions_node = xmlNewTextChild(system_node, ns_results, BAD_CAST "definitions", NULL);
 	struct oval_definition_model *definition_model = oval_results_model_get_definition_model(results_model);
 	struct oval_definition_iterator *oval_definitions = oval_definition_model_get_definitions(definition_model);
 	while(oval_definition_iterator_has_more(oval_definitions)) {
@@ -504,7 +514,7 @@ xmlNode *oval_result_system_to_dom
 
 	struct oval_result_test_iterator *result_tests = (struct oval_result_test_iterator *)  oval_string_map_values(tstmap);
 	if (oval_result_test_iterator_has_more(result_tests)) {
-		xmlNode *tests_node = xmlNewChild(system_node, ns_results, BAD_CAST "tests", NULL);
+		xmlNode *tests_node = xmlNewTextChild(system_node, ns_results, BAD_CAST "tests", NULL);
 		while (oval_result_test_iterator_has_more(result_tests)) {
 			struct oval_result_test *result_test = oval_result_test_iterator_next(result_tests);
 			/* report the test */
@@ -694,10 +704,14 @@ static void _oval_result_system_scan_set_for_references
 			}
 		}
 	oval_object_iterator_free(objects);
-	struct oval_state_iterator *states = oval_setobject_get_filters(set);
-	if (states)
-		while (oval_state_iterator_has_more(states)) {
-			struct oval_state *state = oval_state_iterator_next(states);
+	struct oval_filter_iterator *filters = oval_setobject_get_filters(set);
+	if (filters)
+		while (oval_filter_iterator_has_more(filters)) {
+			struct oval_filter *filter;
+			struct oval_state *state;
+
+			filter = oval_filter_iterator_next(filters);
+			state = oval_filter_get_state(filter);
 			char *sttid = oval_state_get_id(state);
 			void *value = oval_string_map_get_value(sttmap, sttid);
 			if (value == NULL) {
@@ -706,7 +720,7 @@ static void _oval_result_system_scan_set_for_references
 									      varmap, sysmap);
 			}
 		}
-	oval_state_iterator_free(states);
+	oval_filter_iterator_free(filters);
 	struct oval_setobject_iterator *subsets = oval_setobject_get_subsets(set);
 	if (subsets)
 		while (oval_setobject_iterator_has_more(subsets)) {
