@@ -29,6 +29,7 @@
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  *      "Lukas Kuklinek" <lkuklinek@redhat.com>
+ *      "Tomas Heinrich" <theinric@redhat.com>
  */
 #include <inttypes.h>
 #include <stdlib.h>
@@ -41,6 +42,8 @@
 #include "probes/public/probe-api.h"
 #include "oval_definitions_impl.h"
 #include "oval_system_characteristics_impl.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 #ifndef _A
 # define _A(x) assert(x)
@@ -87,7 +90,17 @@ SEXP_t *oval_value_to_sexp(struct oval_value *val, oval_datatype_t dtype)
 	case OVAL_DATATYPE_BOOLEAN:
 		val_sexp = SEXP_number_newb(oval_value_get_boolean(val));
 		break;
+	case OVAL_DATATYPE_BINARY:
+	case OVAL_DATATYPE_FILESET_REVISION:
+	case OVAL_DATATYPE_IOS_VERSION:
+		// todo:
+		oscap_dlprintf(DBG_E, "Unsupported datatype: %s.\n", dtype);
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unsupported datatype");
+		val_sexp = NULL;
+		break;
 	default:
+		oscap_dlprintf(DBG_E, "Unknown datatype: %s.\n", dtype);
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unknown datatype");
 		val_sexp = NULL;
 		break;
 	}
@@ -143,7 +156,7 @@ static SEXP_t *oval_entity_to_sexp(struct oval_entity *ent)
 	return (elm);
 }
 
-static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_syschar_model *syschar_model, void *sess)
+static int oval_varref_to_sexp(void *sess, struct oval_entity *entity, struct oval_syschar *syschar, SEXP_t **out_sexp)
 {
 	unsigned int val_cnt = 0;
 	SEXP_t *val_lst, *val_sexp, *varref, *id_sexp, *val_cnt_sexp;
@@ -151,25 +164,57 @@ static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_sysch
 	struct oval_variable *var;
 	struct oval_value_iterator *vit;
 	struct oval_value *val;
+	oval_syschar_collection_flag_t flag;
+	char msg[100];
+	int ret = 0;
 
-	val_lst = SEXP_list_new(NULL);
 	dt = oval_entity_get_datatype(entity);
 
 	var = oval_entity_get_variable(entity);
-	if (oval_probe_session_query_variable(sess, var) != 0) {
-		SEXP_free(val_lst);
-		return NULL;
+	if (oval_probe_query_variable(sess, var) != 0) {
+		dE("Can't convert variable reference to SEXP.\n");
+		return -1;
 	}
-	vit = oval_variable_get_values(var);
+
+	flag = oval_variable_get_collection_flag(var);
+	switch (flag) {
+	case SYSCHAR_FLAG_COMPLETE:
+	case SYSCHAR_FLAG_INCOMPLETE:
+		vit = oval_variable_get_values(var);
+		if (oval_value_iterator_has_more(vit))
+			break;
+		/* fall through */
+	case SYSCHAR_FLAG_DOES_NOT_EXIST:
+		snprintf(msg, sizeof(msg), "Referenced variable has no values (%s).\n",
+			 oval_variable_get_id(var));
+		ret = 1;
+		break;
+	default:
+		snprintf(msg, sizeof(msg), "There was a problem processing referenced variable (%s).\n",
+			 oval_variable_get_id(var));
+		ret = 1;
+	}
+
+	if (ret) {
+		dW(msg);
+		oval_syschar_add_new_message(syschar, msg, OVAL_MESSAGE_LEVEL_WARNING);
+		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_DOES_NOT_EXIST);
+		return ret;
+	}
+
+	val_lst = SEXP_list_new(NULL);
 
 	while (oval_value_iterator_has_more(vit)) {
 		val = oval_value_iterator_next(vit);
 
 		val_sexp = oval_value_to_sexp(val, dt);
 		if (val_sexp == NULL) {
+			oval_syschar_add_new_message(syschar, "Failed to convert variable value.\n",
+						     OVAL_MESSAGE_LEVEL_ERROR);
+			oval_syschar_set_flag(syschar, SYSCHAR_FLAG_ERROR);
 			SEXP_free(val_lst);
 			oval_value_iterator_free(vit);
-			return NULL;
+			return -1;
 		}
 
 		SEXP_list_add(val_lst, val_sexp);
@@ -187,7 +232,27 @@ static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_sysch
 	SEXP_free(val_cnt_sexp);
 	SEXP_free(val_lst);
 
-	return varref;
+	*out_sexp = varref;
+	return 0;
+}
+
+static SEXP_t *oval_filter_to_sexp(struct oval_filter *filter)
+{
+	SEXP_t *elm, *attr, *r0, *r1;
+	oval_filter_action_t act;
+	struct oval_state *ste;
+	char *ste_id;
+
+	act = oval_filter_get_filter_action(filter);
+	ste = oval_filter_get_state(filter);
+	ste_id = oval_state_get_id(ste);
+	attr = probe_attr_creat("action", r0 = SEXP_number_newu(act), NULL);
+	elm = probe_ent_creat1("filter",
+			       attr,
+			       r1 = SEXP_string_newf("%s", ste_id));
+	SEXP_vfree(attr, r0, r1, NULL);
+
+	return (elm);
 }
 
 static SEXP_t *oval_set_to_sexp(struct oval_setobject *set)
@@ -225,14 +290,11 @@ static SEXP_t *oval_set_to_sexp(struct oval_setobject *set)
 		break;
 	case OVAL_SET_COLLECTIVE:{
 			struct oval_object_iterator *oit;
-			struct oval_state_iterator *sit;
+			struct oval_filter_iterator *fit;
 			struct oval_object *obj;
-			struct oval_state *ste;
 			SEXP_t *subelm;
 
 			oit = oval_setobject_get_objects(set);
-			sit = oval_setobject_get_filters(set);
-
 			while (oval_object_iterator_has_more(oit)) {
 				obj = oval_object_iterator_next(oit);
 
@@ -245,23 +307,18 @@ static SEXP_t *oval_set_to_sexp(struct oval_setobject *set)
 
 				SEXP_free(subelm);
 			}
-
 			oval_object_iterator_free(oit);
 
-			while (oval_state_iterator_has_more(sit)) {
-				ste = oval_state_iterator_next(sit);
-
-				subelm = SEXP_list_new(r0 = SEXP_string_new("filter", 6),
-						       r1 = SEXP_string_newf("%s", oval_state_get_id(ste)), NULL);
-				SEXP_free(r0);
-				SEXP_free(r1);
+			fit = oval_setobject_get_filters(set);
+			while (oval_filter_iterator_has_more(fit)) {
+				struct oval_filter *fil;
 
+				fil = oval_filter_iterator_next(fit);
+				subelm = oval_filter_to_sexp(fil);
 				SEXP_list_add(elm, subelm);
-
 				SEXP_free(subelm);
 			}
-
-			oval_state_iterator_free(sit);
+			oval_filter_iterator_free(fit);
 		}
 		break;
 	default:
@@ -302,17 +359,21 @@ static SEXP_t *oval_behaviors_to_sexp(struct oval_behavior_iterator *bit)
 	return (r0);
 }
 
-SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct oval_syschar_model *syschar_model, void *sess)
+int oval_object2sexp(void *sess, const char *typestr, struct oval_syschar *syschar, SEXP_t **out_sexp)
 {
 	unsigned int ent_cnt, varref_cnt;
+	int ret = -1;
 	SEXP_t *obj_sexp, *obj_name, *elm, *varrefs, *ent_lst, *lst, *stmp;
 	SEXP_t *r0, *r1, *r2;
 
+	struct oval_object *object;
 	struct oval_object_content_iterator *cit;
 	struct oval_behavior_iterator *bit;
 	struct oval_object_content *content;
 	struct oval_entity *entity;
 
+	object = oval_syschar_get_object(syschar);
+
 	/*
 	 * Object name & attributes (id)
 	 */
@@ -359,9 +420,9 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct
 			}
 
 			if (oval_entity_get_varref_type(entity) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
-				stmp = oval_varref_to_sexp(entity, syschar_model, sess);
+				ret = oval_varref_to_sexp(sess, entity, syschar, &stmp);
 
-				if (stmp == NULL) {
+				if (ret != 0) {
 					SEXP_free(elm);
 					elm = NULL;
 					break;
@@ -384,6 +445,10 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct
 			elm = oval_set_to_sexp(oval_object_content_get_setobject(content));
 			break;
 
+		case OVAL_OBJECTCONTENT_FILTER:
+			elm = oval_filter_to_sexp(oval_object_content_get_filter(content));
+			break;
+
 		case OVAL_OBJECTCONTENT_UNKNOWN:
 			break;
 		}
@@ -395,7 +460,7 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct
 				SEXP_free(varrefs);
 			oval_object_content_iterator_free(cit);
 
-			return (NULL);
+			return (ret);
 		}
 
 		SEXP_list_add(lst, elm);
@@ -431,13 +496,15 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct
 	}
 	oval_behavior_iterator_free(bit);
 
-	return (obj_sexp);
+	*out_sexp = obj_sexp;
+
+	return (0);
 }
 
-SEXP_t *oval_state2sexp(struct oval_state * state)
+int oval_state2sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
 {
 	SEXP_t *ste, *ste_name, *ste_ent;
-	SEXP_t *r0, *r1, *r2;
+	SEXP_t *r0, *r1, *r2, *r3, *r4;
 	char buffer[128];
 	size_t buflen;
 	const char *subtype_name;
@@ -447,7 +514,7 @@ SEXP_t *oval_state2sexp(struct oval_state * state)
 
 	if (subtype_name == NULL) {
 		_D("FAIL: unknown subtype: %d\n", oval_state_get_subtype(state));
-		return (NULL);
+		return (-1);
 	}
 
 	buflen = snprintf(buffer, sizeof buffer, "%s_state", subtype_name);
@@ -455,36 +522,24 @@ SEXP_t *oval_state2sexp(struct oval_state * state)
 
 	ste_name = SEXP_list_new(r0 = SEXP_string_new(buffer, buflen),
 				 r1 = SEXP_string_new(":id", 3),
-				 r2 = SEXP_string_newf("%s", oval_state_get_id(state)), NULL);
-
-	SEXP_free(r0);
-	SEXP_free(r1);
-	SEXP_free(r2);
+				 r2 = SEXP_string_newf("%s", oval_state_get_id(state)),
+				 r3 = SEXP_string_new(":operator", 9),
+				 r4 = SEXP_number_newu(oval_state_get_operator(state)),
+				 NULL);
 
 	ste = SEXP_list_new(ste_name, NULL);
-
-	SEXP_free(ste_name);
-
-	//entities = oval_state_entities (state);
+	SEXP_vfree(r0, r1, r2, r3, r4, ste_name, NULL);
 
 	contents = oval_state_get_contents(state);
-
-	/*
-	   while (oval_entity_iterator_has_more (entities)) {
-	   ste_ent = oval_entity_to_sexp (oval_entity_iterator_next (entities));
-	   SEXP_list_add (ste, ste_ent);
-	   }
-	 */
 	while (oval_state_content_iterator_has_more(contents)) {
 		oval_check_t ochk;
+		struct oval_entity *ent;
 		struct oval_state_content *content = oval_state_content_iterator_next(contents);
 
-		ste_ent = oval_entity_to_sexp(oval_state_content_get_entity(content));
-
+		ent = oval_state_content_get_entity(content);
+		ste_ent = oval_entity_to_sexp(ent);
 		if (ste_ent == NULL) {
-			SEXP_free(ste);
-			ste = NULL;
-			break;
+			goto fail;
 		}
 
 		ochk = oval_state_content_get_var_check(content);
@@ -499,16 +554,87 @@ SEXP_t *oval_state2sexp(struct oval_state * state)
 			SEXP_free(r0);
 		}
 
+		if (oval_entity_get_varref_type(ent) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
+			SEXP_t *val_lst;
+			struct oval_variable *var;
+			struct oval_value_iterator *val_itr;
+			oval_datatype_t dt;
+			oval_syschar_collection_flag_t flag;
+
+			var = oval_entity_get_variable(ent);
+			if (oval_probe_query_variable(sess, var) != 0) {
+				goto fail;
+			}
+
+			dt = oval_entity_get_datatype(ent);
+			val_lst = SEXP_list_new(NULL);
+
+			flag = oval_variable_get_collection_flag(var);
+			if (flag == SYSCHAR_FLAG_COMPLETE
+			    || flag == SYSCHAR_FLAG_INCOMPLETE) {
+				val_itr = oval_variable_get_values(var);
+				while (oval_value_iterator_has_more(val_itr)) {
+					struct oval_value *val;
+					SEXP_t *vs;
+
+					val = oval_value_iterator_next(val_itr);
+					vs = oval_value_to_sexp(val, dt);
+					if (vs == NULL) {
+						oscap_dlprintf(DBG_E, "Failed to convert OVAL value to SEXP: "
+							       "datatype: %s, text: %s.\n",
+							       oval_datatype_get_text(dt),
+							       oval_value_get_text(val));
+						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Failed to convert OVAL value to SEXP");
+						oval_value_iterator_free(val_itr);
+						SEXP_free(val_lst);
+						goto fail;
+					}
+					SEXP_list_add(val_lst, vs);
+					SEXP_free(vs);
+				}
+				oval_value_iterator_free(val_itr);
+			}
+
+			SEXP_list_add(ste_ent, val_lst);
+			SEXP_free(val_lst);
+		}
+
 		SEXP_list_add(ste, ste_ent);
 		SEXP_free(ste_ent);
 	}
-
 	oval_state_content_iterator_free(contents);
 
-	return (ste);
+	*out_sexp = ste;
+	return (0);
+
+ fail:
+	oval_state_content_iterator_free(contents);
+	SEXP_vfree(ste, ste_ent, NULL);
+	return (-1);
 }
 
-static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
+static struct oval_message *oval_sexp2msg(const SEXP_t *msg)
+{
+	struct oval_message *message;
+	SEXP_t *r0;
+	oval_message_level_t lvl;
+	char *str;
+
+	message = oval_message_new();
+	r0 = SEXP_list_first(msg);
+	lvl = SEXP_number_getu(r0);
+	SEXP_free(r0);
+	oval_message_set_level(message, lvl);
+	r0 = SEXP_list_nth(msg, 2);
+	str = SEXP_string_cstr(r0);
+	SEXP_free(r0);
+	oval_message_set_text(message, str);
+	oscap_free(str);
+
+	return message;
+}
+
+static struct oval_sysent *oval_sysent_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
 {
 	_A(sexp);
 	SEXP_t *sval;
@@ -580,16 +706,16 @@ static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *mo
 
 	int status = probe_ent_getstatus(sexp);
 
-	struct oval_sysitem *item = oval_sysitem_new(model);
+	struct oval_sysent *item = oval_sysent_new(model);
 
-	oval_sysitem_set_status(item, status);
-	oval_sysitem_set_name(item, key);
-	oval_sysitem_set_mask(item, probe_ent_getmask(sexp));
+	oval_sysent_set_status(item, status);
+	oval_sysent_set_name(item, key);
+	oval_sysent_set_mask(item, probe_ent_getmask(sexp));
 
 	if (status == OVAL_STATUS_EXISTS)
-		oval_sysitem_set_value(item, val);
+		oval_sysent_set_value(item, val);
 
-	oval_sysitem_set_datatype(item, datatype);
+	oval_sysent_set_datatype(item, datatype);
 
 	SEXP_free(sval);
 	oscap_free(key);
@@ -598,14 +724,14 @@ static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *mo
 	return item;
 }
 
-static struct oval_sysdata *oval_sysdata_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
+static struct oval_sysitem *oval_sysitem_from_sexp(struct oval_syschar_model *model, SEXP_t * sexp)
 {
 	_A(sexp);
 
 	static int id_counter = 1;	/* TODO better ID generator */
 
 	char *name;
-	struct oval_sysdata *sysdata = NULL;
+	struct oval_sysitem *sysitem = NULL;
 
 	name = probe_ent_getname(sexp);
 
@@ -626,82 +752,60 @@ static struct oval_sysdata *oval_sysdata_from_sexp(struct oval_syschar_model *mo
 
 	char id[16];
 	SEXP_t *sub;
-	struct oval_sysitem *sysitem;
+	struct oval_sysent *sysent;
 
 	int status = probe_ent_getstatus(sexp);
 
 	sprintf(id, "%d", id_counter++);
-	sysdata = oval_sysdata_get_new(model, id);
-	oval_sysdata_set_status(sysdata, status);
-	oval_sysdata_set_subtype(sysdata, type);
-	//oval_sysdata_set_subtype_name(sysdata, name);
+	sysitem = oval_sysitem_get_new(model, id);
+	oval_sysitem_set_status(sysitem, status);
+	oval_sysitem_set_subtype(sysitem, type);
+	//oval_sysitem_set_subtype_name(sysitem, name);
 
 	if (status == OVAL_STATUS_EXISTS) {
 		for (int i = 2; (sub = SEXP_list_nth(sexp, i)) != NULL; ++i) {
-			if ((sysitem = oval_sysitem_from_sexp(model, sub)) != NULL)
-				oval_sysdata_add_item(sysdata, sysitem);
+			if ((sysent = oval_sysent_from_sexp(model, sub)) != NULL)
+				oval_sysitem_add_item(sysitem, sysent);
 			SEXP_free(sub);
 		}
 	}
 
  cleanup:
 	oscap_free(name);
-	return sysdata;
+	return sysitem;
 }
 
-struct oval_syschar *oval_sexp2sysch(const SEXP_t * s_exp, struct oval_syschar_model *model, struct oval_object *object)
+int oval_sexp2sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 {
-	struct oval_syschar *sysch;
-
-	_A(s_exp != NULL);
-
-	sysch = oval_syschar_new(model, object);
-
-	if (oval_sysch_apply_sexp(sysch, s_exp, object) != 0) {
-		oval_syschar_free(sysch);
-		return (NULL);
-	}
-
-	return (sysch);
-}
-
-int oval_sysch_apply_sexp(struct oval_syschar *sysch, const SEXP_t *cobj, struct oval_object *object)
-{
-	_A(cobj != NULL);
-	_A(sysch != NULL);
-
-	SEXP_t *s_exp, *items;
-	struct oval_sysdata *sysdata;
-	struct oval_syschar_model *model;
 	oval_syschar_collection_flag_t flag;
+	SEXP_t *messages, *msg, *items, *item;
+	struct oval_syschar_model *model;
 
-	if (oval_syschar_get_object(sysch) == NULL) {
-		if (object != NULL)
-			oval_syschar_set_object(sysch, object);
-		else
-			return (-1);
+	_A(cobj != NULL);
 
-	} else if (object == NULL) {
-		object = oval_syschar_get_object(sysch);
+	flag = probe_cobj_get_flag(cobj);
+	oval_syschar_set_flag(syschar, flag);
 
-		if (object == NULL)
-			return (-1);
+	messages = probe_cobj_get_msgs(cobj);
+	SEXP_list_foreach(msg, messages) {
+		struct oval_message *omsg;
+
+		omsg = oval_sexp2msg(msg);
+		if (omsg != NULL)
+			oval_syschar_add_message(syschar, omsg);
 	}
+	SEXP_free(messages);
 
-	_A(object == oval_syschar_get_object(sysch));
+	model = oval_syschar_get_model(syschar);
+	items = probe_cobj_get_items(cobj);
+	SEXP_list_foreach(item, items) {
+		struct oval_sysitem *sysitem;
 
-	model = oval_syschar_get_model(sysch);
-	flag = _probe_cobj_get_flag(cobj);
-	oval_syschar_set_flag(sysch, flag);
-
-	items = _probe_cobj_get_items(cobj);
-	SEXP_list_foreach(s_exp, items) {
-		sysdata = oval_sysdata_from_sexp(model, s_exp);
-
-		if (sysdata != NULL)
-			oval_syschar_add_sysdata(sysch, sysdata);
+		sysitem = oval_sysitem_from_sexp(model, item);
+		if (sysitem != NULL)
+			oval_syschar_add_sysitem(syschar, sysitem);
 	}
 	SEXP_free(items);
 
-	return (0);
+	return 0;
 }
