@@ -137,7 +137,14 @@ int oval_state_get_operator(struct oval_state *state)
 
 struct oval_state *oval_state_new(struct oval_definition_model *model, const char *id)
 {
-	oval_state_t *state = (oval_state_t *) oscap_alloc(sizeof(oval_state_t));
+	oval_state_t *state;
+
+	if (model && oval_definition_model_is_locked(model)) {
+		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+		return NULL;
+	}
+
+	state = (oval_state_t *) oscap_alloc(sizeof(oval_state_t));
 	if (state == NULL)
 		return NULL;
 
@@ -150,6 +157,9 @@ struct oval_state *oval_state_new(struct oval_definition_model *model, const cha
 	state->notes = oval_collection_new();
 	state->contents = oval_collection_new();
 	state->model = model;
+
+	oval_definition_model_add_state(model, state);
+
 	return state;
 }
 
@@ -161,7 +171,7 @@ bool oval_state_is_valid(struct oval_state * state)
 	*/
 
 	if (state == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
         }
 	/*
@@ -213,8 +223,6 @@ struct oval_state *oval_state_clone(struct oval_definition_model *new_model, str
 			oval_state_add_content(new_state, oval_state_content_clone(new_model, content));
 		}
 		oval_state_content_iterator_free(contents);
-
-		oval_definition_model_add_state(new_model, new_state);
 	}
 	return new_state;
 }
@@ -395,7 +403,7 @@ xmlNode *oval_state_to_dom(struct oval_state *state, xmlDoc * doc, xmlNode * par
 	char state_name[strlen(subtype_text) + 7];
 	*state_name = '\0';
 	strcat(strcat(state_name, subtype_text), "_state");
-	xmlNode *state_node = xmlNewChild(parent, NULL, BAD_CAST state_name, NULL);
+	xmlNode *state_node = xmlNewTextChild(parent, NULL, BAD_CAST state_name, NULL);
 
 	oval_family_t family = oval_state_get_family(state);
 	const char *family_text = oval_family_get_text(family);
@@ -430,10 +438,10 @@ xmlNode *oval_state_to_dom(struct oval_state *state, xmlDoc * doc, xmlNode * par
 	struct oval_string_iterator *notes = oval_state_get_notes(state);
 	if (oval_string_iterator_has_more(notes)) {
 		xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
-		xmlNode *notes_node = xmlNewChild(state_node, ns_definitions, BAD_CAST "notes", NULL);
+		xmlNode *notes_node = xmlNewTextChild(state_node, ns_definitions, BAD_CAST "notes", NULL);
 		while (oval_string_iterator_has_more(notes)) {
 			char *note = oval_string_iterator_next(notes);
-			xmlNewChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
+			xmlNewTextChild(notes_node, ns_definitions, BAD_CAST "note", BAD_CAST note);
 		}
 	}
 	oval_string_iterator_free(notes);
