@@ -88,9 +88,10 @@ bool oval_value_get_boolean(struct oval_value * value)
 {
 	__attribute__nonnull__(value);
 
-	if (strncmp("false", (value)->text, 5))
-		return true;
-	return false;
+	if (strcmp("false", (value)->text) == 0
+	    || strcmp("0", (value)->text) == 0)
+		return false;
+	return true;
 }
 
 float oval_value_get_float(struct oval_value *value)
@@ -122,7 +123,7 @@ struct oval_value *oval_value_new(oval_datatype_t datatype, char *text_value)
 
 bool oval_value_is_valid(struct oval_value * value)
 {
-        oscap_dprintf("WARNING: NOOP.\n");
+        oscap_dlprintf(DBG_W, "NOOP.\n");
 	return true;		//TODO
 }
 
@@ -148,6 +149,18 @@ void oval_value_free(struct oval_value *value)
 	}
 }
 
+int oval_value_cast(struct oval_value *value, oval_datatype_t new_dt)
+{
+	oval_datatype_t old_dt;
+
+	old_dt = oval_value_get_datatype(value);
+
+	// todo: attempt a proper cast here
+	value->datatype = new_dt;
+
+	return 0;
+}
+
 /*
 void oval_value_set_datatype(struct oval_value *value,
 			     oval_datatype_t datatype)
