@@ -46,6 +46,7 @@ typedef struct _oval_variable_model_frame {
 } _oval_variable_model_frame_t;
 
 typedef struct oval_variable_model {
+	struct oval_generator *generator;
 	struct oval_string_map *varmap;
 	bool is_locked;
 } oval_variable_model_t;
@@ -61,7 +62,6 @@ static int _oval_variable_model_parse_variable(xmlTextReader * reader, struct ov
 static int _oval_variable_model_parse_variables(xmlTextReader * reader, struct oval_parser_context *context,
 						struct oval_variable_model *model);
 static int _oval_variable_model_parse(struct oval_variable_model *model, xmlTextReader * reader, void *user_param);
-static int _oval_generator_parse_tag(xmlTextReader * reader, struct oval_parser_context *context, char *label);
 
 static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, const char *comm, oval_datatype_t datatype)
 {
@@ -81,7 +81,7 @@ bool oval_variable_model_is_valid(struct oval_variable_model *variable_model)
 	struct oval_string_iterator *varids_itr;
 
 	if (variable_model == NULL) {
-                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
 		return false;
         }
 
@@ -90,12 +90,12 @@ bool oval_variable_model_is_valid(struct oval_variable_model *variable_model)
 		char *varid = oval_string_iterator_next(varids_itr);
 
                 if (oval_variable_model_get_datatype(variable_model, varid) == OVAL_DATATYPE_UNKNOWN) {
-                        oscap_dprintf("WARNING: argument is not valid: variable (%s) datatype == OVAL_DATATYPE_UNKNOWN.\n", varid);
+                        oscap_dlprintf(DBG_W, "Argument is not valid: variable (%s) datatype == OVAL_DATATYPE_UNKNOWN.\n", varid);
                         is_valid = false;
                         break;
                 }
 		if (oval_variable_model_get_comment(variable_model, varid) == NULL) {
-                        oscap_dprintf("WARNING: argument is not valid: variable (%s) comment == NULL.\n", varid);
+                        oscap_dlprintf(DBG_W, "Argument is not valid: variable (%s) comment == NULL.\n", varid);
                         is_valid = false;
                         break;
                 }
@@ -138,6 +138,7 @@ struct oval_variable_model *oval_variable_model_new()
 	oval_variable_model_t *model = (oval_variable_model_t *) oscap_alloc(sizeof(oval_variable_model_t));
 	if (model == NULL)
 		return NULL;
+	model->generator = oval_generator_new();
 	model->varmap = oval_string_map_new();
 	model->is_locked = false;
 	return model;
@@ -169,6 +170,17 @@ void oval_variable_model_free(struct oval_variable_model *model)
 	}
 }
 
+struct oval_generator *oval_variable_model_get_generator(struct oval_variable_model *model)
+{
+	return model->generator;
+}
+
+void oval_variable_model_set_generator(struct oval_variable_model *model, struct oval_generator *generator)
+{
+	oval_generator_free(model->generator);
+	model->generator = generator;
+}
+
 void oval_variable_model_add(struct oval_variable_model *model, char *varid, const char *comm,
 			     oval_datatype_t datatype, char *value)
 {
@@ -183,42 +195,6 @@ void oval_variable_model_add(struct oval_variable_model *model, char *varid, con
 }
 
 #define NAMESPACE_VARIABLES "http://oval.mitre.org/XMLSchema/oval-variables-5"
-#define NAMESPACE_COMMON    "http://oval.mitre.org/XMLSchema/oval-common-5"
-static int _oval_generator_parse_tag(xmlTextReader * reader, struct oval_parser_context *context, char *label)
-{
-	char *tagname = (char *)xmlTextReaderLocalName(reader);
-	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
-	int return_code;
-	bool is_common_ns = strcmp(NAMESPACE_COMMON, namespace) == 0;
-	if (is_common_ns && strcmp("product_name", tagname) == 0) {
-		return_code = xmlTextReaderRead(reader);
-		char *value = (char *)xmlTextReaderValue(reader);
-		oscap_dprintf("INFO: %s:    product name: %s", label, value);
-		oscap_free(value);
-	} else if (is_common_ns && strcmp("product_version", tagname) == 0) {
-		return_code = xmlTextReaderRead(reader);
-		char *value = (char *)xmlTextReaderValue(reader);
-		oscap_dprintf("INFO: %s: product version: %s", label, value);
-		oscap_free(value);
-	} else if (is_common_ns && strcmp("schema_version", tagname) == 0) {
-		return_code = xmlTextReaderRead(reader);
-		char *value = (char *)xmlTextReaderValue(reader);
-		oscap_dprintf("INFO: %s:  schema version: %s", label, value);
-		oscap_free(value);
-	} else if (is_common_ns && strcmp("timestamp", tagname) == 0) {
-		return_code = xmlTextReaderRead(reader);
-		char *value = (char *)xmlTextReaderValue(reader);
-		oscap_dprintf("INFO: %s:      time stamp: %s", label, value);
-		oscap_free(value);
-	} else {
-		oscap_dprintf("WARNING: UNPROCESSED TAG <%s:%s>", namespace, tagname);
-		oval_parser_skip_tag(reader, context);
-		return_code = 0;
-	}
-	oscap_free(tagname);
-	oscap_free(namespace);
-	return return_code;
-}
 
 static int _oval_variable_model_parse_variable_values
     (xmlTextReader * reader, struct oval_parser_context *context, _oval_variable_model_frame_t * frame) {
@@ -232,7 +208,7 @@ static int _oval_variable_model_parse_variable_values
 		oval_collection_add(frame->values, strdup(value));
 		oscap_free(value);
 	} else {
-		oscap_dprintf("WARNING: UNPROCESSED TAG <%s:%s>", namespace, tagname);
+		oscap_dlprintf(DBG_W, "Unprocessed tag: <%s:%s>.\n", namespace, tagname);
 		oval_parser_skip_tag(reader, context);
 		return_code = 0;
 	}
@@ -250,7 +226,7 @@ static int _oval_variable_model_parse_variable
 	int return_code = 1;
 	if (frame) {
 		if (frame->datatype != datatype) {
-			oscap_dprintf("WARNING: Unmatched variable datatypes %s:%s",
+			oscap_dlprintf(DBG_W, "Unmatched variable datatypes: %s:%s.\n",
 				      oval_datatype_get_text(frame->datatype), oval_datatype_get_text(datatype));
 			oval_parser_skip_tag(reader, context);
 			return_code = 0;
@@ -276,7 +252,7 @@ static int _oval_variable_model_parse_variables
 	if (is_variable_ns && strcmp("variable", tagname) == 0) {
 		return_code = _oval_variable_model_parse_variable(reader, context, model);
 	} else {
-		oscap_dprintf("WARNING: UNPROCESSED TAG <%s:%s>", namespace, tagname);
+		oscap_dlprintf(DBG_W, "Unprocessed tag: <%s:%s>.\n", namespace, tagname);
 		oval_parser_skip_tag(reader, context);
 
 		/*oscap_seterr */
@@ -294,13 +270,15 @@ static int _oval_variable_model_parse_tag
 	int return_code;
 	bool is_variable_ns = strcmp(NAMESPACE_VARIABLES, namespace) == 0;
 	if (is_variable_ns && strcmp("generator", tagname) == 0) {
-		return_code =
-		    oval_parser_parse_tag(reader, context, (oval_xml_tag_parser) _oval_generator_parse_tag, "oval_variables");
+		struct oval_generator *gen;
+
+		gen = oval_variable_model_get_generator(context->variable_model);
+		return_code = oval_generator_parse_tag(reader, context, gen);
 	} else if (is_variable_ns && strcmp("variables", tagname) == 0) {
 		return_code =
 		    oval_parser_parse_tag(reader, context, (oval_xml_tag_parser) _oval_variable_model_parse_variables, model);
 	} else {
-		oscap_dprintf("WARNING: UNPROCESSED TAG <%s:%s>", namespace, tagname);
+		oscap_dlprintf(DBG_W, "Unprocessed tag: <%s:%s>.\n", namespace, tagname);
 		oval_parser_skip_tag(reader, context);
 		return_code = 0;
 	}
@@ -313,6 +291,7 @@ static int _oval_variable_model_parse(struct oval_variable_model *model, xmlText
 {
 	int return_code = 0;
 	struct oval_parser_context context;
+	context.variable_model = model;
 	context.reader = reader;
 	context.user_data = user_param;
 	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, &context);
@@ -323,7 +302,7 @@ static int _oval_variable_model_parse(struct oval_variable_model *model, xmlText
 		return_code =
 		    oval_parser_parse_tag(reader, &context, (oval_xml_tag_parser) _oval_variable_model_parse_tag, model);
 	} else {
-		oscap_dprintf("WARNING: UNPROCESSED TAG <%s:%s>", namespace, tagname);
+		oscap_dlprintf(DBG_W, "Unprocessed tag: <%s:%s>.\n", namespace, tagname);
 		return_code = oval_parser_skip_tag(reader, &context);
 	}
 	oscap_free(tagname);
@@ -333,47 +312,28 @@ static int _oval_variable_model_parse(struct oval_variable_model *model, xmlText
 
 struct oval_variable_model * oval_variable_model_import(const char *file)
 {
-	int return_code;
+	int ret;
 	struct oval_variable_model * model;
-	xmlDoc *doc = xmlParseFile(file);
-	if (doc == NULL) {
-		oscap_setxmlerr(xmlGetLastError());
-		return NULL;
-	}
+
 	xmlTextReader *reader = xmlNewTextReaderFilename(file);
 	if (reader == NULL) {
-		oscap_setxmlerr(xmlGetLastError());
-		return NULL;
+                if(errno)
+                        oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, strerror(errno));
+                oscap_dlprintf(DBG_E, "Unable to open file.\n");
+                return NULL;
 	}
-	xmlTextReaderRead(reader);
 
+	xmlTextReaderRead(reader);
 	model = oval_variable_model_new();
-	return_code = _oval_variable_model_parse(model, reader, NULL);
-	if (return_code != 1) {
+	ret = _oval_variable_model_parse(model, reader, NULL);
+	if (ret != 1) {
 		oval_variable_model_free(model);
 		model = NULL;
-	}
-		
+	}		
 	xmlFreeTextReader(reader);
-	xmlFreeDoc(doc);
 
 	return model;
 }
-static int _generator_to_dom(xmlDocPtr doc, xmlNode * tag_generator)
-{
-	xmlNs *ns_common = xmlSearchNsByHref(doc, tag_generator, OVAL_COMMON_NAMESPACE);
-	xmlNewChild(tag_generator, ns_common, BAD_CAST "product_name", BAD_CAST "OPEN SCAP");
-	xmlNewChild(tag_generator, ns_common, BAD_CAST "schema_version", BAD_CAST "5.5");
-	time_t epoch_time[] = { 0 };
-	time(epoch_time);
-	struct tm *lt = localtime(epoch_time);
-	char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-	snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-		 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-	xmlNewChild(tag_generator, ns_common, BAD_CAST "timestamp", BAD_CAST timestamp);
-
-	return 1;
-}
 
 
 static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable_model,
@@ -383,7 +343,7 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 	xmlNodePtr root_node;
 
 	if (parent) {
-		root_node = xmlNewChild(parent, NULL, BAD_CAST "oval_variables", NULL);
+		root_node = xmlNewTextChild(parent, NULL, BAD_CAST "oval_variables", NULL);
 	} else {
 		root_node = xmlNewNode(NULL, BAD_CAST "oval_variables");
 		xmlDocSetRootElement(doc, root_node);
@@ -394,11 +354,9 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 	xmlSetNs(root_node, ns_common);
 	xmlSetNs(root_node, ns_variables);
 
-	xmlNode *tag_generator = xmlNewChild(root_node, ns_variables, BAD_CAST "generator", NULL);
+	oval_generator_to_dom(variable_model->generator, doc, root_node);
 
-	_generator_to_dom(doc, tag_generator);
-
-	xmlNode *variables = xmlNewChild(root_node, ns_variables, BAD_CAST "variables", NULL);
+	xmlNode *variables = xmlNewTextChild(root_node, ns_variables, BAD_CAST "variables", NULL);
 
 
 	struct oval_string_iterator *varids = oval_variable_model_get_variable_ids(variable_model);
@@ -407,7 +365,7 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 		oval_datatype_t datatype = oval_variable_model_get_datatype(variable_model, varid);
 		const char *comm = oval_variable_model_get_comment(variable_model, varid);
 
-                xmlNode *variable = xmlNewChild(variables, ns_variables, BAD_CAST "variable", NULL);
+                xmlNode *variable = xmlNewTextChild(variables, ns_variables, BAD_CAST "variable", NULL);
                 xmlNewProp(variable, BAD_CAST "id", BAD_CAST varid);
                 xmlNewProp(variable, BAD_CAST "datatype", BAD_CAST oval_datatype_get_text(datatype));
                 xmlNewProp(variable, BAD_CAST "comment", BAD_CAST comm);
@@ -415,7 +373,7 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 		struct oval_string_iterator *values = oval_variable_model_get_values(variable_model, varid);
 		while (oval_string_iterator_has_more(values)) {
 			char *value = oval_string_iterator_next(values);
-                        xmlNewChild(variable, ns_variables, BAD_CAST "value", BAD_CAST value);
+                        xmlNewTextChild(variable, ns_variables, BAD_CAST "value", BAD_CAST value);
 		}
                 oval_string_iterator_free(values);
 	}
