@@ -91,7 +91,7 @@ void __seap_debuglog (const char *file, const char *fn, size_t line, const char
 
 #if defined(SEAP_THREAD_SAFE)
         /* XXX: non-portable usage of pthread_t */
-        fprintf (__debuglog_fp, "(%u:%u) [%s: %zu: %s] ", (unsigned int)getpid (), (unsigned int)pthread_self(), file, line, fn);
+        fprintf (__debuglog_fp, "(%u:%llx) [%s: %zu: %s] ", (unsigned int)getpid (), (unsigned long long)pthread_self(), file, line, fn);
 #else
         fprintf (__debuglog_fp, "(%u) [%s: %zu: %s] ", (unsigned int)getpid (), file, line, fn);
 #endif
