@@ -94,7 +94,7 @@ SEXP_numtype_t SEXP_rawval_number_type (SEXP_val_t *dsc)
 size_t SEXP_rawval_list_length (struct SEXP_val_list *list)
 {
         size_t length;
-        struct SEXP_val_lblk *lblk;
+        register struct SEXP_val_lblk *lblk;
 
         length = 0;
         lblk   = SEXP_VALP_LBLK(list->b_addr);
@@ -172,7 +172,6 @@ uintptr_t SEXP_rawval_lblk_fill (uintptr_t lblkp, SEXP_t *s_exp[], uint16_t s_ex
         for (; s_exp_count > 0; --s_exp_count) {
                 lblk->memb[s_exp_count - 1].s_valp = SEXP_rawval_incref (s_exp[s_exp_count - 1]->s_valp);
                 lblk->memb[s_exp_count - 1].s_type = s_exp[s_exp_count - 1]->s_type;
-                lblk->memb[s_exp_count - 1].s_flgs = 0;
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
                 lblk->memb[s_exp_count - 1].__magic0 = s_exp[s_exp_count - 1]->__magic0;
                 lblk->memb[s_exp_count - 1].__magic1 = s_exp[s_exp_count - 1]->__magic1;
@@ -186,12 +185,12 @@ uintptr_t SEXP_rawval_lblk_add (uintptr_t lblkp, const SEXP_t *s_exp)
 {
         uintptr_t lb_prev;
         uintptr_t lb_head;
-        struct SEXP_val_lblk *lblk;
+        register struct SEXP_val_lblk *lblk;
 
         lblk = SEXP_VALP_LBLK(lblkp);
 
         if (lblk == NULL) {
-                lb_head = SEXP_rawval_lblk_new (2);
+                lb_head = SEXP_rawval_lblk_new (1);
                 lb_prev = lb_head;
         } else {
                 lb_head = lblkp;
@@ -250,7 +249,6 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
 
                 lblk->memb[lblk->real].s_valp = SEXP_rawval_incref (s_exp->s_valp);
                 lblk->memb[lblk->real].s_type = s_exp->s_type;
-                lblk->memb[lblk->real].s_flgs = 0;
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
                 lblk->memb[lblk->real].__magic0 = s_exp->__magic0;
                 lblk->memb[lblk->real].__magic1 = s_exp->__magic1;
@@ -259,9 +257,12 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
 
                 return (lblkp);
         } else {
-                uint8_t   new_sz = ((lblk->nxsz & SEXP_LBLKS_MASK) + 1) % 16;
+                uint8_t   new_sz;
                 uintptr_t new_lb;
 
+		new_sz = lblk->nxsz & SEXP_LBLKS_MASK;
+		new_sz = new_sz == 15 ? 6 : new_sz + 1;
+
                 new_lb     = SEXP_rawval_lblk_new (new_sz);
                 lblk->nxsz = (new_lb & SEXP_LBLKP_MASK) | (lblk->nxsz & SEXP_LBLKS_MASK);
 
@@ -282,8 +283,8 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
 
 uintptr_t SEXP_rawval_lblk_last (uintptr_t lblkp)
 {
-        struct SEXP_val_lblk *lblk;
-        uintptr_t last;
+        register struct SEXP_val_lblk *lblk;
+        register uintptr_t last;
 
         last = lblkp;
         lblk = SEXP_VALP_LBLK(last);
@@ -298,7 +299,7 @@ uintptr_t SEXP_rawval_lblk_last (uintptr_t lblkp)
 
 SEXP_t *SEXP_rawval_lblk_nth (uintptr_t lblkp, uint32_t n)
 {
-        struct SEXP_val_lblk *lblk;
+        register struct SEXP_val_lblk *lblk;
 
         _LOGCALL_;
         _D("n = %u\n", n);
@@ -380,7 +381,6 @@ replace:
         (*o_val) = SEXP_new ();
         (*o_val)->s_valp = memb->s_valp;
         (*o_val)->s_type = memb->s_type;
-        (*o_val)->s_flgs = memb->s_flgs;
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
         (*o_val)->__magic0 = memb->__magic0;
         (*o_val)->__magic1 = memb->__magic1;
@@ -388,7 +388,6 @@ replace:
 
         memb->s_valp = SEXP_rawval_incref (n_val->s_valp);
         memb->s_type = n_val->s_type;
-        memb->s_flgs = n_val->s_flgs;
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
         memb->__magic0 = n_val->__magic0;
         memb->__magic1 = n_val->__magic1;
@@ -398,8 +397,8 @@ replace:
 
 int SEXP_rawval_lblk_cb (uintptr_t lblkp, int (*func) (SEXP_t *, void *), void *arg, uint32_t n)
 {
-        struct SEXP_val_lblk *lblk;
-        uint16_t bi;
+        register struct SEXP_val_lblk *lblk;
+        register uint16_t bi;
 
         lblk = SEXP_VALP_LBLK(lblkp);
 
@@ -506,7 +505,6 @@ uintptr_t SEXP_rawval_lblk_copy (uintptr_t lblkp, uint16_t n_skip)
                 while (off_n < (1 << cur_sz) && off_o < lb_old->real) {
                         lb_new->memb[off_n].s_valp = SEXP_rawval_incref (lb_old->memb[off_o].s_valp);
                         lb_new->memb[off_n].s_type = lb_old->memb[off_o].s_type;
-                        lb_new->memb[off_n].s_flgs = lb_old->memb[off_o].s_flgs;
 #if !defined(NDEBUG) || defined(VALIDATE_SEXP)
                         lb_new->memb[off_n].__magic0 = lb_old->memb[off_o].__magic0;
                         lb_new->memb[off_n].__magic1 = lb_old->memb[off_o].__magic1;
