@@ -57,6 +57,7 @@
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -69,7 +70,7 @@
 #include <seap.h>
 #include <probe-api.h>
 #include <probe-entcmp.h>
-#include <findfile.h>
+#include <oval_fts.h>
 #include <alloc.h>
 #include "common/assume.h"
 
@@ -205,7 +206,7 @@ struct pfdata {
 	char *pattern;
 	SEXP_t *filename_ent;
 	SEXP_t *instance_ent;
-	SEXP_t *item_list;
+	SEXP_t *cobj;
 };
 
 static int process_file(const char *path, const char *file, void *arg)
@@ -226,12 +227,25 @@ static int process_file(const char *path, const char *file, void *arg)
 
 	re = pcre_compile(pfd->pattern, PCRE_UTF8, &error, &erroffset, NULL);
 	if (re == NULL) {
+		char s[1024];
+		SEXP_t *msg;
+		snprintf(s, sizeof (s), "pcre_compile() '%s' %s", pfd->pattern, error);
+		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
+		probe_cobj_add_msg(pfd->cobj, msg);
+		SEXP_free(msg);
 		return -1;
 	}
 #elif defined USE_REGEX_POSIX
 	regex_t _re, *re = &_re;
 
-	if (regcomp(re, pfd->pattern, REG_EXTENDED | REG_NEWLINE) != 0) {
+	int err;
+	if ( (err=regcomp(re, pfd->pattern, REG_EXTENDED | REG_NEWLINE)) != 0) {
+		char s[1024];
+		SEXP_t *msg;
+		snprintf(s, sizeof (s), "regcomp() '%s' returned %d", pfd->pattern, err);
+		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
+		probe_cobj_add_msg(pfd->cobj, msg);
+		SEXP_free(msg);
 		return -1;
 	}
 #endif
@@ -253,6 +267,12 @@ static int process_file(const char *path, const char *file, void *arg)
 
 	fp = fopen(whole_path, "rb");
 	if (fp == NULL) {
+		char s[1024];
+		SEXP_t *msg;
+		snprintf(s, sizeof (s), "fopen() '%s' %s", whole_path, strerror(errno));
+		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
+		probe_cobj_add_msg(pfd->cobj, msg);
+		SEXP_free(msg);
 		ret = -2;
 		goto cleanup;
 	}
@@ -280,7 +300,7 @@ static int process_file(const char *path, const char *file, void *arg)
 
                                 item = create_item(path, file, pfd->pattern,
                                                    cur_inst, substrs, substr_cnt);
-                                SEXP_list_add(pfd->item_list, item);
+				probe_cobj_add_item(pfd->cobj, item);
                                 SEXP_free(item);
 
 				for (k = 0; k < substr_cnt; ++k)
@@ -304,7 +324,7 @@ static int process_file(const char *path, const char *file, void *arg)
 	return ret;
 }
 
-SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
+int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
 {
 	SEXP_t *path_ent, *file_ent, *inst_ent, *bh_ent, *patt_ent;
         SEXP_t *r0, *r1;
@@ -312,31 +332,31 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 	int fcnt;
 	struct pfdata pfd;
 
+	OVAL_FTS    *ofts;
+	OVAL_FTSENT *ofts_ent;
+
         (void)arg;
 
-	if (probe_in == NULL) {
-		*err = PROBE_EINVAL;
-		return NULL;
+	if (probe_in == NULL || probe_out == NULL) {
+		return(PROBE_EINVAL);
 	}
 
-        *err = PROBE_ENOELM;
-
         if ((path_ent = probe_obj_getent(probe_in, "path",     1)) == NULL)
-                return(NULL);
+		return(PROBE_ENOELM);
         if ((file_ent = probe_obj_getent(probe_in, "filename", 1)) == NULL) {
                 SEXP_free(path_ent);
-                return(NULL);
+		return(PROBE_ENOELM);
         }
         if ((inst_ent = probe_obj_getent(probe_in, "instance", 1)) == NULL) {
                 SEXP_free(path_ent);
                 SEXP_free(file_ent);
-                return(NULL);
+		return(PROBE_ENOELM);
         }
         if ((patt_ent = probe_obj_getent(probe_in, "pattern",  1)) == NULL) {
                 SEXP_free(path_ent);
                 SEXP_free(file_ent);
                 SEXP_free(inst_ent);
-                return(NULL);
+		return(PROBE_ENOELM);
         } else {
                 SEXP_t *ent_val;
 
@@ -374,28 +394,28 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 	pfd.pattern      = pattern;
 	pfd.filename_ent = file_ent;
 	pfd.instance_ent = inst_ent;
-	pfd.item_list    = SEXP_list_new(NULL);
+	pfd.cobj         = probe_out;
 
-	fcnt = find_files(path_ent, file_ent, bh_ent, process_file, (void *) &pfd);
+	fcnt = 0;
 
-	if (fcnt < 0) {
-		SEXP_t *item;
-		item = probe_item_creat("textfilecontent_item", NULL,
-                                        "path", NULL,
-                                        r0 = probe_ent_getval(path_ent),
-                                        NULL);
-                probe_item_setstatus(item, OVAL_STATUS_ERROR);
-		SEXP_list_add(pfd.item_list, item);
-                SEXP_vfree(r0, item, NULL);
+	if ((ofts = oval_fts_open(path_ent, file_ent, NULL, bh_ent)) != NULL) {
+		for (; (ofts_ent = oval_fts_read(ofts)) != NULL; ++fcnt) {
+			process_file(ofts_ent->path, ofts_ent->file, &pfd);
+			oval_ftsent_free(ofts_ent);
+		}
+
+		oval_fts_close(ofts);
 	}
 
+	if (fcnt < 0)
+		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
+
         SEXP_free(file_ent);
         SEXP_free(path_ent);
         SEXP_free(inst_ent);
         SEXP_free(bh_ent);
         oscap_free(pattern);
 
-	*err = 0;
-	return pfd.item_list;
+	return 0;
 }
 
