@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,6 +34,8 @@
 #include <errno.h>
 #include <config.h>
 #include <alloc.h>
+#include <assume.h>
+#include <bfind.h>
 #include "_probe-api.h"
 #include "probe-entcmp.h"
 #include "encache.h"
@@ -44,6 +46,20 @@
 
 #define PROBE_SIGEXIT_CLEAN   1
 #define PROBE_SIGEXIT_UNCLEAN 2
+#define PROBE_SIGEXIT_ABORT   3
+
+static int probe_opthandler_varref(int, va_list);
+static int probe_opthandler_rcache(int, va_list);
+
+probe_option_t OSCAP_GSYM(options)[] = {
+	{ PROBE_VARREF_HANDLING, &probe_opthandler_varref },
+	{ PROBE_RESULT_CACHING,  &probe_opthandler_rcache }
+};
+
+#define OPTIONS_COUNT (sizeof OSCAP_GSYM(options)/sizeof(probe_option_t))
+
+bool probe_handle_varref_cstr(const char *ent_name);
+bool probe_handle_varref_sexp(const SEXP_t *ent_name);
 
 volatile int OSCAP_GSYM(sigexit)   = 0;    /**< signal exit flag */
 SEAP_CTX_t  *OSCAP_GSYM(ctx)       = NULL; /**< SEAP context */
@@ -51,9 +67,12 @@ int          OSCAP_GSYM(sd)        = -1;   /**< SEAP descriptor */
 pcache_t    *OSCAP_GSYM(pcache)    = NULL; /**< probe item cache */
 void        *OSCAP_GSYM(probe_arg) = NULL; /**< pointer for probe_main, provided by probe_init */
 encache_t   *OSCAP_GSYM(encache)   = NULL; /**< element name cache */
-
 struct id_desc_t OSCAP_GSYM(id_desc);
 
+bool   OSCAP_GSYM(varref_handling) = true;
+char **OSCAP_GSYM(no_varref_ents) = NULL;
+size_t OSCAP_GSYM(no_varref_ents_cnt) = 0;
+
 void *probe_worker(void *arg);
 
 #define MAX_EVAL_DEPTH 8 /**< maximum recursion depth for set evaluation */
@@ -115,27 +134,141 @@ static SEXP_t *probe_ste_fetch(SEXP_t * id_list)
 }
 
 /**
- * Evaluate an OVAL object identified by its id. Using a remote synchronous SEAP command, this function
- * executes evaluation of an OVAL object which resulst wasn't found in the probe cache. This indirectly
- * spawns a new thread in the probe process which evaluates the object and stores the result in the probe
- * cache. That result is not send to the library because it doesn't know how to handle it. Instead, the
- * result is fetched by this function from the cache and returned to the caller.
+ * Evaluate an OVAL object identified by its id. Using a remote
+ * synchronous SEAP command, this function executes evaluation of an
+ * OVAL object which results weren't found in the probe cache. This
+ * indirectly spawns a new thread in the probe process which evaluates
+ * the object and stores the result in the probe cache. That result is
+ * not send to the library because it doesn't know how to handle
+ * it. Instead, the result is fetched by this function from the cache
+ * and returned to the caller.
  * @param id the id of the OVAL object to be evaluated
  * @return the result of the evaluation of the object or NULL on failure
  */
 static SEXP_t *probe_obj_eval(SEXP_t * id)
 {
-	SEXP_t *res;
+	SEXP_t *res, *rid;
 
 	_LOGCALL_;
 
 	res = SEAP_cmd_exec(OSCAP_GSYM(ctx), OSCAP_GSYM(sd), 0, PROBECMD_OBJ_EVAL, id, SEAP_CMDTYPE_SYNC, NULL, NULL);
 
-	SEXP_free(res);
+	rid = SEXP_list_first(res);
+	assume_r(SEXP_string_cmp(id, rid) == 0, NULL);
+	SEXP_vfree(res, rid, NULL);
 
 	return pcache_sexp_get(OSCAP_GSYM(pcache), id);
 }
 
+SEXP_t *probe_prepare_filters(SEXP_t *obj)
+{
+	SEXP_t *filters;
+	int i;
+
+	filters = SEXP_list_new(NULL);
+
+	for (i = 1; ; ++i) {
+		SEXP_t *of, *f, *ste, *ste_id, *act;
+
+		of = probe_obj_getent(obj, "filter", i);
+		if (of == NULL)
+			break;
+
+		act = probe_ent_getattrval(of, "action");
+		ste_id = probe_ent_getval(of);
+		ste = pcache_sexp_get(OSCAP_GSYM(pcache), ste_id);
+		if (ste == NULL) {
+			SEXP_t *r0, *r1;
+
+			r0 = SEXP_list_new(ste_id, NULL);
+			r1 = probe_ste_fetch(r0);
+			ste = SEXP_list_first(r1);
+			SEXP_vfree(r0, r1, NULL);
+		}
+		SEXP_vfree(of, ste_id, NULL);
+
+		f = SEXP_list_new(act, ste, NULL);
+		SEXP_list_add(filters, f);
+		SEXP_vfree(act, ste, f, NULL);
+	}
+
+	return filters;
+}
+
+bool probe_item_filtered(SEXP_t *item, SEXP_t *filters)
+{
+	bool filtered = false;
+	SEXP_t *filter, *ste;
+
+	SEXP_list_foreach(filter, filters) {
+		SEXP_t *felm, *ste_res, *r0;
+		oval_result_t ores;
+		oval_operator_t oopr;
+		oval_filter_action_t ofact;
+
+		r0 = SEXP_list_first(filter);
+		ofact = SEXP_number_getu(r0);
+		SEXP_free(r0);
+		ste = SEXP_list_nth(filter, 2);
+		ste_res = SEXP_list_new(NULL);
+
+		SEXP_sublist_foreach(felm, ste, 2, SEXP_LIST_END) {
+			SEXP_t *ielm, *elm_res;
+			char *elm_name;
+			oval_check_t ochk;
+			int i;
+
+			elm_res = SEXP_list_new(NULL);
+			elm_name = probe_ent_getname(felm);
+
+			for (i = 1;; ++i) {
+				ielm = probe_obj_getent(item, elm_name, i);
+
+				if (ielm == NULL)
+					break;
+
+				ores = probe_entste_cmp(felm, ielm);
+				SEXP_list_add(elm_res, r0 = SEXP_number_newi_32(ores));
+
+				SEXP_free(ielm);
+				SEXP_free(r0);
+			}
+
+			oscap_free(elm_name);
+			r0 = probe_ent_getattrval(felm, "entity_check");
+
+			if (r0 == NULL)
+				ochk = OVAL_CHECK_ALL;
+			else
+				ochk = SEXP_number_geti_32(r0);
+
+			SEXP_free(r0);
+
+			ores = probe_ent_result_bychk(elm_res, ochk);
+			SEXP_list_add(ste_res, r0 = SEXP_number_newi_32(ores));
+			SEXP_free(r0);
+			SEXP_free(elm_res);
+		}
+
+		r0 = probe_ent_getattrval(ste, "operator");
+		if (r0 == NULL)
+			oopr = OVAL_OPERATOR_AND;
+		else
+			oopr = SEXP_number_geti_32(r0);
+		ores = probe_ent_result_byopr(ste_res, oopr);
+		SEXP_vfree(ste, ste_res, r0, NULL);
+
+		if ((ores == OVAL_RESULT_TRUE && ofact == OVAL_FILTER_ACTION_EXCLUDE)
+		    || (ores = OVAL_RESULT_FALSE && ofact == OVAL_FILTER_ACTION_INCLUDE)) {
+			filtered = true;
+			SEXP_free(filter);
+			break;
+		}
+	}
+
+	return filtered;
+}
+
 /**
  * Combine two collections of items using an operation.
  * @param cobj1 item collection
@@ -145,7 +278,7 @@ static SEXP_t *probe_obj_eval(SEXP_t * id)
  */
 static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_operation_t op)
 {
-	char append;
+	bool append;
 	SEXP_t *res_items, *item1, *item2, *id1, *id2;
 	SEXP_t *res_cobj, *item_lst1, *item_lst2;
 	oval_syschar_collection_flag_t flag1, flag2, res_flag;
@@ -157,25 +290,25 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_op
 	if (cobj2 == NULL)
 		return SEXP_ref(cobj1);
 
-	item_lst1 = _probe_cobj_get_items(cobj1);
-	item_lst2 = _probe_cobj_get_items(cobj2);
-	flag1 = _probe_cobj_get_flag(cobj1);
-	flag2 = _probe_cobj_get_flag(cobj2);
+	item_lst1 = probe_cobj_get_items(cobj1);
+	item_lst2 = probe_cobj_get_items(cobj2);
+	flag1 = probe_cobj_get_flag(cobj1);
+	flag2 = probe_cobj_get_flag(cobj2);
 
-	res_flag = _probe_cobj_combine_flags(flag1, flag2, op);
+	res_flag = probe_cobj_combine_flags(flag1, flag2, op);
 	res_items = SEXP_list_new(NULL);
 
 	switch (op) {
 	case OVAL_SET_OPERATION_INTERSECTION:
 		SEXP_list_foreach(item1, item_lst1) {
 			id1 = probe_obj_getentval(item1, "id", 1);
-			append = 0;
+			append = false;
 
 			SEXP_list_foreach(item2, item_lst2) {
 				id2 = probe_obj_getentval(item2, "id", 1);
 
 				if (!SEXP_string_cmp(id1, id2)) {
-					append = 1;
+					append = true;
 
 					SEXP_free(id2);
 					SEXP_free(item2);
@@ -205,13 +338,13 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_op
 	case OVAL_SET_OPERATION_COMPLEMENT:
 		SEXP_list_foreach(item1, item_lst1) {
 			id1 = probe_obj_getentval(item1, "id", 1);
-			append = 1;
+			append = true;
 
 			SEXP_list_foreach(item2, item_lst2) {
 				id2 = probe_obj_getentval(item2, "id", 1);
 
 				if (!SEXP_string_cmp(id1, id2)) {
-					append = 0;
+					append = false;
 
 					SEXP_free(id2);
 					SEXP_free(item2);
@@ -230,13 +363,29 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_op
 		}
 		break;
 	default:
-		_D("Unexpected set operation: %d\n", op);
-		SEXP_vfree(item_lst1, item_lst2, res_items, NULL);
+		{
+			char msg[40];
+			SEXP_t *r0, *r1;
 
-		return NULL;
+			snprintf(msg, sizeof (msg), "Unexpected set operation: %d.\n", op);
+			_D(msg);
+			r0 = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, msg);
+			r1 = SEXP_list_new(r0, NULL);
+			res_cobj = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+			SEXP_vfree(item_lst1, item_lst2, res_items, r0, r1, NULL);
+			return res_cobj;
+		}
 	}
 
-	res_cobj = _probe_cobj_new(res_flag, res_items);
+	/*
+	 * If the collected information is complete but all the items are
+	 * removed, the flag is set to SYSCHAR_FLAG_DOES_NOT_EXIST
+	 */
+	if (res_flag == SYSCHAR_FLAG_COMPLETE
+	    && SEXP_list_length(res_items) == 0)
+		res_flag = SYSCHAR_FLAG_DOES_NOT_EXIST;
+
+	res_cobj = probe_cobj_new(res_flag, NULL, res_items);
 	SEXP_vfree(item_lst1, item_lst2, res_items, NULL);
 
 	// todo: variables
@@ -245,28 +394,22 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj1, SEXP_t *cobj2, oval_setobject_op
 }
 
 /**
- * Apply a set of filter on collection of items.
+ * Apply a set of filters to a collected object.
  * @param cobj item collection
  * @param filter set (list) of filters
  * @return collection of items without items that match any of the filters in the input set
  */
 static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
 {
-	int filtered, i;
-	SEXP_t *result_items, *items, *item, *filter, *felm, *ielm;
-	SEXP_t *ste_res, *elm_res, *stmp;
-	char *elm_name;
+	SEXP_t *result_items, *items, *item;
 	oval_syschar_status_t item_status;
-	oval_result_t ores;
-	oval_check_t ochk;
-	oval_operator_t oopr;
 	oval_syschar_collection_flag_t flag;
 
 	_LOGCALL_;
 
 	result_items = SEXP_list_new(NULL);
-	flag = _probe_cobj_get_flag(cobj);
-	items = _probe_cobj_get_items(cobj);
+	flag = probe_cobj_get_flag(cobj);
+	items = probe_cobj_get_items(cobj);
 
 	SEXP_list_foreach(item, items) {
 		item_status = probe_ent_getstatus(item);
@@ -275,78 +418,38 @@ static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
 		case OVAL_STATUS_DOESNOTEXIST:
 			continue;
 		case OVAL_STATUS_ERROR:
+			break;
 		case OVAL_STATUS_NOTCOLLECTED:
-			_D("Supplied item has an invalid status: %d\n", item_status);
-			SEXP_vfree(items, result_items, NULL);
+			{
+				char msg[50];
+				SEXP_t *r0, *r1;
 
-			return NULL;
+				snprintf(msg, sizeof (msg), "Supplied item has an invalid status: %d\n", item_status);
+				_D(msg);
+				r0 = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, msg);
+				r1 = SEXP_list_new(r0, NULL);
+				cobj = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+				SEXP_vfree(items, result_items, r0, r1, NULL);
+				return cobj;
+			}
 		default:
 			break;
 		}
 
-		filtered = 0;
-
-		SEXP_list_foreach(filter, filters) {
-			ste_res = SEXP_list_new(NULL);
-
-			SEXP_sublist_foreach(felm, filter, 2, -1) {
-				elm_res = SEXP_list_new(NULL);
-
-				elm_name = probe_ent_getname(felm);
-
-				for (i = 1;; ++i) {
-					ielm = probe_obj_getent(item, elm_name, i);
-
-					if (ielm == NULL)
-						break;
-
-					ores = probe_entste_cmp(felm, ielm);
-					SEXP_list_add(elm_res, stmp = SEXP_number_newi_32(ores));
-
-					SEXP_free(ielm);
-					SEXP_free(stmp);
-				}
-
-                                oscap_free(elm_name);
-				stmp = probe_ent_getattrval(felm, "entity_check");
-
-				if (stmp == NULL)
-					ochk = OVAL_CHECK_ALL;
-				else
-					ochk = SEXP_number_geti_32(stmp);
-
-				SEXP_free(stmp);
-
-				ores = probe_ent_result_bychk(elm_res, ochk);
-				SEXP_list_add(ste_res, stmp = SEXP_number_newi_32(ores));
-				SEXP_free(stmp);
-                                SEXP_free(elm_res);
-			}
-
-			stmp = probe_ent_getattrval(filter, "operator");
-
-			if (stmp == NULL)
-				oopr = OVAL_OPERATOR_AND;
-			else
-				oopr = SEXP_number_geti_32(stmp);
-
-			SEXP_free(stmp);
-			ores = probe_ent_result_byopr(ste_res, oopr);
-			SEXP_free(ste_res);
-
-			if (ores == OVAL_RESULT_TRUE) {
-				filtered = 1;
-				SEXP_free(filter);
-				break;
-			}
-		}
-
-		if (!filtered) {
+		if (!probe_item_filtered(item, filters)) {
 			SEXP_list_add(result_items, item);
 		}
 	}
 
-	cobj = _probe_cobj_new(flag, result_items);
+	/*
+	 * If the collected information is complete but all the items are
+	 * filtered out, the flag is set to SYSCHAR_FLAG_DOES_NOT_EXIST
+	 */
+	if (flag == SYSCHAR_FLAG_COMPLETE
+	    && SEXP_list_length(result_items) == 0)
+		flag = SYSCHAR_FLAG_DOES_NOT_EXIST;
+
+	cobj = probe_cobj_new(flag, NULL, result_items);
 	SEXP_vfree(items, result_items, NULL);
 
 	return cobj;
@@ -362,7 +465,7 @@ static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
  */
 static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 {
-	SEXP_t *filters_u, *filters_a;
+	SEXP_t *filters_u, *filters_a, *filters_req;
 
 	SEXP_t *s_subset[2];
 	size_t s_subset_i;
@@ -376,19 +479,27 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 	int op_num;
 
 	SEXP_t *result;
+	char msg[70] = "probe_set_eval: Unspecified error condition.";
+	SEXP_t *r0, *r1;
 
 	_LOGCALL_;
 
 	if (depth > MAX_EVAL_DEPTH) {
-		_D("Too many levels: max=%zu\n", MAX_EVAL_DEPTH);
+		snprintf(msg, sizeof (msg), "probe_set_eval: Too many levels: max=%zu.\n", (size_t) MAX_EVAL_DEPTH);
+		_D(msg);
 #ifndef NDEBUG
 		abort();
 #endif
-		return (NULL);
+		r0 = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, msg);
+		r1 = SEXP_list_new(r0, NULL);
+		result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+		SEXP_vfree(r0, r1, NULL);
+		return result;
 	}
 
 	filters_u = SEXP_list_new(NULL);	/* unavailable filters */
 	filters_a = SEXP_list_new(NULL);	/* available filters (cached) */
+	filters_req = SEXP_list_new(NULL);	/* request list for probe_ste_fetch() */
 
 	s_subset[0] = NULL;
 	s_subset[1] = NULL;
@@ -417,7 +528,8 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 
 	probe_set_foreach(member, set) {
 		if (probe_ent_getname_r(member, member_name, sizeof member_name) == 0) {
-			_D("FAIL: Invalid set element: ptr=%p, type=%s\n", member, SEXP_strtype(member));
+			snprintf(msg, sizeof (msg), "probe_set_eval: Invalid set element: ptr=%p, type=%s.\n", member, SEXP_strtype(member));
+			_D(msg);
 			goto eval_fail;
 		}
 
@@ -427,14 +539,17 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 					s_subset[s_subset_i] = probe_set_eval(member, depth + 1);
 
 					if (s_subset[s_subset_i] == NULL) {
-						_D("FAIL: recursive set evaluation failer: m=%p, d=%u\n",
-						   member, depth + 1);
+						snprintf(msg, sizeof (msg),
+							 "probe_set_eval: Recursive set evaluation failed: m=%p, d=%zu.\n",
+							 member, depth + 1);
+						_D(msg);
 						goto eval_fail;
 					}
 
 					++s_subset_i;
 				} else {
-					_D("FAIL: more than 2 \"set\"\n");
+					snprintf(msg, sizeof (msg), "probe_set_eval: More than 2 \"set\".\n");
+					_D(msg);
 					goto eval_fail;
 				}
 			}
@@ -446,7 +561,8 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 				id = probe_ent_getval(member);
 
 				if (id == NULL) {
-					_D("FAIL: set=%p: missing obj_ref value\n", set);
+					snprintf(msg, sizeof (msg), "probe_set_eval: set=%p: Missing obj_ref value.\n", set);
+					_D(msg);
 					goto eval_fail;
 				}
 
@@ -457,11 +573,12 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 					res = probe_obj_eval(id);
 
 					if (res == NULL) {
-#if !defined(NDEBUG)
 						char *tmp = SEXP_string_cstr(id);
-						_D("FAIL: obj=%s: evaluation failed.\n", tmp);
-						oscap_free(tmp);
+						snprintf(msg, sizeof (msg), "probe_set_eval: obj=%s: Evaluation failed.\n", tmp);
+#if !defined(NDEBUG)
+						_D(msg);
 #endif
+						oscap_free(tmp);
 						SEXP_free(id);
 						goto eval_fail;
 					}
@@ -473,8 +590,8 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 					o_subset[o_subset_i] = res;
 					++o_subset_i;
 				} else {
-					_D("FAIL: more than 2 obj_refs\n");
-
+					snprintf(msg, sizeof (msg), "probe_set_eval: More than 2 obj_refs.\n");
+					_D(msg);
 					SEXP_free(res);
 					goto eval_fail;
 				}
@@ -483,29 +600,40 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 			break;
 
 			CASE('f', "ilter") {
-				SEXP_t *id, *res;
+				SEXP_t *id, *act, *ste;
 
 				id = probe_ent_getval(member);
-
 				if (id == NULL) {
-					_D("FAIL: set=%p: missing filter value\n", set);
+					snprintf(msg, sizeof (msg), "probe_set_eval: set=%p: Missing filter value.\n", set);
+					_D(msg);
 					goto eval_fail;
 				}
 
-				res = pcache_sexp_get(OSCAP_GSYM(pcache), id);
+				act = probe_ent_getattrval(member, "action");
+				if (act == NULL) {
+					snprintf(msg, sizeof (msg), "probe_set_eval: set=%p: Missing filter action.\n", set);
+					_D(msg);
+					goto eval_fail;
+				}
 
-				if (res == NULL)
-					SEXP_list_add(filters_u, id);
-				else
-					SEXP_list_add(filters_a, res);
+				ste = pcache_sexp_get(OSCAP_GSYM(pcache), id);
 
-				SEXP_free(id);
-				SEXP_free(res);
+				if (ste == NULL) {
+					SEXP_list_add(filters_req, id);
+					r0 = SEXP_list_new(act, id, NULL);
+					SEXP_list_add(filters_u, r0);
+				} else {
+					r0 = SEXP_list_new(act, ste, NULL);
+					SEXP_list_add(filters_a, r0);
+					SEXP_free(ste);
+				}
 
+				SEXP_vfree(id, act, r0, NULL);
 			}
 			break;
 		default:
-			_D("Unexpected set element: %s\n", member_name);
+			snprintf(msg, sizeof (msg), "probe_set_eval: Unexpected set element: %s.\n", member_name);
+			_D(msg);
 			goto eval_fail;
 		}
 #undef CASE
@@ -515,32 +643,35 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 	member = NULL;
 
 	/* request filters */
-	result = probe_ste_fetch(filters_u);
+	result = probe_ste_fetch(filters_req);
 
 	if (result == NULL) {
+		snprintf(msg, sizeof (msg), "probe_set_eval: Can't get unavailable filters:\n");
 #if !defined(NDEBUG)
-		_D("FAIL: can't get unavailable filters:\n");
-		SEXP_list_foreach(result, filters_u) {
+		_D(msg);
+		SEXP_list_foreach(result, filters_req) {
 			SEXP_fprintfa(stdout, result);
 			printf("\n");
 		}
 #endif
+		SEXP_free(filters_req);
 		goto eval_fail;
 	}
+	SEXP_vfree(filters_req, result, NULL);
 
-	{
-		SEXP_t *filters_j;
+	SEXP_list_foreach(member, filters_u) {
+		SEXP_t *id, *act, *ste;
 
-		filters_j = SEXP_list_join(filters_a, result);
-
-		SEXP_free(filters_a);
-		SEXP_free(result);
-                SEXP_free(filters_u);
-
-		filters_a = filters_j;
+		act = SEXP_list_first(member);
+		id = SEXP_list_nth(member, 2);
+		ste = pcache_sexp_get(OSCAP_GSYM(pcache), id);
+		r0 = SEXP_list_new(act, ste, NULL);
+		SEXP_list_add(filters_a, r0);
+		SEXP_vfree(act, id, ste, r0, NULL);
 	}
 
-	_A((s_subset_i > 0 || o_subset_i > 0));
+	SEXP_free(filters_u);
+
 	_A((s_subset_i > 0 && o_subset_i == 0) || (s_subset_i == 0 && o_subset_i > 0));
 
 	if (o_subset_i > 0) {
@@ -549,8 +680,10 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 
 #ifndef NDEBUG
 			if (s_subset[s_subset_i] == NULL) {
-				_D("FAIL: apply_filters returned NULL: set=%p, filters=%p\n",
-				   o_subset[s_subset_i], filters_a);
+				snprintf(msg, sizeof (msg),
+					 "probe_set_eval: apply_filters returned NULL: set=%p, filters=%p.\n",
+					 o_subset[s_subset_i], filters_a);
+				_D(msg);
 				goto eval_fail;
 			}
 #endif
@@ -581,7 +714,11 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 	SEXP_free(filters_a);
 	SEXP_free(result);
 
-	return (NULL);
+	r0 = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, msg);
+	r1 = SEXP_list_new(r0, NULL);
+	result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+	SEXP_vfree(r0, r1, NULL);
+	return result;
 }
 
 /**
@@ -600,11 +737,11 @@ static void probe_cleanup(void)
 
 /**
  * Signal hook. This function is registered as a signal handler
- * for various signals and its only function is to set a global
+ * for various signals and its only purpose is to set a global
  * flag. This flag is watched by the main loop in the probe and
  * in case it is set, the probe starts the process of a clean
  * shutdown or immediately exits in case the flag was set to
- * PROBE_SIGEXIT_UNCLEAN signaling a fatal error.
+ * PROBE_SIGEXIT_UNCLEAN -- signaling a fatal error.
  */
 static void probe_sigexit(int signum)
 {
@@ -619,6 +756,9 @@ static void probe_sigexit(int signum)
         case SIGABRT:
                 OSCAP_GSYM(sigexit) = PROBE_SIGEXIT_UNCLEAN;
                 break;
+	case SIGUSR1:
+		OSCAP_GSYM(sigexit) = PROBE_SIGEXIT_ABORT;
+		break;
         }
 }
 
@@ -659,7 +799,7 @@ int main(void)
 	SEXP_t *oid;
 
 	pthread_attr_t thread_attr;
-	pthread_t thread;
+	pthread_t      thread;
 
         struct sigaction sigact;
 
@@ -680,7 +820,8 @@ int main(void)
             sigaction(SIGPIPE, &sigact, NULL) != 0 ||
             sigaction(SIGQUIT, &sigact, NULL) != 0 ||
             sigaction(SIGSEGV, &sigact, NULL) != 0 ||
-            sigaction(SIGABRT, &sigact, NULL) != 0)
+            sigaction(SIGABRT, &sigact, NULL) != 0 ||
+	    sigaction(SIGUSR1, &sigact, NULL) != 0)
         {
                 _D("Can't setup signal handlers: errno=%u, %s.\n", errno, strerror(errno));
                 exit(errno);
@@ -738,6 +879,10 @@ int main(void)
                         case PROBE_SIGEXIT_CLEAN:
                                 ret = 0;
                                 break;
+			case PROBE_SIGEXIT_ABORT:
+				ret = ECONNABORTED;
+				exit(ECONNABORTED); /* hack */
+				break;
                         case PROBE_SIGEXIT_UNCLEAN:
                                 /* Try to save cached data & restart? */
                                 _exit(ret);
@@ -814,6 +959,11 @@ int main(void)
                                         SEAP_msg_free(seap_request);
                                         ret = 0;
                                         break;
+				case PROBE_SIGEXIT_ABORT:
+                                        SEAP_msg_free(seap_reply);
+                                        SEAP_msg_free(seap_request);
+					ret = ECONNABORTED;
+					break;
                                 case PROBE_SIGEXIT_UNCLEAN:
                                         /* Try to save cache & restart? */
                                         _exit(ret);
@@ -831,6 +981,18 @@ int main(void)
 		SEAP_msg_free(seap_request);
 	}
 
+	switch(OSCAP_GSYM(sigexit)) {
+	case PROBE_SIGEXIT_CLEAN:
+		ret = 0;
+		break;
+	case PROBE_SIGEXIT_ABORT:
+		ret = ECONNABORTED;
+		break;
+	case PROBE_SIGEXIT_UNCLEAN:
+		/* Try to save cached data & restart? */
+		_exit(ret);
+	}
+
 	return (ret);
 }
 
@@ -883,7 +1045,7 @@ static int probe_varref_create_ctx(const SEXP_t * probe_in, SEXP_t * varrefs, st
 		ent_name = SEXP_list_join(r3, r0);
 		SEXP_vfree(r0, r1, r2, r3, NULL);
 
-		SEXP_sublist_foreach(varref, varrefs, 4, -1) {
+		SEXP_sublist_foreach(varref, varrefs, 4, SEXP_LIST_END) {
 			r0 = SEXP_list_first(varref);
 			if (!SEXP_string_cmp(vid, r0)) {
 				break;
@@ -952,6 +1114,7 @@ static int probe_varref_iterate_ctx(struct probe_varref_ctx *ctx)
 	val_cnt = ent->val_cnt;
 	next_val_idx = &ent->next_val_idx;
 	ent_name_sref = ent->ent_name_sref;
+
 	r0 = SEXP_number_newu(0);
 
 	while (++(*next_val_idx) >= val_cnt) {
@@ -980,8 +1143,8 @@ static int probe_varref_iterate_ctx(struct probe_varref_ctx *ctx)
  */
 void *probe_worker(void *arg)
 {
-	int probe_ret;
-	SEXP_t *probe_in, *set, *probe_out;
+	int probe_ret = 0;
+	SEXP_t *probe_in, *set, *probe_out = NULL, *oid;
 	SEXP_t *varrefs;
 
 	SEAP_msg_t *seap_reply, *seap_request;
@@ -997,29 +1160,21 @@ void *probe_worker(void *arg)
 		/* complex object */
 		probe_out = probe_set_eval(set, 0);
 		SEXP_free(set);
-		probe_ret = (probe_out == NULL ? PROBE_ESETEVAL : 0);
+		// todo: in case of an internal error set probe_ret accordingly
 	} else {
 		/* simple object */
 		varrefs = probe_obj_getent(probe_in, "varrefs", 1);
 
-		if (varrefs == NULL) {
-			SEXP_t *r0;
-
+		if (varrefs == NULL || !OSCAP_GSYM(varref_handling)) {
 			_D("probe_main1\n");
-			probe_ret = -1;
-			probe_out = r0 = probe_main(probe_in, &probe_ret, OSCAP_GSYM(probe_arg));
-			if (r0 != NULL) {
-				probe_out = _probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, r0);
-				SEXP_free(r0);
-			}
-			_A(probe_ret != -1);
+			probe_out = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
+			probe_ret = probe_main(probe_in, probe_out, OSCAP_GSYM(probe_arg));
+			probe_cobj_compute_flag(probe_out);
 		} else {
 			/*
 			 * there are variable references in the object.
 			 * create ctx, iterate through all variable combinations
 			 */
-			SEXP_t *cobj = NULL;
-			SEXP_t *r0, *r1;
 			struct probe_varref_ctx *ctx;
 
 			_D("probe_main2\n");
@@ -1028,61 +1183,48 @@ void *probe_worker(void *arg)
 			SEXP_free(varrefs);
 
 			do {
-				probe_ret = -1;
-				probe_out = probe_main(ctx->pi2, &probe_ret, OSCAP_GSYM(probe_arg));
-				_A(probe_ret != -1);
+				SEXP_t *cobj, *r0;
 
-				if (probe_out == NULL || probe_ret != 0) {
-					SEXP_free(cobj);
-					cobj = NULL;
-					break;
-				}
+				cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
+				probe_ret = probe_main(ctx->pi2, cobj, OSCAP_GSYM(probe_arg));
+				probe_cobj_compute_flag(cobj);
+				r0 = probe_out;
+				probe_out = probe_set_combine(r0, cobj, OVAL_SET_OPERATION_UNION);
+				SEXP_vfree(cobj, r0, NULL);
+			} while (probe_ret == 0
+				 && probe_varref_iterate_ctx(ctx));
 
-				r0 = _probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, probe_out);
-				r1 = cobj;
-				cobj = probe_set_combine (r0, r1, OVAL_SET_OPERATION_UNION);
-
-				SEXP_vfree (probe_out, r0, r1, NULL);
-			} while (probe_varref_iterate_ctx(ctx));
-
-			probe_out = cobj;
 			probe_varref_destroy_ctx(ctx);
 		}
 	}
 
 	_D("probe_out = %p, probe_ret = %d\n", (void *)probe_out, probe_ret);
 
-	if (probe_out == NULL || probe_ret != 0) {
+	SEXP_VALIDATE(probe_out);
+
+	oid = probe_obj_getattrval(probe_in, "id");
+	_A(oid != NULL);
+	SEXP_free(probe_in);
+	if (pcache_sexp_add(OSCAP_GSYM(pcache), oid, probe_out) != 0) {
+		/* TODO */
+		abort();
+	}
+
+	SEXP_free(oid);
+
+	if (probe_ret != 0) {
 		if (SEAP_replyerr(OSCAP_GSYM(ctx), OSCAP_GSYM(sd), seap_request, probe_ret) == -1) {
 			int ret = errno;
 
 			_D("An error ocured while sending error status. errno=%u, %s.\n", errno, strerror(errno));
 
 			SEAP_msg_free(seap_request);
-                        SEXP_free(probe_in);
-
-                        if (probe_out != NULL)
-                                SEXP_free(probe_out);
+			SEXP_free(probe_out);
 
 			/* FIXME */
 			exit(ret);
 		}
 	} else {
-		SEXP_t *oid;
-
-		SEXP_VALIDATE(probe_out);
-
-		oid = probe_obj_getattrval(probe_in, "id");
-		_A(oid != NULL);
-                SEXP_free(probe_in);
-
-		if (pcache_sexp_add(OSCAP_GSYM(pcache), oid, probe_out) != 0) {
-			/* TODO */
-			abort();
-		}
-
-		SEXP_free(oid);
-
 		seap_reply = SEAP_msg_new();
 		SEAP_msg_set(seap_reply, probe_out);
 
@@ -1105,3 +1247,110 @@ void *probe_worker(void *arg)
 	SEAP_msg_free(seap_request);
 	return (NULL);
 }
+
+static int probe_optkcmp(int *a, probe_option_t *b)
+{
+	if (*a > b->option)
+		return  1;
+	if (*a < b->option)
+		return -1;
+	else
+		return  0;
+}
+
+int probe_setoption(int option, ...)
+{
+	va_list ap;
+	probe_option_t *optrec;
+	int ret;
+
+	optrec = oscap_bfind (OSCAP_GSYM(options), OPTIONS_COUNT, sizeof(probe_option_t), &option, (int(*)(void *, void *))&probe_optkcmp);
+
+	if (optrec == NULL) {
+		errno = EINVAL;
+		return (-1);
+	}
+
+	if (optrec->handler == NULL)
+		return 0;
+
+	va_start(ap, option);
+	ret = optrec->handler(option, ap);
+	va_end(ap);
+
+	return (ret);
+}
+
+static int probe_optekcmp(const char *a, char **b)
+{
+	return strcmp(a, *b);
+}
+
+static int probe_optekcmp_sexp(const SEXP_t *a, char **b)
+{
+	return SEXP_strcmp(a, *b);
+}
+
+static int probe_optecmp(char **a, char **b)
+{
+	return strcmp(*a, *b);
+}
+
+static int probe_opthandler_varref(int option, va_list args)
+{
+	bool  o_switch;
+	char *o_name;
+	char *o_temp;
+
+	o_switch = va_arg(args, int);
+	o_name   = va_arg(args, char *);
+
+	if (o_name == NULL) {
+		/* switch varref handling on/off globally */
+		OSCAP_GSYM(varref_handling) = o_switch;
+		return (0);
+	}
+
+	o_temp = oscap_bfind (OSCAP_GSYM(no_varref_ents), OSCAP_GSYM(no_varref_ents_cnt),
+			      sizeof(char *), o_name, (int(*)(void *, void *)) &probe_optecmp);
+
+	if (o_temp != NULL)
+		return (0);
+
+	OSCAP_GSYM(no_varref_ents) = oscap_realloc(OSCAP_GSYM(no_varref_ents),
+						   sizeof (char *) * ++OSCAP_GSYM(no_varref_ents_cnt));
+	OSCAP_GSYM(no_varref_ents)[OSCAP_GSYM(no_varref_ents_cnt) - 1] = strdup(o_name);
+
+	qsort(OSCAP_GSYM(no_varref_ents), OSCAP_GSYM(no_varref_ents_cnt), sizeof (char *), (int(*)(const void *, const void *))&probe_optecmp);
+
+	return (0);
+}
+
+static int probe_opthandler_rcache(int option, va_list args)
+{
+	return (0);
+}
+
+bool probe_handle_varref_cstr (const char *ent_name)
+{
+	if (ent_name == NULL)
+		return OSCAP_GSYM(varref_handling);
+
+	if (OSCAP_GSYM(no_varref_ents_cnt) == 0)
+		return true;
+
+	return (oscap_bfind (OSCAP_GSYM(no_varref_ents), OSCAP_GSYM(no_varref_ents_cnt),
+			     sizeof(char *), (void *)ent_name, (int(*)(void *, void *))&probe_optekcmp) == NULL ? true : false);
+}
+
+bool probe_handle_varref_sexp (const SEXP_t *ent_name)
+{
+	if (ent_name == NULL)
+		return OSCAP_GSYM(varref_handling);
+
+	if (OSCAP_GSYM(no_varref_ents_cnt) == 0)
+		return true;
+
+	return (oscap_bfind (OSCAP_GSYM(no_varref_ents), OSCAP_GSYM(no_varref_ents_cnt),
+			     sizeof(char *), (void *)ent_name, (int(*)(void *, void *))&probe_optekcmp_sexp) == NULL ? true : false);
+}
