@@ -9,7 +9,7 @@
 
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -119,7 +119,8 @@ typedef enum {
 /// Unknown subtypes
 typedef enum {
 	OVAL_SUBTYPE_UNKNOWN = 0,
-	OVAL_SUBTYPE_ALL = 1
+	OVAL_SUBTYPE_ALL = 1,
+        OVAL_SUBTYPE_SYSINFO = 2
 } oval_subtype_t;
 
 /// AIX subtypes
@@ -287,8 +288,8 @@ typedef enum {
 /**
  * @addtogroup OVAL
  * @{
- * @addtogroup OVALVAR OVAL External Variable Binding
- * Public interface to OVAL External Variable Binding
+ * @addtogroup OVALVAR
+ * Interface to Variable model
  * @{
  *
  * @struct oval_variable_model
