@@ -22,10 +22,11 @@
 
 #include <string.h>
 #include "item.h"
+#include <text.h>
 #include "helpers.h"
 #include "xccdf_impl.h"
 #include "common/_error.h"
-#include <text.h>
+#include "common/debug_priv.h"
 
 #define XCCDF_SUPPORTED "1.1.4"
 
@@ -38,8 +39,12 @@ struct xccdf_backref {
 struct xccdf_benchmark *xccdf_benchmark_import(const char *file)
 {
 	xmlTextReaderPtr reader = xmlReaderForFile(file, NULL, 0);
-	if (!reader)
+	if (!reader) {
+                if(errno)
+                        oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, strerror(errno));
+                oscap_dlprintf(DBG_E, "Unable to open file.\n");
 		return NULL;
+	}
 	while (xmlTextReaderRead(reader) == 1 && xmlTextReaderNodeType(reader) != 1) ;
 	struct xccdf_item *benchmark = XITEM(xccdf_benchmark_new());
 	xccdf_benchmark_parse(benchmark, reader);
@@ -72,7 +77,7 @@ struct xccdf_benchmark *xccdf_benchmark_clone(const struct xccdf_benchmark *old_
     xccdf_item_base_clone(&new_benchmark->item, &old->item);
 	new_benchmark->type = old->type;
 	//second argument is a pointer to the benchmark being created which will be the parent of all of its sub elements.
-    xccdf_benchmark_item_clone(&new_benchmark->sub.benchmark, old_benchmark);
+    xccdf_benchmark_item_clone(new_benchmark, old_benchmark);
 	return XBENCHMARK(new_benchmark);
 }
 
@@ -89,6 +94,7 @@ bool xccdf_benchmark_parse(struct xccdf_item * benchmark, xmlTextReaderPtr reade
 	}
 	benchmark->sub.benchmark.style = xccdf_attribute_copy(reader, XCCDFA_STYLE);
 	benchmark->sub.benchmark.style_href = xccdf_attribute_copy(reader, XCCDFA_STYLE_HREF);
+    benchmark->sub.benchmark.lang = (char *) xmlTextReaderXmlLang(reader);
 	if (xccdf_attribute_has(reader, XCCDFA_RESOLVED))
 		benchmark->item.flags.resolved = xccdf_attribute_get_bool(reader, XCCDFA_RESOLVED);
 
@@ -195,6 +201,10 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 	else
 		xmlNewProp(root_node, BAD_CAST "resolved", BAD_CAST "0");
 
+    const char *xmllang = xccdf_benchmark_get_lang(benchmark);
+	if (xmllang)
+		xmlNewProp(root_node, BAD_CAST "xml:lang", BAD_CAST xmllang);
+
 	const char *style = xccdf_benchmark_get_style(benchmark);
 	if (style)
 		xmlNewProp(root_node, BAD_CAST "style", BAD_CAST style);
@@ -225,7 +235,7 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 
 	struct oscap_string_iterator *platforms = xccdf_benchmark_get_platforms(benchmark);
 	while (oscap_string_iterator_has_more(platforms)) {
-		xmlNode *platform_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "platform", NULL);
+		xmlNode *platform_node = xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "platform", NULL);
 
 		const char *idref = oscap_string_iterator_next(platforms);
 		if (idref)
@@ -233,14 +243,18 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 	}
 	oscap_string_iterator_free(platforms);
 
+	const char *version = xccdf_benchmark_get_version(benchmark);
+	if (version)
+		xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
+
 	const char *metadata = xccdf_benchmark_get_metadata(benchmark);
 	if (metadata) {
-		xmlNode *metadata_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "metadata", NULL);
-		xmlNewChild(metadata_node, ns_xccdf, NULL, BAD_CAST metadata);
+		xmlNode *metadata_node = xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "metadata", NULL);
+		xmlNewTextChild(metadata_node, ns_xccdf, NULL, BAD_CAST metadata);
 	}
 
 	OSCAP_FOR(xccdf_model, model, xccdf_benchmark_get_models(benchmark)) {
-		xmlNode *model_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "model", NULL);
+		xmlNode *model_node = xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "model", NULL);
 		xmlNewProp(model_node, BAD_CAST "system", BAD_CAST xccdf_model_get_system(model));
 	}
 
@@ -304,6 +318,7 @@ void xccdf_benchmark_free(struct xccdf_benchmark *benchmark)
 		oscap_free(bench->sub.benchmark.style);
 		oscap_free(bench->sub.benchmark.style_href);
 		oscap_free(bench->sub.benchmark.metadata);
+		oscap_free(bench->sub.benchmark.lang);
 		oscap_list_free(bench->sub.benchmark.front_matter, (oscap_destruct_func) oscap_text_free);
 		oscap_list_free(bench->sub.benchmark.rear_matter, (oscap_destruct_func) oscap_text_free);
 		oscap_list_free(bench->sub.benchmark.notices, (oscap_destruct_func) xccdf_notice_free);
@@ -312,8 +327,8 @@ void xccdf_benchmark_free(struct xccdf_benchmark *benchmark)
 		oscap_list_free(bench->sub.benchmark.values, (oscap_destruct_func) xccdf_value_free);
 		oscap_list_free(bench->sub.benchmark.results, (oscap_destruct_func) xccdf_result_free);
 		oscap_list_free(bench->sub.benchmark.plain_texts, (oscap_destruct_func) xccdf_plain_text_free);
-		oscap_htable_free(bench->sub.benchmark.dict, NULL);
 		oscap_list_free(bench->sub.benchmark.profiles, (oscap_destruct_func) xccdf_profile_free);
+		oscap_htable_free(bench->sub.benchmark.dict, NULL);
 		xccdf_item_release(bench);
 	}
 }
@@ -321,6 +336,7 @@ void xccdf_benchmark_free(struct xccdf_benchmark *benchmark)
 XCCDF_ACCESSOR_STRING(benchmark, metadata)
 XCCDF_ACCESSOR_STRING(benchmark, style)
 XCCDF_ACCESSOR_STRING(benchmark, style_href)
+XCCDF_ACCESSOR_STRING(benchmark, lang)
 XCCDF_LISTMANIP_TEXT(benchmark, front_matter, front_matter)
 XCCDF_LISTMANIP_TEXT(benchmark, rear_matter, rear_matter)
 XCCDF_LISTMANIP(benchmark, notice, notices)
@@ -336,7 +352,7 @@ XCCDF_LISTMANIP(benchmark, plain_text, plain_texts)
 XCCDF_HTABLE_GETTER(struct xccdf_item *, benchmark, item, sub.benchmark.dict)
 XCCDF_STATUS_CURRENT(benchmark)
 OSCAP_ITERATOR_GEN(xccdf_plain_text)
-
+OSCAP_ITERATOR_REMOVE_F(xccdf_plain_text)
 
 XCCDF_ITEM_ADDER_REG(benchmark, result, results)
 XCCDF_ITEM_ADDER_REG(benchmark, rule, content)
@@ -497,20 +513,26 @@ bool xccdf_benchmark_register_item(struct xccdf_benchmark *benchmark, struct xcc
 	struct xccdf_item *found = xccdf_benchmark_get_item(benchmark, id);
 	if (found != NULL) return found == item; // already registered
 
+    if (item->type == XCCDF_GROUP) {
+        OSCAP_FOR(xccdf_item, cnt, xccdf_group_get_content(XGROUP(item)))
+            xccdf_benchmark_register_item(benchmark, cnt);
+        OSCAP_FOR(xccdf_value, val, xccdf_group_get_values(XGROUP(item)))
+            xccdf_benchmark_register_item(benchmark, XITEM(val));
+    }
+
 	return oscap_htable_add(XITEM(benchmark)->sub.benchmark.dict, xccdf_item_get_id(item), item);
 }
 
 bool xccdf_benchmark_unregister_item(struct xccdf_item *item)
 {
-	if (item == NULL)
-		return false;
+	if (item == NULL) return false;
 
-	if (xccdf_item_get_benchmark(item) == NULL)
-		return false;
+	struct xccdf_benchmark *bench = xccdf_item_get_benchmark(item);
+	if (bench == NULL) return false;
 
-	assert(xccdf_benchmark_get_item(xccdf_item_get_benchmark(item), xccdf_item_get_id(item)) == item);
+	assert(xccdf_benchmark_get_item(bench, xccdf_item_get_id(item)) == item);
 
-	return oscap_htable_detach(XITEM(xccdf_item_get_benchmark(item))->sub.benchmark.dict, xccdf_item_get_id(item)) != NULL;
+	return oscap_htable_detach(XITEM(bench)->sub.benchmark.dict, xccdf_item_get_id(item)) != NULL;
 }
 
 bool xccdf_benchmark_rename_item(struct xccdf_item *item, const char *newid)
