@@ -113,7 +113,7 @@ struct xccdf_item *xccdf_item_clone(const struct xccdf_item *old_item)
 	switch (new_item->type) {
 	case XCCDF_BENCHMARK:
 		 //new_item->sub.benchmark = *(xccdf_benchmark_item_clone(&(old_item->sub.benchmark), new_item));
-        xccdf_benchmark_item_clone(&new_item->sub.benchmark, XBENCHMARK(old_item));
+        xccdf_benchmark_item_clone(new_item, XBENCHMARK(old_item));
 		break;
 	case XCCDF_RULE:
 	    xccdf_rule_item_clone(&new_item->sub.rule, &old_item->sub.rule);
@@ -158,7 +158,7 @@ void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_
 
 	new_base->warnings = oscap_list_clone(old_base->warnings, (oscap_clone_func) xccdf_warning_clone);
 	new_base->statuses = oscap_list_clone(old_base->statuses, (oscap_clone_func) xccdf_status_clone);
-	new_base->references = oscap_list_clone(old_base->references, (oscap_clone_func) xccdf_reference_clone);
+	new_base->references = oscap_list_clone(old_base->references, (oscap_clone_func) oscap_reference_clone);
 	new_base->platforms = oscap_list_clone(old_base->platforms, (oscap_clone_func) oscap_strdup);
 
 	/* Handling flags */
@@ -166,20 +166,6 @@ void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_
 	new_base->defined_flags = old_base->defined_flags;
 }
 
-
-
-/* Performs a deep copy of xccdf_reference and returns a pointer to that copy */
-struct xccdf_reference *xccdf_reference_clone(const struct xccdf_reference *old_reference)
-{
-	struct xccdf_reference *new_reference = oscap_calloc(1, sizeof(struct xccdf_reference));
-	//new_reference->override = old_reference->override;
-	new_reference->href = oscap_strdup(old_reference->href);
-	new_reference->text = oscap_text_clone(old_reference->text);
-	//new_reference->content = oscap_strdup(old_reference->content);
-	//new_reference->lang = oscap_strdup(old_reference->lang);
-	return new_reference;
-}
-
 /* Performs a deep copy of xccdf_status and returns a pointer to that copy */
 struct xccdf_status *xccdf_status_clone(const struct xccdf_status *old_status)
 {
@@ -210,7 +196,8 @@ void xccdf_item_release(struct xccdf_item *item)
 		oscap_list_free(item->item.rationale, (oscap_destruct_func) oscap_text_free);
 		oscap_list_free(item->item.question, (oscap_destruct_func) oscap_text_free);
 		oscap_list_free(item->item.warnings, (oscap_destruct_func) xccdf_warning_free);
-		oscap_list_free(item->item.references, (oscap_destruct_func) xccdf_reference_free);
+		oscap_list_free(item->item.references, (oscap_destruct_func) oscap_reference_free);
+		if (item->type != XCCDF_BENCHMARK) xccdf_benchmark_unregister_item(item);
 		oscap_free(item->item.id);
 		oscap_free(item->item.cluster_id);
 		oscap_free(item->item.version_update);
@@ -316,7 +303,7 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	if (parent == NULL)
 		item_node = xmlNewNode(NULL, BAD_CAST "Item");
 	else
-		item_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "Item", NULL);
+		item_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "Item", NULL);
 
 	/* Handle generic item attributes */
 	const char *id = xccdf_item_get_id(item);
@@ -327,21 +314,13 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 		xmlNewProp(item_node, BAD_CAST "cluster-id", BAD_CAST cluster_id);
 
 	if (xccdf_item_get_hidden(item))
-		xmlNewProp(item_node, BAD_CAST "hidden", BAD_CAST "True");
+		xmlNewProp(item_node, BAD_CAST "hidden", BAD_CAST "true");
 
 	if (xccdf_item_get_prohibit_changes(item))
-		xmlNewProp(item_node, BAD_CAST "prohibitChanges", BAD_CAST "True");
+		xmlNewProp(item_node, BAD_CAST "prohibitChanges", BAD_CAST "true");
 
 	if (xccdf_item_get_abstract(item))
-		xmlNewProp(item_node, BAD_CAST "abstract", BAD_CAST "True");
-
-	/* Handle generic item child nodes */
-	xccdf_texts_to_dom(xccdf_item_get_title(item), item_node, "title");
-	xccdf_texts_to_dom(xccdf_item_get_description(item), item_node, "description");
-
-	const char *version= xccdf_item_get_version(item);
-	if (version)
-		xmlNewChild(item_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
+		xmlNewProp(item_node, BAD_CAST "abstract", BAD_CAST "true");
 
 	struct xccdf_status_iterator *statuses = xccdf_item_get_statuses(item);
 	while (xccdf_status_iterator_has_more(statuses)) {
@@ -350,14 +329,22 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	}
 	xccdf_status_iterator_free(statuses);
 
+	/* Handle generic item child nodes */
+	xccdf_texts_to_dom(xccdf_item_get_title(item), item_node, "title");
+	xccdf_texts_to_dom(xccdf_item_get_description(item), item_node, "description");
+
+	const char *version = xccdf_item_get_version(item);
+	if (version && xccdf_item_get_type(item) != XCCDF_BENCHMARK)
+		xmlNewTextChild(item_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
+
 	xccdf_texts_to_dom(xccdf_item_get_question(item), item_node, "question");
 
-	struct xccdf_reference_iterator *references = xccdf_item_get_references(item);
-	while (xccdf_reference_iterator_has_more(references)) {
-		struct xccdf_reference *ref = xccdf_reference_iterator_next(references);
-		xccdf_reference_to_dom(ref, doc, item_node);
+	struct oscap_reference_iterator *references = xccdf_item_get_references(item);
+	while (oscap_reference_iterator_has_more(references)) {
+		struct oscap_reference *ref = oscap_reference_iterator_next(references);
+		oscap_reference_to_dom(ref, item_node, doc, "reference");
 	}
-        xccdf_reference_iterator_free(references);
+    oscap_reference_iterator_free(references);
 
 	/* Handle type specific attributes and children */
 	switch (xccdf_item_get_type(item)) {
@@ -397,16 +384,6 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	return item_node;
 }
 
-xmlNode *xccdf_reference_to_dom(struct xccdf_reference *ref, xmlDoc *doc, xmlNode *parent)
-{
-	xmlNode *reference_node = oscap_text_to_dom(xccdf_reference_get_text(ref), parent, "reference");
-	xmlNewProp(reference_node, BAD_CAST "href", BAD_CAST xccdf_reference_get_href(ref));
-
-        /* TODO: Dublin Core Elements /XML spec p. 69/ */
-
-	return reference_node;
-}
-
 xmlNode *xccdf_profile_note_to_dom(struct xccdf_profile_note *note, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNode *note_node = oscap_text_to_dom(xccdf_profile_note_get_text(note), parent, "profile-note");
@@ -422,7 +399,7 @@ xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *
 	xmlNode *status_node = NULL;
 	xccdf_status_type_t type = xccdf_status_get_status(status);
 	if (type != XCCDF_STATUS_NOT_SPECIFIED)
-		status_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "status",
+		status_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "status",
 							BAD_CAST XCCDF_STATUS_MAP[type - 1].string);
 
 	time_t date_time = xccdf_status_get_date(status);
@@ -442,7 +419,8 @@ xmlNode *xccdf_fixtext_to_dom(struct xccdf_fixtext *fixtext, xmlDoc *doc, xmlNod
 		xmlNewProp(fixtext_node, BAD_CAST "reboot", BAD_CAST "True");
 
 	const char *fixref = xccdf_fixtext_get_fixref(fixtext);
-	xmlNewProp(fixtext_node, BAD_CAST "fixref", BAD_CAST fixref);
+    if (fixref)
+	    xmlNewProp(fixtext_node, BAD_CAST "fixref", BAD_CAST fixref);
 
 	xccdf_level_t complexity = xccdf_fixtext_get_complexity(fixtext);
         if (complexity != 0)
@@ -485,11 +463,11 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 	        xmlNewProp(fix_node, BAD_CAST "strategy", BAD_CAST XCCDF_STRATEGY_MAP[strategy-1].string);
 
         // Sub element is used to store XCCDF value substitutions, not a content
-	//xmlNewChild(fix_node, ns_xccdf, BAD_CAST "sub", BAD_CAST content);
+	//xmlNewTextChild(fix_node, ns_xccdf, BAD_CAST "sub", BAD_CAST content);
 
 	// This is in the XCCDF Spec, but not implemented in OpenSCAP
 	//const char *instance = xccdf_fix_get_instance(fix);
-	//xmlNewChild(fix_node, ns_xccdf, BAD_CAST "instance", BAD_CAST instance);
+	//xmlNewTextChild(fix_node, ns_xccdf, BAD_CAST "instance", BAD_CAST instance);
 	
 	return fix_node;
 }
@@ -498,7 +476,7 @@ xmlNode *xccdf_ident_to_dom(struct xccdf_ident *ident, xmlDoc *doc, xmlNode *par
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
 	const char *id = xccdf_ident_get_id(ident);
-	xmlNode *ident_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "ident", BAD_CAST id);
+	xmlNode *ident_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "ident", BAD_CAST id);
 
 	const char *sys = xccdf_ident_get_system(ident);
 	xmlNewProp(ident_node, BAD_CAST "system", BAD_CAST sys);
@@ -511,9 +489,9 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
 	xmlNode *check_node = NULL;
 	if (xccdf_check_get_complex(check))
-		check_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "complex-check", NULL);
+		check_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "complex-check", NULL);
 	else
-		check_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "check", NULL);
+		check_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "check", NULL);
 
 	const char *id = xccdf_check_get_id(check);
 	if (id)
@@ -539,7 +517,7 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 		struct xccdf_check_import *import = xccdf_check_import_iterator_next(imports);
 		const char *name = xccdf_check_import_get_name(import);
 		const char *content = xccdf_check_import_get_content(import);
-		xmlNode *import_node = xmlNewChild(check_node, ns_xccdf, BAD_CAST "check-import", BAD_CAST content);
+		xmlNode *import_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-import", BAD_CAST content);
 		xmlNewProp(import_node, BAD_CAST "import-name", BAD_CAST name);
 	}
 	xccdf_check_import_iterator_free(imports);
@@ -551,7 +529,7 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 		//const char *name = xccdf_check_export_get_name(export);
 		const char *name = export->name;
 		const char *value= xccdf_check_export_get_value(export);
-		xmlNode *export_node = xmlNewChild(check_node, ns_xccdf, BAD_CAST "check-export", NULL);
+		xmlNode *export_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-export", NULL);
 		xmlNewProp(export_node, BAD_CAST "export-name", BAD_CAST name);
 		xmlNewProp(export_node, BAD_CAST "value-id", BAD_CAST value);
 	}
@@ -559,12 +537,12 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 
 	const char *content = xccdf_check_get_content(check);
 	if (content)
-		xmlNewChild(check_node, ns_xccdf, BAD_CAST "check-content", BAD_CAST content);
+		xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-content", BAD_CAST content);
 
 	struct xccdf_check_content_ref_iterator *refs = xccdf_check_get_content_refs(check);
 	while (xccdf_check_content_ref_iterator_has_more(refs)) {
 		struct xccdf_check_content_ref *ref = xccdf_check_content_ref_iterator_next(refs);
-		xmlNode *ref_node = xmlNewChild(check_node, ns_xccdf, BAD_CAST "check-content-ref", NULL);
+		xmlNode *ref_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-content-ref", NULL);
 
 		const char *name = xccdf_check_content_ref_get_name(ref);
 		xmlNewProp(ref_node, BAD_CAST "name", BAD_CAST name);
@@ -625,7 +603,7 @@ bool xccdf_item_process_element(struct xccdf_item * item, xmlTextReaderPtr reade
         oscap_list_add(item->item.warnings, xccdf_warning_new_parse(reader));
 		return true;
 	case XCCDFE_REFERENCE:
-        oscap_list_add(item->item.references, xccdf_reference_new_parse(reader));
+        oscap_list_add(item->item.references, oscap_reference_new_parse(reader));
 		return true;
 	case XCCDFE_STATUS:{
         const char *date = xccdf_attribute_get(reader, XCCDFA_DATE);
@@ -715,14 +693,14 @@ XCCDF_FLAG_GETTER(abstract)
 XCCDF_FLAG_GETTER(interactive)
 XCCDF_ITEM_SIGETTER(platforms)
 XCCDF_ITEM_ADDER_STRING(platform, platforms)
-XCCDF_ITEM_IGETTER(reference, references)
 XCCDF_ITEM_IGETTER(warning, warnings)
 XCCDF_ITEM_IGETTER(status, statuses)
-XCCDF_ITEM_ADDER(struct xccdf_reference *, reference, references)
+XCCDF_ITEM_ADDER(struct oscap_reference *, reference, references)
 XCCDF_ITEM_ADDER(struct xccdf_warning *, warning, warnings)
 XCCDF_ITEM_ADDER(struct xccdf_status *, status, statuses)
-XCCDF_ITERATOR_GEN_S(item) XCCDF_ITERATOR_GEN_S(status) XCCDF_ITERATOR_GEN_S(reference)
-OSCAP_ITERATOR_GEN_T(struct xccdf_warning *, xccdf_warning)
+XCCDF_ITERATOR_GEN_S(item) XCCDF_ITERATOR_GEN_S(status)
+OSCAP_ITERATOR_GEN(xccdf_warning)
+OSCAP_ITERATOR_REMOVE_F(xccdf_warning)
 
 XCCDF_ITEM_SETTER_SIMPLE(xccdf_numeric, weight)
 XCCDF_ITEM_SETTER_SIMPLE(time_t, version_time)
@@ -737,6 +715,13 @@ XCCDF_SETTER_ID(item) XCCDF_SETTER_ID(benchmark) XCCDF_SETTER_ID(profile)
 XCCDF_SETTER_ID(rule) XCCDF_SETTER_ID(group) XCCDF_SETTER_ID(value) XCCDF_SETTER_ID(result)
 #undef XCCDF_SETTER_ID
 
+struct oscap_reference_iterator *xccdf_item_get_references(const struct xccdf_item *item) { return oscap_iterator_new(item->item.references); }
+struct oscap_reference_iterator *xccdf_benchmark_get_references(const struct xccdf_benchmark *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_value_get_references(const struct xccdf_value *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_group_get_references(const struct xccdf_group *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_rule_get_references(const struct xccdf_rule *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_profile_get_references(const struct xccdf_profile *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+
 struct xccdf_item_iterator *xccdf_item_get_content(const struct xccdf_item *item)
 {
 	if (item == NULL) return NULL;
@@ -897,35 +882,6 @@ void xccdf_warning_free(struct xccdf_warning * w)
 OSCAP_ACCESSOR_SIMPLE(xccdf_warning_category_t, xccdf_warning, category)
 OSCAP_ACCESSOR_TEXT(xccdf_warning, text)
 
-struct xccdf_reference *xccdf_reference_new(void)
-{
-    struct xccdf_reference *ref = oscap_calloc(1, sizeof(struct xccdf_reference));
-    return ref;
-}
-
-struct xccdf_reference *xccdf_reference_new_parse(xmlTextReaderPtr reader)
-{
-    struct xccdf_reference *ref = xccdf_reference_new();
-    //if (xccdf_attribute_has(reader, XCCDFA_OVERRIDE))
-    //        ref->override = oscap_string_to_enum(OSCAP_BOOL_MAP, xccdf_attribute_get(reader, XCCDFA_OVERRIDE));
-    ref->href = xccdf_attribute_copy(reader, XCCDFA_HREF);
-	ref->text = oscap_text_new_parse(OSCAP_TEXT_TRAITS_HTML, reader);
-    //ref->content = oscap_element_string_copy(reader);
-    // TODO Dublin Core
-    return ref;
-}
-
-void xccdf_reference_free(struct xccdf_reference *ref)
-{
-    if (ref != NULL) {
-        //oscap_free(ref->lang);
-        oscap_free(ref->href);
-		oscap_text_free(ref->text);
-        //oscap_free(ref->content);
-        oscap_free(ref);
-    }
-}
-
 //clones the specific types of items
 void xccdf_profile_item_clone(struct xccdf_profile_item *clone, const struct xccdf_profile_item * item)
 {
@@ -936,10 +892,10 @@ void xccdf_profile_item_clone(struct xccdf_profile_item *clone, const struct xcc
 	clone->refine_rules = oscap_list_clone(item->refine_rules, (oscap_clone_func) xccdf_refine_rule_clone);
 }
 
-struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_benchmark_item *clone, const struct xccdf_benchmark * bench)
+struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_item *parent, const struct xccdf_benchmark * bench)
 {
     struct xccdf_benchmark_item *item = &XITEM(bench)->sub.benchmark;
-    struct xccdf_item *parent = XITEM(bench);
+    struct xccdf_benchmark_item *clone = &parent->sub.benchmark;
 
 	clone->dict = oscap_htable_new();
 	clone->notices = oscap_list_clone(item->notices, (oscap_clone_func) xccdf_notice_clone);
@@ -947,6 +903,7 @@ struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_benchmark_
 	
 	clone->style = oscap_strdup(item->style);
 	clone->style_href = oscap_strdup(item->style_href);
+	clone->lang = oscap_strdup(item->lang);
 	
 	clone->rear_matter  = oscap_list_clone(item->rear_matter, (oscap_clone_func) oscap_text_clone);
 	clone->front_matter = oscap_list_clone(item->front_matter, (oscap_clone_func) oscap_text_clone);
@@ -991,55 +948,18 @@ void xccdf_group_item_clone(struct xccdf_item *parent, const struct xccdf_group_
 	xccdf_reparent_list(clone->content, parent);
 }
 
-union xccdf_value_unit *xccdf_value_unit_clone_str(const union xccdf_value_unit *unit)
-{
-	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
-	val->s = oscap_strdup(unit->s);
-	return val;
-}
-
-union xccdf_value_unit *xccdf_value_unit_clone_numeric(const union xccdf_value_unit *unit)
-{
-	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
-	val->n = unit->n;
-	return val;
-}
-
-union xccdf_value_unit *xccdf_value_unit_clone_bool(const union xccdf_value_unit *unit)
-{
-	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
-	val->b = unit->b;
-	return val;
-}
-
 struct xccdf_value_instance * xccdf_value_instance_clone(const struct xccdf_value_instance * val)
 {
 	struct xccdf_value_instance * clone = oscap_calloc(1, sizeof(struct xccdf_value_instance));
     clone->type = val->type;
 	
-	switch (val->type) {
-	case XCCDF_TYPE_STRING:
-		clone->value.s = oscap_strdup(val->value.s);
-		clone->defval.s = oscap_strdup(val->defval.s);
-		clone->choices = oscap_list_clone(val->choices, (oscap_clone_func) xccdf_value_unit_clone_str);
-		clone->limits.s.match = oscap_strdup(val->limits.s.match);
-		break;
-	case XCCDF_TYPE_NUMBER:
-		clone->value.n = val->value.n;
-		clone->defval.n = val->defval.n;
-		clone->choices = oscap_list_clone(val->choices, (oscap_clone_func) xccdf_value_unit_clone_numeric);
-		clone->limits.n.lower_bound = val->limits.n.lower_bound;
-		clone->limits.n.upper_bound = val->limits.n.upper_bound;
-		break;
-	case XCCDF_TYPE_BOOLEAN:
-		clone->value.b = val->value.b;
-		clone->defval.b = val->defval.b;
-		clone->choices = oscap_list_clone(val->choices, (oscap_clone_func) xccdf_value_unit_clone_bool);
-		break;
-	default:
-		assert(false);
-	}
-	
+    clone->value = oscap_strdup(val->value);
+    clone->defval = oscap_strdup(val->defval);
+    clone->choices = oscap_list_clone(val->choices, (oscap_clone_func) oscap_strdup);
+    clone->match = oscap_strdup(val->match);
+    clone->lower_bound = val->lower_bound;
+    clone->upper_bound = val->upper_bound;
+
 	clone->flags = val->flags;
 	xccdf_value_instance_set_selector(clone, xccdf_value_instance_get_selector(val));
 	return clone;
@@ -1172,9 +1092,6 @@ void xccdf_reparent_item(struct xccdf_item * item, struct xccdf_item * parent)
 	}
 }
 
-OSCAP_ACCESSOR_STRING(xccdf_reference, href)
-OSCAP_ACCESSOR_TEXT(xccdf_reference, text)
-
 const struct oscap_text_traits XCCDF_TEXT_PLAIN    = { .can_override = true };
 const struct oscap_text_traits XCCDF_TEXT_HTML     = { .html = true, .can_override = true };
 const struct oscap_text_traits XCCDF_TEXT_PLAINSUB = { .can_override = true, .can_substitute = true };
