@@ -119,6 +119,7 @@ XCCDF_LISTMANIP(result, setvalue, setvalues)
 XCCDF_LISTMANIP(result, rule_result, rule_results)
 XCCDF_LISTMANIP(result, score, scores)
 OSCAP_ITERATOR_GEN(xccdf_result)
+OSCAP_ITERATOR_REMOVE_F(xccdf_result)
 
 
 struct xccdf_rule_result *xccdf_rule_result_new(void)
@@ -157,9 +158,13 @@ OSCAP_IGETINS(xccdf_ident, xccdf_rule_result, idents, ident)
 OSCAP_IGETINS(xccdf_fix, xccdf_rule_result, fixes, fix)
 OSCAP_IGETINS(xccdf_check, xccdf_rule_result, checks, check)
 OSCAP_IGETINS_GEN(xccdf_override, xccdf_rule_result, overrides, override)
+OSCAP_ITERATOR_REMOVE_F(xccdf_override)
 OSCAP_IGETINS_GEN(xccdf_message, xccdf_rule_result, messages, message)
+OSCAP_ITERATOR_REMOVE_F(xccdf_message)
 OSCAP_IGETINS_GEN(xccdf_instance, xccdf_rule_result, instances, instance)
+OSCAP_ITERATOR_REMOVE_F(xccdf_instance)
 OSCAP_ITERATOR_GEN(xccdf_rule_result)
+OSCAP_ITERATOR_REMOVE_F(xccdf_rule_result)
 
 
 struct xccdf_identity *xccdf_identity_new(void)
@@ -179,6 +184,7 @@ OSCAP_ACCESSOR_EXP(bool, xccdf_identity, authenticated, sub.authenticated)
 OSCAP_ACCESSOR_EXP(bool, xccdf_identity, privileged, sub.privileged)
 OSCAP_ACCESSOR_STRING(xccdf_identity, name)
 OSCAP_ITERATOR_GEN(xccdf_identity)
+OSCAP_ITERATOR_REMOVE_F(xccdf_identity)
 
 struct xccdf_score *xccdf_score_new(void)
 {
@@ -200,6 +206,7 @@ OSCAP_ACCESSOR_SIMPLE(xccdf_numeric, xccdf_score, maximum)
 OSCAP_ACCESSOR_SIMPLE(xccdf_numeric, xccdf_score, score)
 OSCAP_ACCESSOR_STRING(xccdf_score, system)
 OSCAP_ITERATOR_GEN(xccdf_score)
+OSCAP_ITERATOR_REMOVE_F(xccdf_score)
 
 struct xccdf_override *xccdf_override_new(void)
 {
@@ -281,6 +288,7 @@ OSCAP_GETTER(xccdf_value_type_t, xccdf_target_fact, type)
 OSCAP_GETTER(const char *, xccdf_target_fact, value)
 OSCAP_ACCESSOR_STRING(xccdf_target_fact, name)
 OSCAP_ITERATOR_GEN(xccdf_target_fact)
+OSCAP_ITERATOR_REMOVE_F(xccdf_target_fact)
 
 struct xccdf_instance *xccdf_instance_new(void)
 {
@@ -464,30 +472,30 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 	struct oscap_string_iterator *orgs = xccdf_result_get_organizations(result);
 	while (oscap_string_iterator_has_more(orgs)) {
 		const char *org = oscap_string_iterator_next(orgs);
-		xmlNewChild(result_node, ns_xccdf, BAD_CAST "organization", BAD_CAST org);
+		xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "organization", BAD_CAST org);
 	}
 	oscap_string_iterator_free(orgs);
 
 	struct xccdf_identity_iterator *identities = xccdf_result_get_identities(result);
 	while (xccdf_identity_iterator_has_more(identities)) {
 		struct xccdf_identity *identity = xccdf_identity_iterator_next(identities);
-		xmlNode *identity_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "identity", BAD_CAST xccdf_identity_get_name(identity));
+		xmlNode *identity_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "identity", BAD_CAST xccdf_identity_get_name(identity));
 		
 		if (xccdf_identity_get_authenticated(identity))
-			xmlNewProp(identity_node, BAD_CAST "authenticated", BAD_CAST "True");
+			xmlNewProp(identity_node, BAD_CAST "authenticated", BAD_CAST "true");
 		else
-			xmlNewProp(identity_node, BAD_CAST "authenticated", BAD_CAST "False");
+			xmlNewProp(identity_node, BAD_CAST "authenticated", BAD_CAST "false");
 
 		if (xccdf_identity_get_privileged(identity))
-			xmlNewProp(identity_node, BAD_CAST "privileged", BAD_CAST "True");
+			xmlNewProp(identity_node, BAD_CAST "privileged", BAD_CAST "true");
 		else
-			xmlNewProp(identity_node, BAD_CAST "privileged", BAD_CAST "False");
+			xmlNewProp(identity_node, BAD_CAST "privileged", BAD_CAST "false");
 	}
 	xccdf_identity_iterator_free(identities);
 
 	const char *profile = xccdf_result_get_profile(result);
 	if (profile) {
-		xmlNode *prof_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "profile", NULL);
+		xmlNode *prof_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "profile", NULL);
 		xmlNewProp(prof_node, BAD_CAST "idref", BAD_CAST profile);
 	}
 
@@ -501,23 +509,25 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 	struct oscap_string_iterator *targets = xccdf_result_get_targets(result);
 	while (oscap_string_iterator_has_more(targets)) {
 		const char *target = oscap_string_iterator_next(targets);
-		xmlNewChild(result_node, ns_xccdf, BAD_CAST "target", BAD_CAST target);
+		xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "target", BAD_CAST target);
 	}
 	oscap_string_iterator_free(targets);
 
 	struct oscap_string_iterator *target_addresses = xccdf_result_get_target_addresses(result);
 	while (oscap_string_iterator_has_more(target_addresses)) {
 		const char *target_address = oscap_string_iterator_next(target_addresses);
-		xmlNewChild(result_node, ns_xccdf, BAD_CAST "target_address", BAD_CAST target_address);
+		xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "target-address", BAD_CAST target_address);
 	}
 	oscap_string_iterator_free(target_addresses);
 
+    // platform
+
 	struct xccdf_target_fact_iterator *target_facts = xccdf_result_get_target_facts(result);
         if (xccdf_target_fact_iterator_has_more(target_facts)) {
-                xmlNode *target_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "target_facts", NULL);
+                xmlNode *target_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "target-facts", NULL);
                 while (xccdf_target_fact_iterator_has_more(target_facts)) {
                         struct xccdf_target_fact *target_fact = xccdf_target_fact_iterator_next(target_facts);
-                        xmlNode *fact_node = xmlNewChild(target_node, ns_xccdf, BAD_CAST "fact", BAD_CAST target_fact->value);
+                        xmlNode *fact_node = xmlNewTextChild(target_node, ns_xccdf, BAD_CAST "fact", BAD_CAST target_fact->value);
 
                         const char *name = xccdf_target_fact_get_name(target_fact);
                         if (name)
@@ -541,18 +551,18 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 		struct xccdf_score *score = xccdf_score_iterator_next(scores);
 
                 float value = xccdf_score_get_score(score);
-                char value_str[10];
-                sprintf(value_str, "%f", value);
-                xmlNode *score_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "score", BAD_CAST value_str);
+                char *value_str = oscap_sprintf("%f", value);
+                xmlNode *score_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "score", BAD_CAST value_str);
+                oscap_free(value_str);
 
 		const char *sys = xccdf_score_get_system(score);
 		if (sys)
 			xmlNewProp(score_node, BAD_CAST "system", BAD_CAST sys);
 
 		float max = xccdf_score_get_maximum(score);
-		char max_str[10];
-		sprintf(max_str, "%f", max);
+		char *max_str = oscap_sprintf("%f", max);
 		xmlNewProp(score_node, BAD_CAST "maximum", BAD_CAST max_str);
+        oscap_free(max_str);
 	}
 	xccdf_score_iterator_free(scores);
 }
@@ -641,7 +651,7 @@ static struct xccdf_rule_result *xccdf_rule_result_new_parse(xmlTextReaderPtr re
 xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *result_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "rule-result", NULL);
+	xmlNode *result_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "rule-result", NULL);
 
 	/* Handle attributes */
 	const char *idref = xccdf_rule_result_get_idref(result);
@@ -670,14 +680,14 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 		xmlNewProp(result_node, BAD_CAST "version", BAD_CAST version);
 
 	float weight = xccdf_rule_result_get_weight(result);
-	char weight_str[10];
-	sprintf(weight_str, "%f", weight);
+	char *weight_str = oscap_sprintf("%f", weight);
 	xmlNewProp(result_node, BAD_CAST "weight", BAD_CAST weight_str);
+    oscap_free(weight_str);
 
 	/* Handle children */
 	xccdf_test_result_type_t test_res = xccdf_rule_result_get_result(result);
 	if (test_res != 0)
-            xmlNewChild(result_node, ns_xccdf, BAD_CAST "result", BAD_CAST XCCDF_RESULT_MAP[test_res - 1].string);
+            xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "result", BAD_CAST XCCDF_RESULT_MAP[test_res - 1].string);
 
 	struct xccdf_override_iterator *overrides = xccdf_rule_result_get_overrides(result);
 	while (xccdf_override_iterator_has_more(overrides)) {
@@ -697,7 +707,7 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	while (xccdf_message_iterator_has_more(messages)) {
 		struct xccdf_message *message = xccdf_message_iterator_next(messages);
 		const char *content = xccdf_message_get_content(message);
-		xmlNode *message_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "message", BAD_CAST content);
+		xmlNode *message_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "message", BAD_CAST content);
 
                 xccdf_level_t message_severity = xccdf_message_get_severity(message);
 		if (message_severity != 0)
@@ -709,7 +719,7 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	while (xccdf_instance_iterator_has_more(instances)) {
 		struct xccdf_instance *instance = xccdf_instance_iterator_next(instances);
 		const char *content = xccdf_instance_get_content(instance);
-		xmlNode *instance_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "instance", BAD_CAST content);
+		xmlNode *instance_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "instance", BAD_CAST content);
 		
 		const char *context = xccdf_instance_get_context(instance);
 		if (context)
@@ -772,7 +782,7 @@ static struct xccdf_override *xccdf_override_new_parse(xmlTextReaderPtr reader)
 xmlNode *xccdf_override_to_dom(struct xccdf_override *override, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *override_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "override", NULL);
+	xmlNode *override_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "override", NULL);
 
 	/* Handle attributes */
 	time_t date = xccdf_override_get_time(override);
@@ -786,13 +796,14 @@ xmlNode *xccdf_override_to_dom(struct xccdf_override *override, xmlDoc *doc, xml
 	if (authority)
 		xmlNewProp(override_node, BAD_CAST "authority", BAD_CAST authority);
 
+	/* Handle children */
 	xccdf_test_result_type_t old = xccdf_override_get_old_result(override);
 	if (old != 0)
-		xmlNewProp(override_node, BAD_CAST "old-result", BAD_CAST XCCDF_RESULT_MAP[old - 1].string);
+		xmlNewTextChild(override_node, ns_xccdf, BAD_CAST "old-result", BAD_CAST XCCDF_RESULT_MAP[old - 1].string);
 
 	xccdf_test_result_type_t new = xccdf_override_get_new_result(override);
 	if (new != 0)
-		xmlNewProp(override_node, BAD_CAST "new-result", BAD_CAST XCCDF_RESULT_MAP[new - 1].string);
+		xmlNewTextChild(override_node, ns_xccdf, BAD_CAST "new-result", BAD_CAST XCCDF_RESULT_MAP[new - 1].string);
 
 	oscap_text_to_dom(xccdf_override_get_remark(override), override_node, "remark");
 
