@@ -24,6 +24,7 @@
 #ifndef NDEBUG
 # include <stdio.h>
 # include <stdarg.h>
+# include <string.h>
 # include <stdlib.h>
 # include <sys/types.h>
 # include <sys/file.h>
@@ -32,12 +33,15 @@
 # include "public/debug.h"
 # include "debug_priv.h"
 
+#define PATH_SEPARATOR '/'
+
 #  if defined(OSCAP_THREAD_SAFE)
 #   include <pthread.h>
 static pthread_mutex_t __debuglog_mutex = PTHREAD_MUTEX_INITIALIZER;
 #  endif
 static FILE *__debuglog_fp = NULL;
-int __debuglog_level = -1;
+int __debuglog_level  = -1;
+static int __debuglog_pstrip = -1;
 
 #if defined(OSCAP_THREAD_SAFE)
 # define __LOCK_FP    do { if (pthread_mutex_lock   (&__debuglog_mutex) != 0) abort(); } while(0)
@@ -52,15 +56,44 @@ static void __oscap_debuglog_close(void)
         fclose(__debuglog_fp);
 }
 
-void __oscap_dprintf(const char *file, const char *fn, size_t line, const char *fmt, ...)
+static const char *__oscap_path_rstrip(const char *path, int num)
 {
-	va_list ap;
+	register size_t len;
+	register char  *res;
+
+	len = strlen(path);
+	res = (char *)path;
+
+	for (len = strlen(path); len > 0; --len) {
+		if (path[len - 1] == PATH_SEPARATOR)
+			--num;
+		if (num == 0)
+			return (path + len);
+	}
+
+	return (path);
+}
+
+static void __oscap_vdlprintf(int level, const char *file, const char *fn, size_t line, const char *fmt, va_list ap)
+{
+	char  l;
+	const char *f;
 
 	__LOCK_FP;
 
-        if (__debuglog_level == -1)
-                __debuglog_level = atoi (getenv (OSCAP_DEBUG_LEVEL_ENV) == NULL ? "0" : getenv (OSCAP_DEBUG_LEVEL_ENV));
-        
+	if (__debuglog_level == -1) {
+		char *env;
+
+		env = getenv(OSCAP_DEBUG_LEVEL_ENV);
+		if (env == NULL)
+			__debuglog_level = DBG_I;
+		else
+			__debuglog_level = atoi(env);
+	}
+	if (__debuglog_level < level) {
+		__UNLOCK_FP;
+		return;
+	}
 	if (__debuglog_fp == NULL) {
 		char *logfile;
 		char *st;
@@ -86,21 +119,49 @@ void __oscap_dprintf(const char *file, const char *fn, size_t line, const char *
 		fprintf(__debuglog_fp, "\n=============== LOG: %.24s ===============\n", st);
                 atexit(&__oscap_debuglog_close);
 	}
+	if (__debuglog_pstrip == -1) {
+		char *pstrip;
+
+		pstrip = getenv(OSCAP_DEBUG_PATHSTRIP_ENV);
+
+		if (pstrip == NULL)
+			__debuglog_pstrip = 0;
+		else
+			__debuglog_pstrip = atol(pstrip);
+
+	}
+	if (__debuglog_pstrip != 0)
+		f = __oscap_path_rstrip(file, __debuglog_pstrip);
+	else
+		f = file;
 
 	if (flock(fileno(__debuglog_fp), LOCK_EX) == -1) {
 		__UNLOCK_FP;
 		return;
 	}
+
+	switch (level) {
+	case DBG_E:
+		l = 'E';
+		break;
+	case DBG_W:
+		l = 'W';
+		break;
+	case DBG_I:
+		l = 'I';
+		break;
+	default:
+		l = '0';
+	}
 #if defined(OSCAP_THREAD_SAFE)
 	/* XXX: non-portable usage of pthread_t */
-	fprintf(__debuglog_fp, "(%u:%u) [%s: %zu: %s] ", (unsigned int)getpid(), (unsigned int)pthread_self(), file,
-		line, fn);
+	fprintf(__debuglog_fp, "(%u:%llx) [%c:%s:%zu:%s] ", (unsigned int) getpid(),
+		(unsigned long long) pthread_self(), l, f, line, fn);
 #else
-	fprintf(__debuglog_fp, "(%u) [%s: %zu: %s] ", (unsigned int)getpid(), file, line, fn);
+	fprintf(__debuglog_fp, "(%u) [%c:%s:%zu:%s] ", (unsigned int) getpid(),
+		l, f, line, fn);
 #endif
-	va_start(ap, fmt);
 	vfprintf(__debuglog_fp, fmt, ap);
-	va_end(ap);
 
 	if (flock(fileno(__debuglog_fp), LOCK_UN) == -1) {
 		/* __UNLOCK_FP; */
@@ -111,4 +172,22 @@ void __oscap_dprintf(const char *file, const char *fn, size_t line, const char *
 	return;
 }
 
+void __oscap_dprintf(const char *file, const char *fn, size_t line, const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	__oscap_vdlprintf(0, file, fn, line, fmt, ap);
+	va_end(ap);
+}
+
+void __oscap_dlprintf(int level, const char *file, const char *fn, size_t line, const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	__oscap_vdlprintf(level, file, fn, line, fmt, ap);
+	va_end(ap);
+}
+
 #endif
