@@ -77,6 +77,7 @@ typedef uint16_t oscap_errcode_t;
  * @{
  */
 #define OVAL_EOVALINT        1
+#define OVAL_EPROBEINVAL     254
 #define OVAL_EPROBECONTEXT   255
 #define OVAL_EPROBEINIT      256
 #define OVAL_EPROBECONN      257
@@ -87,6 +88,8 @@ typedef uint16_t oscap_errcode_t;
 #define OVAL_EPROBECLOSE     262
 #define OVAL_EPROBESEND      263
 #define OVAL_EPROBERECV      264
+#define OVAL_EPROBEOBJTRANS  265
+#define OVAL_EPROBEITEMTRANS 266
 #define OVAL_EPROBEUNKNOWN   511
 /** @} */
 
@@ -99,6 +102,7 @@ typedef uint16_t oscap_errcode_t;
 #define XCCDF_EUNKNOWNTYPE    3   /**< Bad type of xccdf item */  
 #define XCCDF_EUNKNOWNCB      4   /**< Unknown callback - missing registration */
 #define XCCDF_EBADID          5   /**< Bad id in reference - item with that id does not exist */
+#define XCCDF_EVALUE          6   /**< Can't find referenced value instance */
 /** @} */
 
 
