@@ -35,8 +35,10 @@
 #include <wchar.h>
 
 #include "text.h"
+#include "reference.h"
 #include "reporter.h"
 
+
 /**
  * @addtogroup ITER
  * @{
@@ -57,8 +59,8 @@
 /**
  * Iterate over an array, given an iterator.
  * Execute @a code for each array member stored in @a val.
- * It is NOT safe to use return or goto inside of the @a code
- * or iterator will not be freed properly.
+ * It is NOT safe to use return or goto inside of the @a code,
+ * the iterator would not be freed properly.
  */
 #define OSCAP_FOREACH_GENERIC(itype, vtype, val, init_val, code) \
     {                                                            \
@@ -128,121 +130,17 @@
 
 /** @} */
 
-/**
- * @defgroup XMLMETA Element metadata
- * @{
- * Describes XML elemenent metadata such as namespace or language.
- */
 
 /**
- * @struct xml_metadata
- * XML Metadata.
+ * Initialize OpenSCAP library.
  *
- * Here should be every general attribute that can be present
- * in every xml element such as xml:lang or xml namespace.
+ * This is currently needed only in multithreaded applications
+ * (needs to be called before any child threads are spawned)
+ * or applications using the XSLT facility w/ EXSLT extensions.
+ * However, it is a good practice to call this function
+ * always at the beginning of the program execution.
  */
-struct xml_metadata;
-
-/** 
- * @struct xml_metadata_iterator
- * Iterator over XML metadata.
- * @see oscap_iterator
- */
-struct xml_metadata_iterator;
-
-/** 
- * Constructor of XML metadata
- * @memberof xml_metadata
- */
-struct xml_metadata *xml_metadata_new(void);
-
-/************************************************************/
-/**
- * @name Getters
- * Return value is pointer to structure's member. Do not free unless you null the pointer in the structure. 
- * Use remove function otherwise.
- * @{
- * */
-
-/** 
- * Get namespace attribute from XML metadata
- * @memberof xml_metadata
- */
-const char *xml_metadata_get_nspace(const struct xml_metadata *xml);
-
-/** 
- * Get xml:lang attribute from XML metadata
- * @memberof xml_metadata
- */
-const char *xml_metadata_get_lang(const struct xml_metadata *xml);
-
-/** 
- * Get URI attribute from XML metadata
- * @memberof xml_metadata
- */
-const char *xml_metadata_get_URI(const struct xml_metadata *xml);
-
-/************************************************************/
-/** @} End of Getters group */
-
-/************************************************************/
-/**
- * @name Setters
- * For lists use add functions. Parameters of set functions are duplicated in memory and need to 
- * be freed by caller.
- * @{
- */
-
-/** 
- * Set namespace attribute of XML metadata
- * @memberof xml_metadata
- */
-bool xml_metadata_set_nspace(struct xml_metadata *xml, const char *new_namespace);
-
-/** 
- * Set xml:lang attribute of XML metadata
- * @memberof xml_metadata
- */
-bool xml_metadata_set_lang(struct xml_metadata *xml, const char *new_lang);
-
-/** 
- * Set URI attribute of XML metadata
- * @memberof xml_metadata
- */
-bool xml_metadata_set_URI(struct xml_metadata *xml, const char *new_uri);
-
-/************************************************************/
-/** @} End of Setters group */
-
-/** 
- * Destructor of XML metadata structure
- * @memberof xml_metadata
- */
-void xml_metadata_free(struct xml_metadata *xml);
-
-/************************************************************/
-/**
- * @name Iterators
- * @{
- * */
-
-
-/// @memberof xml_metadata_iterator
-struct xml_metadata *xml_metadata_iterator_next(struct xml_metadata_iterator *it);
-
-/// @memberof xml_metadata_iterator
-bool xml_metadata_iterator_has_more(struct xml_metadata_iterator *it);
-
-/// @memberof xml_metadata_iterator
-void xml_metadata_iterator_free(struct xml_metadata_iterator *it);
-
-/// @memberof xml_metadata_iterator
-void xml_metadata_iterator_remove(struct xml_metadata_iterator *it);
-
-
-/************************************************************/
-/** @} End of Evaluators group */
-/** @} */
+void oscap_init(void);
 
 /**
  * Release library internal caches.
@@ -253,126 +151,63 @@ void xml_metadata_iterator_remove(struct xml_metadata_iterator *it);
  */
 void oscap_cleanup(void);
 
+/// Get version of the OpenSCAP library
+const char *oscap_get_version(void);
+
 
 /**
- * @addtogroup IMPORTEXPORT
+ * @addtogroup VALID
  * @{
- * Handling input and output files plus validation according to XML schema.
+ * XML schema based validation of XML representations of SCAP documents.
  */
 
+/// SCAP document type identifiers
+typedef enum oscap_document_type {
+	OSCAP_DOCUMENT_OVAL_DEFINITIONS = 1,  ///< OVAL Definitions file
+	OSCAP_DOCUMENT_OVAL_SYSCHAR,          ///< OVAL system characteristics file
+	OSCAP_DOCUMENT_OVAL_RESULTS,          ///< OVAL results file
+	OSCAP_DOCUMENT_XCCDF,                 ///< XCCDF benchmark file
+	OSCAP_DOCUMENT_CPE_LANGUAGE,          ///< CPE language file
+	OSCAP_DOCUMENT_CPE_DICTIONARY,        ///< CPE dictionary file
+} oscap_document_type_t;
+
+
 /**
- * @struct oscap_nsinfo
- * Information on namespaces for given document.
+ * Validate a SCAP document file against a XML schema.
+ *
+ * Schemas are searched relative to path specified by the OSCAP_SCHEMA_PATH environment variable,
+ * which contains a list of colon-separated paths.
+ * If the variable does not exist a default path is used (usually something like $PREFIX/share/openscap/schemas).
+ *
+ * Directory structure must adhere $SCHEMA_PATH/$STANDARD/$VERSION/$SCHEMAFILE.xsd structure, where $STANDARD
+ * is oval, xccdf, etc., and $VERSION is a version of the standard.
+ *
+ * @param xmlfile File to be validated.
+ * @param doctype Document type represented by the file.
+ * @param version Version of the document, use NULL for library's default.
+ * @param reporetr A reporter to by notified of encountered issues. Can be NULL, if a binary document validates / does not validate answer is satisfactonary.
+ * @param arg Argument for the reporter.
+ * @return Success or failure.
  */
-struct oscap_nsinfo;
+bool oscap_validate_document(const char *xmlfile, oscap_document_type_t doctype, const char *version, oscap_reporter reporter, void *arg);
+
 /**
- * @struct oscap_nsinfo_entry
- * Namespace information entry.
- * This structure carries namespace prefix,
- * Namespace URI and schema location.
+ * Apply a XSLT stylesheet to a XML file.
+ *
+ * Stylesheets are searched relative to path specified by the OSCAP_XSLT_PATH environment variable,
+ * which contains a list of colon-separated paths.
+ * If the variable does not exist a default path is used (usually something like $PREFIX/share/openscap/schemas).
+ *
+ * @param xmlfile File to be transformed.
+ * @param xsltfile XSLT filename
+ * @param outfile Result file shall be written here (NULL for stdout).
+ * @param params list of key-value pairs to pass to the stylesheet.
+ * @return Success or failure.
  */
-struct oscap_nsinfo_entry;
-/**
- * @struct oscap_nsinfo_entry_iterator
- * Namespace info iterator
- * @see oscap_nsinfo
- */
-struct oscap_nsinfo_entry_iterator;
-
-/// @memberof oscap_nsinfo
-struct oscap_nsinfo *oscap_nsinfo_new(void);
-/// @memberof oscap_nsinfo
-struct oscap_nsinfo *oscap_nsinfo_new_file(const char *fname);
-/// @memberof oscap_nsinfo
-void oscap_nsinfo_free(struct oscap_nsinfo *info);
+bool oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params);
 
 /************************************************************/
-/**
- * @name Getters
- * If return value is a pointer to structure's member, do not free it unless you null the pointer in the structure.
- * @{
- * */
-/// @memberof oscap_nsinfo
-struct oscap_nsinfo_entry_iterator *oscap_nsinfo_get_entries(const struct oscap_nsinfo *item);
-/// @memberof oscap_nsinfo
-struct oscap_nsinfo_entry *oscap_nsinfo_get_root_entry(const struct oscap_nsinfo *item);
-/// @memberof oscap_nsinfo
-struct oscap_nsinfo_entry *oscap_nsinfo_get_entry_by_ns(struct oscap_nsinfo *info, const char *ns);
-/************************************************************/
-/** @} End of Getters group */
-
-/**
- * @name Setters
- * For lists use add functions. Parameters of set functions are duplicated in memory and need to 
- * be freed by caller.
- * @{
- */
-/// @memberof oscap_nsinfo
-bool oscap_nsinfo_add_entry(struct oscap_nsinfo *obj, struct oscap_nsinfo_entry *item);
-/// @memberof oscap_nsinfo
-bool oscap_nsinfo_set_root_entry(struct oscap_nsinfo *obj, struct oscap_nsinfo_entry *newval);
-/************************************************************/
-/** @} End of Setters group */
-
-
-/// @memberof oscap_nsinfo_entry
-struct oscap_nsinfo_entry *oscap_nsinfo_entry_new(void);
-/// @memberof oscap_nsinfo_entry
-struct oscap_nsinfo_entry *oscap_nsinfo_entry_new_fill(const char *nsprefix, const char *nsname);
-/// @memberof oscap_nsinfo_entry
-void oscap_nsinfo_entry_free(struct oscap_nsinfo_entry *entry);
-/************************************************************/
-/**
- * @name Getters
- * If return value is a pointer to structure's member, do not free it unless you null the pointer in the structure.
- * @{
- * */
-/// @memberof oscap_nsinfo_entry
-const char *oscap_nsinfo_entry_get_nsname(const struct oscap_nsinfo_entry *item);
-/// @memberof oscap_nsinfo_entry
-const char *oscap_nsinfo_entry_get_nsprefix(const struct oscap_nsinfo_entry *item);
-/// @memberof oscap_nsinfo_entry
-const char *oscap_nsinfo_entry_get_schema_location(const struct oscap_nsinfo_entry *item);
-/************************************************************/
-/** @} End of Getters group */
-
-/**
- * @name Setters
- * For lists use add functions. Parameters of set functions are duplicated in memory and need to 
- * be freed by caller.
- * @{
- */
-/// @memberof oscap_nsinfo_entry
-bool oscap_nsinfo_entry_set_nsname(struct oscap_nsinfo_entry *obj, const char *newval);
-/// @memberof oscap_nsinfo_entry
-bool oscap_nsinfo_entry_set_nsprefix(struct oscap_nsinfo_entry *obj, const char *newval);
-/// @memberof oscap_nsinfo_entry
-bool oscap_nsinfo_entry_set_schema_location(struct oscap_nsinfo_entry *obj, const char *newval);
-/************************************************************/
-/** @} End of Setters group */
-
-/************************************************************/
-/**
- * @name Iterators
- * @{
- */
-/// @memberof oscap_nsinfo_entry_iterator
-bool oscap_nsinfo_entry_iterator_has_more(struct oscap_nsinfo_entry_iterator *it);
-/// @memberof oscap_nsinfo_entry_iterator
-struct oscap_nsinfo_entry *oscap_nsinfo_entry_iterator_next(struct oscap_nsinfo_entry_iterator *it);
-/// @memberof oscap_nsinfo_entry_iterator
-void oscap_nsinfo_entry_iterator_free(struct oscap_nsinfo_entry_iterator *it);
-/************************************************************/
-/** @} End of Iterators group */
-
-/**
- * @name Evaluators
- * @{
- */
-/// Validate a xml file against given xml schema
-bool oscap_validate_xml(const char *xmlfile, const char *schemafile, struct oscap_reporter* reporter);
-/************************************************************/
-/** @} End of Evaluators group */
+/** @} validation group end */
 
 /** @} */
 
