@@ -85,7 +85,7 @@ struct oscap_text * oscap_text_clone(const struct oscap_text * text)
 
 struct oscap_text *oscap_text_new_html(void)
 {
-    assert(false); // TODO implement
+    //assert(false); // TODO implement
     return oscap_text_new_full(OSCAP_TEXT_TRAITS_HTML, NULL, NULL);
 }
 
@@ -113,8 +113,6 @@ struct oscap_text *oscap_text_new_parse(struct oscap_text_traits traits, xmlText
 		text->text = oscap_get_xml(reader);
     else text->text = oscap_element_string_copy(reader);
 
-    // TODO substitution support
-    
     return text;
 }
 
@@ -126,7 +124,7 @@ xmlNode *oscap_text_to_dom(struct oscap_text *text, xmlNode *parent, const char
 
 	if (text->traits.html || text->traits.can_substitute)
 		text_node = oscap_xmlstr_to_dom(parent, elname, text->text);
-	else text_node = xmlNewChild(parent, NULL, BAD_CAST elname, BAD_CAST text->text);
+	else text_node = xmlNewTextChild(parent, NULL, BAD_CAST elname, BAD_CAST text->text);
 
 	if (text_node == NULL) return NULL;
 
@@ -139,3 +137,55 @@ xmlNode *oscap_text_to_dom(struct oscap_text *text, xmlNode *parent, const char
 	return text_node;
 }
 
+bool oscap_text_export(struct oscap_text *text, xmlTextWriter *writer, const char *elname)
+{
+	if (text == NULL || writer == NULL) return false;
+
+	if (elname) xmlTextWriterStartElement(writer, BAD_CAST elname);
+
+	if (text->lang)
+		xmlTextWriterWriteAttribute(writer, BAD_CAST "xml:lang", BAD_CAST text->lang);
+	if (text->traits.can_override && text->traits.override_given)
+		xmlTextWriterWriteAttribute(writer, BAD_CAST "override", BAD_CAST (text->traits.overrides ? "true" : "false"));
+
+	if (text->traits.html || text->traits.can_substitute)
+		xmlTextWriterWriteRaw(writer, BAD_CAST text->text);
+	else xmlTextWriterWriteString(writer, BAD_CAST text->text);
+
+	if (elname) xmlTextWriterEndElement(writer);
+
+	return true;
+}
+
+char *oscap_text_get_plaintext(const struct oscap_text *text)
+{
+    if (text == NULL) return NULL;
+
+    if (!text->traits.html) return oscap_strdup(text->text);
+
+    char *out = NULL;
+    char *str = oscap_sprintf("<x xmlns='http://www.w3.org/1999/xhtml'>%s</x>", text->text);
+    xmlDoc *doc = xmlParseMemory(str, strlen(str));
+    if (doc == NULL) goto cleanup;
+    xmlNode *root = xmlDocGetRootElement(doc);
+    if (root == NULL) goto cleanup;
+
+    // TODO: better HTML -> plaintext conversion
+    out = (char*) xmlNodeGetContent(root);
+    xmlFreeDoc(doc);
+
+cleanup:
+    oscap_free(str);
+    return out;
+}
+
+bool oscap_textlist_export(struct oscap_text_iterator *texts, xmlTextWriter *writer, const char *elname)
+{
+	if (texts == NULL || writer == NULL || elname == NULL) return false;
+
+	OSCAP_FOR(oscap_text, text, texts)
+		oscap_text_export(text, writer, elname);
+	
+	return true;
+}
+
